@@ -2,7 +2,7 @@
 #  -*- mode: python; indent-tabs-mode: nil; -*-
 
 """
-ImageParser.py   $Id: ImageParser.py,v 1.58 2003/09/13 01:23:53 chrish Exp $
+ImageParser.py   $Id: ImageParser.py,v 1.60 2004/03/23 21:17:38 chrish Exp $
 
 This defines various classes to parse an image to a PluckerImageDocument.
 
@@ -179,7 +179,7 @@ class ImageParser:
         which uses the "alt_maxwidth" and "alt_maxheight" parameters, and which is linked
         to the the smaller version.  There may be other reasons to add alternate versions
         of an image, too.  This method generates a list of (URL, ATTRIBUTES) pairs, each
-        describing a desired alternate version of the current image, and rturns them."""
+        describing a desired alternate version of the current image, and returns them."""
 
         new_sizes = []
 
@@ -187,14 +187,16 @@ class ImageParser:
             # this is an in-line image.  Let's create the right attributes for a larger
             # version.
             if self._config.get_string('alt_maxwidth') or self._config.get_string('alt_maxheight'):
-                new_attributes = self._attribs.copy()
-                del new_attributes['_plucker_from_image']
-                new_attributes['_plucker_alternate_image'] = 1
-                if self._config.get_string('alt_maxwidth', None):
-                    new_attributes['maxwidth'] = self._config.get_string('alt_maxwidth', None)
-                if self._config.get_string('alt_maxheight', None):
-                    new_attributes['maxheight'] = self._config.get_string('alt_maxheight', None)
-                new_sizes.append((self._url, new_attributes))
+                # We can turn off alt images by maxing either one or the other to a string of "-1".
+                if self._config.get_string('alt_maxwidth') != "-1" and self._config.get_string('alt_maxheight') != "-1":
+                    new_attributes = self._attribs.copy()
+                    del new_attributes['_plucker_from_image']
+                    new_attributes['_plucker_alternate_image'] = 1
+                    if self._config.get_string('alt_maxwidth', None):
+                        new_attributes['maxwidth'] = self._config.get_string('alt_maxwidth', None)
+                    if self._config.get_string('alt_maxheight', None):
+                        new_attributes['maxheight'] = self._config.get_string('alt_maxheight', None)
+                    new_sizes.append((self._url, new_attributes))
 
         return new_sizes
 
@@ -243,10 +245,13 @@ class ImageParser:
                     width = int(scaling_factor * width)
                     height = int(scaling_factor * height)
                     newbits = self.convert(width, height, depth, section)
+            else:
+                message(2, "You aren't using a try_reduce_depth=1 nor a "
+                        "try_reduce_dimension=1. Will proceed to multiimage..")
 
         if len(newbits) == 0:
             # Oops, nothing fetched?!?
-            raise ImageSize("Converted image size for %s is zero bytes" % self._url)
+            raise ImageSize("Converted image size for %s is zero bytes. Nothing fetched?" % self._url)
 
         elif len(newbits) > SimpleImageMaxSize:
             # image bits too large for a _SINGLE_ Plucker image record
@@ -320,6 +325,11 @@ class ImageParser:
             X = 0
             Y = Y + high
 
+        # check for alternative versions of this image
+        # First, figure out if the image has been scaled down at all
+        full_width, full_height = self.size()
+        versions = self._related_images(width < full_width or height < full_height)
+        map(lambda x, doc=doc: doc.add_related_image(x[0], x[1]), versions)
         return doc
 
 
