@@ -2,7 +2,7 @@
 #  -*- mode: python; indent-tabs-mode: nil; -*-
 
 """
-Spider.py   $Id: Spider.py,v 1.87 2003/08/17 20:33:54 chrish Exp $
+Spider.py   $Id: Spider.py,v 1.96 2004/04/10 23:06:39 chrish Exp $
 
 Recursively gets documents and thus collects a document set.
 
@@ -524,7 +524,9 @@ class Spider:
         # collection of documents
         self._home_url = URL(config.get_string ('home_url', 'plucker:/home.html'))
         bpp = config.get_int ('bpp', 1)
-        self._doing_big_imgs = config.get_string('alt_maxwidth', None) or config.get_string('alt_maxheight', None)
+        # ROB (Oct 26,2003) Commenting out this line. doing_big_imgs is never used. We check again in ImageParser.py
+        # _related_images anyway, together with the rest of image parsing.
+        #self._doing_big_imgs = config.get_string('alt_maxwidth', None) or config.get_string('alt_maxheight', None)
         attributes = {'maxdepth': "%d" % config.get_int ('home_maxdepth', 2),
                       'bpp': "%d" % bpp}
         if config.get_bool ('home_stayonhost', 0):
@@ -673,6 +675,8 @@ class Spider:
         #        f.write ("  %s  %s\n" % (repr(key), str(attr)))
         #    f.close ()
 
+        import tempfile
+
         if self._queue:
             if estimate:
                 message ("---- %d expected, %d collected, %d to do ----\n", estimate, len (self._collected), len (self._queue))
@@ -704,7 +708,7 @@ class Spider:
             urltext = url.as_string (with_fragment=0)
             # if there are parameters, we add them back on
             if post_data is not None:
-                urltext_key = urltext_key + post_data
+                urltext_key = urltext + post_data
             else:
                 urltext_key = urltext
             # OK, now that we know the URL used as the mapping key, check it
@@ -769,7 +773,7 @@ class Spider:
                        new_url = URL (new_url).as_string (with_fragment=0)
                 # again, we form the mapping key to see if it's already been processed
                 if post_data is not None:
-                    new_url_key = new_url_key + post_data
+                    new_url_key = new_url + post_data
                 else:
                     new_url_key = new_url
                 # now check to see if the new URL is different from the original,
@@ -794,6 +798,25 @@ class Spider:
                         message("  Is excluded.")
                         return
 
+                # Check for a filter to run document through
+                if header['content-type'][:5] == "text/":
+                    filter = self._config.get_string ('filter')
+                    if filter is not None:
+                        try:
+                            tmpfile = tempfile.mktemp()
+                            f = open(tmpfile, "wb")
+                            f.write(document)
+                            f.close()
+
+                            command = filter + " " + tmpfile
+                            pipe = os.popen(command)
+                            document = pipe.read()
+                            pipe.close()
+
+                        finally:
+                            try: os.unlink(tmpfile) 
+                            except: pass
+
                 # OK, it's fair game, so we parse it
                 try:
                     pluckerdoc = self._parser (new_url,
@@ -881,7 +904,7 @@ class Spider:
                 # For image documents, we want to create alternate renditions
                 # of the image, if called for
 
-                elif pluckerdoc.is_image_document():
+                elif pluckerdoc.is_image_document() or pluckerdoc.is_multiimage_document():
 
                     alternate_count = 0
                     others = pluckerdoc.get_related_images()
@@ -955,7 +978,7 @@ def main (config, excl_lists=[]):
     import PyPlucker.PluckerDocs
 
     PyPlucker.PluckerDocs.PluckerTextDocument.seamless_fragments = config.get_bool ('seamless_fragments',1)
-    PyPlucker.PluckerDocs.PluckerTextDocument.link_fragments = config.get_bool ('link_fragments',1)
+    PyPlucker.PluckerDocs.PluckerTextDocument.link_fragments = config.get_bool ('link_fragments',0)
 
     pluckerdir = config.get_string ('pluckerdir')
     assert pluckerdir is not None
@@ -1238,6 +1261,11 @@ def realmain (outputstream, argv=None):
         message(0, "                   Set alternative maximum height of images to <n> pixels.  This value")
         message(0, "                   is used for 'big' versions of inline images that had to be scaled")
         message(0, "                   down in size to obey the MAXWIDTH and MAXHEIGHT parameters.")
+        message(0, "                   Special values that can be used are 0 and -1, which do the following:")
+        message(0, "                   Set --alt-maxheight=0 and --alt-maxheight=0 to link to original sized")
+        message(0, "                   'big' versions.")
+        message(0, "                   Set --alt-maxheight=-1 and --alt-maxheight=-1 to not link to any")
+        message(0, "                   'big' versions at all.")
         message(0, "    --alt-maxwidth=<n>:")
         message(0, "                   Set alternative maximum width of images to <n> pixels.  This value")
         message(0, "                   is used for 'big' versions of inline images that had to be scaled")
@@ -1284,6 +1312,19 @@ def realmain (outputstream, argv=None):
         message(0, "                     link:     insert links to previous/next fragment")
         message(0, "                     both:     insert links and support seamless connection")
         message(0, "                   Defaults to seamless.")
+        message(0, "    --creator-id=<CrId>")
+        message(0, "                   Sets 4 character Palm creator ID of output database. Example is Plkr") 
+        message(0, "                   Plucker viewer loads documents with creator ID of Plkr into")     
+        message(0, "                     the document library.") 
+        message(0, "                   Defaults to Plkr")   
+        message(0, "    --filter=<filter name>:")
+        message(0, "                   Pass fetched documents through filter prior to parsing.")
+        message(0, "    --bookmarks=<bookmark type>:")
+        message(0, "                   Set bookmarks added to database to <bookmark type>.")
+        message(0, "                   pages:    add a bookmark for each text/* page collected.")
+        message(0, "                   links:    add a bookmark for each bookmark type link tag.")
+        message(0, "                   both:     add both kinds of bookmarks.")
+        message(0, "                   Defaults to None.")   
         message(0, "")
         message(0, "Note that you must specify either -f or specify HOMEURL as an argument,")
         message(0, " or specify -c to update a cache.")
@@ -1325,24 +1366,37 @@ def realmain (outputstream, argv=None):
         statusfile = None
         depthfirst = None
         tables = None
+        filter = None
+        bookmarks = None
+        bookmark_pages = None
         http_proxy = None
         http_proxy_user = None
         http_proxy_pass = None
+        creator_id = None
 
         (opts, args) = getopt.getopt(argv[1:], "f:chqvV:p:P:H:E:M:N:s:", \
-                                     [  "db-file=", "doc-file=", "help", "quiet", "pluckerdir=", "pluckerhome=",
+                                     [  "db-file=", "doc-file=", "help",
+                                        "quiet", "pluckerdir=", "pluckerhome=",
                                         "bpp=", "noimages", "exclusion-list=",
                                         "maxdepth=", "db-name=", "doc-name=",
                                         "extra-section=", "verbosity=",
                                         "zlib-compression", "doc-compression",
-                                        "no-urlinfo", "stayondomain", "stayonhost", "staybelow=", "category=",
-                                        "maxheight=", "maxwidth=", "alt-maxheight=", "alt-maxwidth=",
-                                        "compression=", "home-url=", "update-cache", "launchable",
-                                        "not-launchable", "backup", "no-backup", "beamable", "not-beamable",
-                                        "icon=", "charset=", "owner-id=", "url-pattern=", "referrer=",
-                                        "user-agent=", "title=", "author=", "status-file=", "version",
-                                        "tables", "depth-first", "http-proxy=", "http-proxy-user=", "http-proxy-pass=",
-                                        "fragments="])
+                                        "no-urlinfo", "stayondomain",
+                                        "stayonhost", "staybelow=", "category=",
+                                        "maxheight=", "maxwidth=",
+                                        "alt-maxheight=", "alt-maxwidth=",
+                                        "compression=", "home-url=",
+                                        "update-cache", "launchable",
+                                        "not-launchable", "backup",
+                                        "no-backup", "beamable", "not-beamable",
+                                        "icon=", "charset=", "owner-id=",
+                                        "url-pattern=", "referrer=",
+                                        "user-agent=", "title=", "author=",
+                                        "status-file=", "version",
+                                        "tables", "depth-first", "http-proxy=",
+                                        "http-proxy-user=", "http-proxy-pass=",
+                                        "fragments=", "creator-id=",
+                                        "bookmarks="])
         if args:
             # usage ("Only options are allowed as arguments.")
             if len(args) > 1:
@@ -1481,6 +1535,26 @@ def realmain (outputstream, argv=None):
                 http_proxy_user = arg
             elif opt == "--http-proxy-pass":
                 http_proxy_pass = arg
+            elif opt == "--creator-id":
+                creator_id = arg
+                if len(creator_id) > 4:
+                    message(0, "Only the first 4 characters \"%s\" of the creator ID will be used.", arg[:4])
+                    creator_id = arg[:4]    
+            elif opt == "--filter":
+                filter = arg
+            elif opt == "--bookmarks":
+                if arg == "both":
+                    bookmarks        = 'true'
+                    bookmark_pages   = 'true'
+                elif arg == "links":
+                    bookmarks        = 'true'
+                    bookmark_pages   = 'false'
+                elif arg == "pages":
+                    bookmarks        = 'false'
+                    bookmark_pages   = 'true'
+                else:
+                    bookmarks        = 'false'
+                    bookmark_pages   = 'false'
             else:
                 usage ("Error:  Unknown option '%s'" % opt)
     except getopt.error, text:
@@ -1660,6 +1734,16 @@ def realmain (outputstream, argv=None):
         config.set ('depth_first', 1)
     if tables is not None:
         config.set ('tables', 1)
+    if creator_id is None:
+        creator_id = config.get_string ( 'creator_id', 'Plkr' )
+    if creator_id is not None:
+        config.set ('creator_id', creator_id)
+    if filter is not None:
+        config.set ('filter', filter)
+    if bookmarks is not None:
+        config.set ('bookmarks', bookmarks)
+    if bookmark_pages is not None:
+        config.set ('bookmark_pages', bookmark_pages)
 
     for i in range (len (exclusion_lists)):
         exclusion_lists[i] = os.path.join (pluckerdir, exclusion_lists[i])
