@@ -1,9 +1,9 @@
 #!/usr/bin/env python
-#  -*- mode: python; indent-tabs-mode: nil; -*-
+#  -*- mode: python; indent-tabs-mode: nil; -*- coding: iso-8859-1 -*-
 
 """
 
-TextParser.py   $Id: TextParser.py,v 1.53 2003/08/03 14:07:02 nordstrom Exp $
+TextParser.py   $Id: TextParser.py,v 1.66 2004/04/13 03:37:16 jimj Exp $
 
 
 Copyright 1999,2000 by Holger Duerer <holly@starship.python.net>
@@ -41,10 +41,22 @@ except ImportError:
     import sgmllib
     # Fix for the missing comma in raw attributes
     import re, string
-    sgmllib.attrfind = re.compile(
-        '[%s]*([a-zA-Z_][-.a-zA-Z_0-9]*)' % string.whitespace
-        + ('([%s]*=[%s]*' % (string.whitespace, string.whitespace))
-        + r'(\'[^\']*\'|"[^"]*"|[-a-zA-Z0-9,@./:+*%?!\(\)_#=~]*))?')
+    
+    #
+    #  Originally, plucker overrode this to catch the "@" of 
+    #  mailto links.  This bugfix is already in newer pythons.
+    #  If you have a new enough version (the sgmllib distributed
+    #  with Python 2.3.3), then we should use the (newer) python
+    #  version.  This checks whether or not attrfind knows that 
+    #  @ can appear in unquoted attribute values. If so, use the
+    #  python version.
+    #
+    temp=sgmllib.attrfind.match('href=mailto:user@host').groups()[2]
+    if temp is None or '@' not in temp:
+        sgmllib.attrfind = re.compile(
+            r'\s*([a-zA-Z_][-:.a-zA-Z_0-9]*)(\s*=\s*'
+            r'(\'[^\']*\'|"[^"]*"|[-a-zA-Z0-9./,:;+*%?!&$\(\)_#=~\'"@]*))?')
+
 
 import sys
 import struct
@@ -70,8 +82,28 @@ _RE_WHITESPACE = re.compile ("[\n\f \t]+")
 _RE_NONSPACEWHITESPACE = re.compile ("[\n\f\t\t]+")
 _CLEANUP_TRANSTABLE = string.maketrans ("\f", "\n")
 
-## the following is the default set of colornames understood by HTML
-Colornames = {
+##
+## There are several colorsets in use on the web.
+## 
+## Plucker has traditionally used the HTML4 color names
+## (Colornames_Strict). 
+## 
+## CSS2.1 <URL: http://www.w3.org/TR/CSS21/syndata.html#color-units> 
+## adds orange ("FFA500") and several "colors" that depend on 
+## current system settings.  It isn't clear what to do with 
+## system colors, except maybe defaulting to black.
+##
+## SVG adds several other colors (from the X windows color 
+## set, plus some grays).
+##
+## CSS3 <URL: http://www.w3.org/TR/2003/CR-css3-color-20030514 > 
+## color names represent the future of standards, but splits
+## color names into several different profiles.  Full support 
+## is more complicated than Plucker currently supports.
+##
+## Some web sites use Crayola colors.
+##
+Colornames_Strict = {
         'black':   "000000",
         'silver':  "C0C0C0",
         'gray':    "808080",
@@ -90,39 +122,207 @@ Colornames = {
         'aqua':    "00FFFF",
         }
 
+Colornames_SVG = {
+        'aliceblue':            'F0F8FF',
+        'antiquewhite':         'FAEBD7',
+        'aqua':                 '00FFFF',
+        'aquamarine':           '7FFFD4',
+        'azure':                'F0FFFF',
+        'beige':                'F5F5DC',
+        'bisque':               'FFE4C4',
+        'black':                '000000',
+        'blanchedalmond':       'FFEBCD',
+        'blue':                 '0000FF',
+        'blueviolet':           '8A2BE2',
+        'brown':                'A52A2A',
+        'burlywood':            'DEB887',
+        'cadetblue':            '5F9EA0',
+        'chartreuse':           '7FFF00',
+        'chocolate':            'D2691E',
+        'coral':                'FF7F50',
+        'cornflowerblue':       '6495ED',
+        'cornsilk':             'FFF8DC',
+        'crimson':              'DC143C',
+        'cyan':                 '00FFFF',
+        'darkblue':             '00008B',
+        'darkcyan':             '008B8B',
+        'darkgoldenrod':        'B8860B',
+        'darkgray':             'A9A9A9',
+        'darkgreen':            '006400',
+        'darkgrey':             'A9A9A9',
+        'darkkhaki':            'BDB76B',
+        'darkmagenta':          '8B008B',
+        'darkolivegreen':       '556B2F',
+        'darkorange':           'FF8C00',
+        'darkorchid':           '9932CC',
+        'darkred':              '8B0000',
+        'darksalmon':           'E9967A',
+        'darkseagreen':         '8FBC8F',
+        'darkslateblue':        '483D8B',
+        'darkslategray':        '2F4F4F',
+        'darkslategrey':        '2F4F4F',
+        'darkturquoise':        '00CED1',
+        'darkviolet':           '9400D3',
+        'deeppink':             'FF1493',
+        'deepskyblue':          '00BFFF',
+        'dimgray':              '696969',
+        'dimgrey':              '696969',
+        'dodgerblue':           '1E90FF',
+        'firebrick':            'B22222',
+        'floralwhite':          'FFFAF0',
+        'forestgreen':          '228B22',
+        'fuchsia':              'FF00FF',
+        'gainsboro':            'DCDCDC',
+        'ghostwhite':           'F8F8FF',
+        'gold':                 'FFD700',
+        'goldenrod':            'DAA520',
+        'gray':                 '808080',
+        'green':                '008000',
+        'greenyellow':          'ADFF2F',
+        'grey':                 '808080',
+        'honeydew':             'F0FFF0',
+        'hotpink':              'FF69B4',
+        'indianred':            'CD5C5C',
+        'indigo':               '4B0082',
+        'ivory':                'FFFFF0',
+        'khaki':                'F0E68C',
+        'lavender':             'E6E6FA',
+        'lavenderblush':        'FFF0F5',
+        'lawngreen':            '7CFC00',
+        'lemonchiffon':         'FFFACD',
+        'lightblue':            'ADD8E6',
+        'lightcoral':           'F08080',
+        'lightcyan':            'E0FFFF',
+        'lightgoldenrodyellow': 'FAFAD2',
+        'lightgray':            'D3D3D3',
+        'lightgreen':           '90EE90',
+        'lightgrey':            'D3D3D3',
+        'lightpink':            'FFB6C1',
+        'lightsalmon':          'FFA07A',
+        'lightseagreen':        '20B2AA',
+        'lightskyblue':         '87CEFA',
+        'lightslategray':       '778899',
+        'lightslategrey':       '778899',
+        'lightsteelblue':       'B0C4DE',
+        'lightyellow':          'FFFFE0',
+        'lime':                 '00FF00',
+        'limegreen':            '32CD32',
+        'linen':                'FAF0E6',
+        'magenta':              'FF00FF',
+        'maroon':               '800000',
+        'mediumaquamarine':     '66CDAA',
+        'mediumblue':           '0000CD',
+        'mediumorchid':         'BA55D3',
+        'mediumpurple':         '9370DB',
+        'mediumseagreen':       '3CB371',
+        'mediumslateblue':      '7B68EE',
+        'mediumspringgreen':    '00FA9A',
+        'mediumturquoise':      '48D1CC',
+        'mediumvioletred':      'C71585',
+        'midnightblue':         '191970',
+        'mintcream':            'F5FFFA',
+        'mistyrose':            'FFE4E1',
+        'moccasin':             'FFE4B5',
+        'navajowhite':          'FFDEAD',
+        'navy':                 '000080',
+        'oldlace':              'FDF5E6',
+        'olive':                '808000',
+        'olivedrab':            '6B8E23',
+        'orange':               'FFA500',
+        'orangered':            'FF4500',
+        'orchid':               'DA70D6',
+        'palegoldenrod':        'EEE8AA',
+        'palegreen':            '98FB98',
+        'paleturquoise':        'AFEEEE',
+        'palevioletred':        'DB7093',
+        'papayawhip':           'FFEFD5',
+        'peachpuff':            'FFDAB9',
+        'peru':                 'CD853F',
+        'pink':                 'FFC0CB',
+        'plum':                 'DDA0DD',
+        'powderblue':           'B0E0E6',
+        'purple':               '800080',
+        'red':                  'FF0000',
+        'rosybrown':            'BC8F8F',
+        'royalblue':            '4169E1',
+        'saddlebrown':          '8B4513',
+        'salmon':               'FA8072',
+        'sandybrown':           'F4A460',
+        'seagreen':             '2E8B57',
+        'seashell':             'FFF5EE',
+        'sienna':               'A0522D',
+        'silver':               'C0C0C0',
+        'skyblue':              '87CEEB',
+        'slateblue':            '6A5ACD',
+        'slategray':            '708090',
+        'slategrey':            '708090',
+        'snow':                 'FFFAFA',
+        'springgreen':          '00FF7F',
+        'steelblue':            '4682B4',
+        'tan':                  'D2B48C',
+        'teal':                 '008080',
+        'thistle':              'D8BFD8',
+        'tomato':               'FF6347',
+        'turquoise':            '40E0D0',
+        'violet':               'EE82EE',
+        'wheat':                'F5DEB3',
+        'white':                'FFFFFF',
+        'whitesmoke':           'F5F5F5',
+        'yellow':               'FFFF00',
+        'yellowgreen':          '9ACD32',
+}
+
+Colornames = Colornames_SVG
 
 def _parse_color (value):
-    # assess whether a hex color (eg #C0C0C0) or a text colorname (eg 'silver')
-    rgb = None
-
-    if value in Colornames.keys():
-        rgb = Colornames[value]
-    elif value == "":
-        rgb = "000000"
-    elif value[0] == '#':
-        try:
-            val = string.atoi(value[1:], 16)
-            rgb = '%06x' % val
-        except:
-            pass
-    # May need to do error checking, at this last bit, if people using non
-    #  standard color text names    
-    else:
-        rgb = "000000"
-        try:
-            # possible hex value without leading hash mark...
-            val = string.atoi(value, 16)
-            rgb = '%06x' % val
-        except:
-            pass
-
-    return rgb
+    """Get the RGB value.  
+    
+    Try text colorname (e.g. 'Silver'), 
+    then try to lower-case that (e.g., 'silver')
+    then try an RGB value (e.g. #C0C0C0),
+    then try an RGB value missing the # (e.g. C0C0C0)
+    then try RGB (with and without the #) with only one char/color.
+    then default to black ("#000000").
+    """
+    
+    try:
+        return Colornames[value]
+    except KeyError:
+        pass    
+    
+    # This is redundant, if the page is coded entirely properly.
+    try:
+        return Colornames[value.lower()]
+    except KeyError:
+        pass    
+    
+    if value[0] == '#':
+        value = value[1:]
+    
+    try:
+        val = '%06x' % string.atoi(value, 16)
+        return val.upper()
+    except ValueError:
+        pass
+    
+    try:
+        val = '%03x' % string.atoi(value, 16)
+        val = val.upper()
+        val = ((val[0]) * 2) + ((val[1]) * 2) + ((val[2]) * 2)
+        return val
+    except ValueError:
+        pass
+    
+    message (1, "Giving up on color %s, using black.", value)
+    return "000000"
 
 
 def _list_to_dict (alist):
-    """Convert an attribute list as generated by the sgml parser
-    class (a list of (key, value) tuples into a dictionary for easier
-    use"""
+    """Convert [(attr1,val), (attr2,val) ...] to {attr1:val, attr2:val}
+    
+    The sgml parser returns attributes as a list of key-value pairs. 
+    This function puts them in a dictionary, for more easier use."""
+    
     assert type(alist) == type([])
     result = {}
     for (key,val) in alist:
@@ -136,6 +336,9 @@ def _list_to_dict (alist):
 _entitycharref = re.compile('^(.*)&([#a-zA-Z][-.a-zA-Z0-9]*);(.*)$')
 _html_char_ref_pattern = re.compile('^&#([0-9]+);$')
 
+# These junk "alt" attribute values are not worth showing.
+junk_alt_attributes = ("img", "[img]", "spacer", "")
+
 def cleanup_attribute (text):
     m = _entitycharref.search (text)
     if not m:
@@ -148,14 +351,20 @@ def cleanup_attribute (text):
             if 0 <= n <= 255:
                 content = chr (n)
             else:
-                content = "?"
+                #self._add_unicode_char(val, "&#%d;" % val)
+                # Not in a "self", so can't add the unicode properly.
+                content="&#%d" % val
         except ValueError:
-            content = "?"
+            #content = "?"
+            # might as well pass it through.  no worse than a "?"
+            content = "&#" + content
     else:
         if htmlentitydefs.entitydefs.has_key (content):
             content = htmlentitydefs.entitydefs[content]
         else:
-            content = "?"
+            # content = "?"
+            # usually unescaped &amp -- AT&T should return AT&T.
+            content = "&" + content
     return cleanup_attribute (pre) + content + post
     
 
@@ -163,7 +372,8 @@ def cleanup_attribute (text):
 def _clean_newlines (text):
     """Try to clean up newlines in source code to be UNIXy.
     We assume that CP/M derived OSes use \r\n as line terminator and
-    MacOS uses only \r.  Both version are converted to use only \n."""
+    MacOS uses only \r.  Both version are converted to use only \n.
+    XXX We should probably use python's universal newlines now."""
     
     text = string.replace (text, "\r\n", "\n")
     text = string.replace (text, "\r", "\n")
@@ -290,8 +500,10 @@ class AttributeStack:
 
         
     def push (self, tag):
-        """Push the style for 'tag' onto the stack.  Return true if this is a
-        change in style."""
+        """Push the style for 'tag' onto the stack.  
+
+        Return True if the style was actually changed by this."""
+        
         tag = string.lower (tag)
         assert self._tags.has_key (tag), "Unknown style code %s" % tag
         self._stack.append (tag)
@@ -638,8 +850,11 @@ class TextDocBuilder:
 
 
     def _find_text_split (self, line, size):
-        """Find a split of line so that one part is approx. size bytes
-        long.  Return (first_part, rest)."""
+        """Split line so that the first part is approx. size bytes long.
+        
+        Return (first_part, rest)."""
+        # XXX Why do we care?  Could we use TextWrapper?
+        
         first = line[:size]
         rest = line[size:]
 
@@ -828,6 +1043,9 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
     def __init__ (self, url, text, headers = {}, config = None, attribs = {}):
         sgmllib.SGMLParser.__init__ (self)
 
+        # Convert all <tag/> to <tag /> for XHTML compatability
+        text = string.replace (text, "/>", " />")
+
         text = _clean_newlines (text)
         # This we use to build the document
         self._doc = TextDocBuilder (url, config, max_paragraph_size=3000)
@@ -845,7 +1063,11 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
         # Figure out paragraph indentation style
         self._indent_paragraphs = config and config.get_bool('indent_paragraphs', 0)
         # Flag whether things found should be output.
-        # (This gets temporarily set to false durign parse of headers.)
+        # This gets temporarily set to false during parse of headers.
+        # It is now also used to hide non-content "tags", such as
+        # style and script.  It is not perfect though; for example,
+        # javascript:document.write("<div>") turns it back on, because
+        # it only recognizes the div, not the javascript.
         self._visible = 1
         self._charset = headers.has_key('charset') and charset_name_to_mibenum(headers['charset'])
         if self._charset:
@@ -884,7 +1106,6 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
             # as unknown items.
 
             'html' : 1,        
-            'link' : 1,
             'span' : 1,
             'code' : 1,
             'kbd' : 1,
@@ -897,7 +1118,7 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
             'blink': 1,        # you must be kidding!
             'form' : 1,
             'area' : 1,
-            'option' : 1,
+            # 'option' : 1,    # now "handled" by ignoring it. Useless until we process forms.
             'input' : 1,
             'map' : 1,
             'noscript' : 1,
@@ -911,7 +1132,7 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
             
             # the following I found in various documents
             # no idea what they mean or should do...
-            'basefont': 1, #specifies the font to use for a page if not otherwise specified. Depreciated
+            'basefont': 1, #Deprecated.  Default (and original for CSS inheritance) font
             'droplink': 1,
             }
 
@@ -925,7 +1146,7 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
         self.in_cell = 0
         self.table_count = 0
         self.last_table_style = 0
-        self.last_table_forecolor = ["000000"]
+        self.last_table_forecolor = "000000"
         self.last_table_italics = 0
         self.last_table_underline = 0
         self.last_table_strike = 0
@@ -1086,22 +1307,31 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
         """Add some text.  This may contain newlines, however use
         _add_vspace() to do that explicitly if you want to."""
         if self._visible:
-            self._doc.add_text (text)
-            self._element_beginning = 0
+            if self.atable is not None and self.in_cell:
+                self.atable.add_cell_text (text)
+            else:
+                self._doc.add_text (text)
+                self._element_beginning = 0
 
 
     def _add_unicode_char (self, char_code, text_alternative):
         """Add a Unicode character, along with a non-Unicode alternative text string."""
         if self._visible:
-            self._doc.add_unicode_char (char_code, text_alternative)
-            self._element_beginning = 0
+            if self.atable is not None and self.in_cell:
+                self.atable.add_cell_text (text_alternative)
+            else:
+                self._doc.add_unicode_char (char_code, text_alternative)
+                self._element_beginning = 0
 
 
     def _add_vspace (self, n):
         """Add a newline to the document"""
         if self._visible:
-            self._doc.add_vspace (n_units=n)
-            self._element_beginning = 0
+            if self.atable is not None and self.in_cell:
+                self.atable.add_cell_text ('\000\070')
+            else:
+                self._doc.add_vspace (n_units=n)
+                self._element_beginning = 0
 
 
     def _push_verbosity (self, level):
@@ -1300,9 +1530,8 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
                         style_str = struct.pack (">BB", 0, 0x78)
                     self.atable.add_cell_text(style_str)
                     self.last_table_strike = new_strike
-                self.atable.add_cell_text (data)
-            else:
-                self._add_text (data)
+
+            self._add_text (data)
 
 
     def start_body (self, attributes):
@@ -1346,6 +1575,8 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
             if attributes.has_key ('alt'):
                 # Clean up cruft
                 attributes['alt'] = _RE_NONSPACEWHITESPACE.sub (' ', attributes['alt'], 0)
+                if attributes['alt'].lower() in junk_alt_attributes:
+                    del attributes['alt']
             if attributes.has_key ('src'):
                 # Set the forecolor, replacing our '#000000' with a color attribute if specified
                 if attributes.has_key ('color'):
@@ -1362,14 +1593,14 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
                     self._doc._paragraph._image_refs.append((attributes['src'], attributes))
                 else:
                     self._doc.add_image (attributes)
-# CRH do this in either case, so the images get spidered
+                # CRH do this in either case, so the images get spidered
                 self._images.append ((attributes['src'], attributes))
 
                 # Unset the forecolor after img is over
                 if rgb:
                     forecolor = self._doc.get_forecolor ()
                     self._doc.unset_forecolor (forecolor) 
-            
+                
                 self._element_beginning = 0
             else:
                 print "<img> without src!  (%s)" % repr (attributes)
@@ -1399,7 +1630,7 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
                     self._push_element ("a")
 
                     if self.atable is not None and self.in_cell:
-                        self.atable.add_link(link = attributes['href'], attr = attributes)
+                        self.atable.add_link(attr = attributes)
 
         if attributes.has_key ("name"):
             self._doc.add_name (attributes['name'])
@@ -1412,6 +1643,8 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
             if self._visible:
                 self._pop_element ("a")
                 self._doc.add_document_link_end ()
+                if self.atable is not None and self.in_cell:
+                    self.atable.end_link()
                 if self._anchor_forecolor:
                     # Stop coloring the anchor
                     forecolor = self._doc.get_forecolor ()
@@ -1424,10 +1657,17 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
             # clean up cruft
             self._base = Url.CleanURL (attributes['href'], self._url)
 
+    #
+    # The "contents" of a style or script tag is not really intended
+    # for human consumption.  Until we use the information, the best
+    # we can do is make it invisible, so that at least people don't
+    # have to read it.  Note that this is not foolproof - visibility
+    # seems to turn back on at the "div" of 
+    #    javascript:document.write("<div>")
+    #
     def start_script (self, attributes):
         self._push_visibility (0)
 
-
     def end_script (self):
         self._pop_visibility ()
 
@@ -1435,10 +1675,28 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
     def start_javascript (self, attributes):
         self._push_visibility (0)
 
-
     def end_javascript (self):
         self._pop_visibility ()
 
+    # Plucker does not yet handle forms, but we need to display
+    # them because so many sites wrap the full content within a
+    # form.  Even a <select> may contain information about what
+    # sort of information will be required.  Listing all the 
+    # possible options just interrupts the reader.  (Gee, this
+    # site can name all 50 states!)
+    def start_option (self, attributes):
+        self._push_visibility (0)
+
+    def end_option (self):
+        self._pop_visibility ()
+
+    def start_style (self, attributes):
+        self._push_visibility (0)
+
+    def end_style (self):
+        self._pop_visibility ()
+
+
 
     def start_b (self, attributes):
         self._doc.set_style ("b")
@@ -1621,9 +1879,13 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
                 alignment = 3
             else:
                 print "Unknown alignment '%s' requested in <p>" % align
-        self._doc.add_vspace (0)
+        if self.atable is not None and self.in_cell:
+            self.atable.set_align (alignment)
+        else:
+            self._doc.add_vspace (0)
         self._doc.set_alignment (alignment)
-        self.do_p (attributes)
+        if self.atable is None:
+            self.do_p (attributes)
 
     def end_p (self):
         self._doc.add_vspace (0)
@@ -1789,6 +2051,7 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
                     text = string.lowercase[number-1] + ") "
                 else:
                     text = "?) "
+            table_margin = self._ol_list_depth
         else:
             if self._ul_list_depth == 1:
                 text = ((0x2022, "o"), " ")
@@ -1802,10 +2065,16 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
             else:
                 text = "> "
                 indent = 4  # Mike: is this correct?
+            table_margin = self._ul_list_depth
 
         self._doc.indent (indent, 0)
 
         self._doc.set_style ("")  # make sure we render the 'bullet' marker in normal style
+        if self.atable is not None and self.in_cell:
+            self._add_text('\xa0\xa0' * table_margin)
+            style_str = struct.pack (">BBBBB", 0, 0x53, 0, 0, 0) # black
+            self.atable.add_cell_text(style_str)
+
         if type(text) == type(""):
             self._add_text (text)
         elif type(text) == type(()):
@@ -1816,6 +2085,15 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
                     self._add_unicode_char(element[0], element[1])
         self._doc.unset_style ("")
 
+        if self.atable is not None and self.in_cell:
+            new_forecolor = self._doc.get_forecolor()
+            rgb = new_forecolor
+            r = string.atoi (rgb[0:2], 16)
+            g = string.atoi (rgb[2:4], 16)
+            b = string.atoi (rgb[4:6], 16)
+            style_str = struct.pack (">BBBBB", 0, 0x53, r, g, b)
+            self.atable.add_cell_text(style_str)
+
         self._push_element ("li", indent)
 
 
@@ -1899,7 +2177,7 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
                 self._doc.add_table (attributes)
             self.table_count = self.table_count + 1
             self.last_table_style = 0
-            self.last_table_forecolor = ["000000"]
+            self.last_table_forecolor = "000000"
 
 
     def end_table (self):
@@ -2067,6 +2345,15 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
         #self.end_blockquote ()
 
 
+    def do_link (self, attr):
+        if self._config and self._config.get_bool('bookmarks', 0):
+            attrib = _list_to_dict(attr)
+            if attrib.has_key('rel') and attrib['rel'] == 'bookmark':
+                if attrib.has_key('title') and attrib.has_key('href'):
+                    href = Url.CleanURL (attrib['href'], self._base or self._url) 
+                    self._doc._doc.add_bookmark (attrib['title'], href)
+
+
     ## These get called on unhandled things.
     ## We collect info about them in self._unknown and print that out
     ## on demand via print_unknown()
@@ -2139,7 +2426,8 @@ class StructuredHTMLParser (sgmllib.SGMLParser):
                         self.unknown_charref(m.group(1))
             else:
                 self._unknown["entityref-%s"%ref] = 1
-                self.handle_data('?')
+                #self.handle_data('?')
+                self.handle_data("&"+ref)
 
 
 
