@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Url.py   $Id: Url.py,v 1.19 2003/08/22 12:43:09 prussar Exp $
+Url.py   $Id: Url.py,v 1.20 2003/12/17 02:28:47 jimj Exp $
 
 Utility class to encapsulate information about an URL and the useful
 operations thereon.
@@ -131,6 +131,10 @@ class URL:
             self._fragment = url._fragment
         else:            
             url = str (url)
+            # Sometimes an URL is interrupted by a line break.
+            # Fetching works better if the linebreak is removed.
+            url = string.replace(url, "\r", "")
+            url = string.replace(url, "\n", "")
             if base is not None:
                 if sys.platform == 'win32' and string.lower(str (base)[0:5]) == 'file:':
                     url = windows_file_urljoin (str (base), url)
