@@ -2,7 +2,7 @@
 #  -*- mode: python; indent-tabs-mode: nil; -*-
 
 """
-Writer.py   $Id: Writer.py,v 1.36 2003/08/17 20:33:54 chrish Exp $
+Writer.py   $Id: Writer.py,v 1.39 2004/04/08 00:57:34 chrish Exp $
 
 Write a collection of documents into various formats.
 
@@ -129,6 +129,8 @@ class Mapper:
              if isinstance(doc, PluckerDocs.PluckerIndexDocument):
                  # there's only one, and it always has record # 1
                  id = 1
+             elif isinstance(doc, PluckerDocs.PluckerBookmarkDocument):
+                 id = 6
              elif isinstance(doc, PluckerDocs.PluckerLinkIndexDocument):
                  id = 3
              elif isinstance(doc, PluckerDocs.PluckerCategoryDocument):
@@ -328,6 +330,7 @@ class Writer:
             self._mapper.print_mapping()
 
         out_dict = {}
+        bookmarks = {}
         for pluckerdoc in self._mapper.get_docs():
             id = self._mapper.get_or_add(pluckerdoc)
             _print_convert_msg(pluckerdoc.get_url(), verbose, self._config)
@@ -345,6 +348,28 @@ class Writer:
                     charsets[doc_mibenum].append(id)
                 else:
                     charsets[doc_mibenum] = [id]
+
+                # Add doc.bookmarks to bookmark list
+                if self._config and self._config.get_bool('bookmark_pages', 0):
+                    key = pluckerdoc.get_url()
+                    pid = self._mapper.get_or_add(key)
+                    key = string.split(key, ":")
+                    key = key[-1]
+                    key = string.split(key, "/")
+                    key = key[-1]
+                    key = string.split(key, "?")
+                    key = key[0]
+                    if not len(key):
+                        key = 'Home Page'
+                    if not bookmarks.has_key(key):
+                        bookmarks[key] = (pid, 0)
+
+                if self._config and self._config.get_bool('bookmarks', 0):
+                    tmp_book = pluckerdoc.get_bookmark_ids()
+                    for key in tmp_book.keys():
+                        if not bookmarks.has_key(key):
+                            bookmarks[key] = tmp_book[key]
+
             self._write_doc (out_dict, pluckerdoc, pluckerdoc.get_url(), id, verbose)
 
         ## Do some error checking
@@ -391,9 +416,15 @@ class Writer:
 
         ## write the index record
         tmp_url = "plucker:/~special~/index"
-        type = PluckerDocs.PluckerIndexDocument (tmp_url, self._config, metadata)
+        type = PluckerDocs.PluckerIndexDocument (tmp_url, self._config, metadata, bookmarks)
         self._write_doc (out_dict, type, tmp_url, 1, verbose)
 
+        ## write the bookmark record (if any)
+        if len(bookmarks):
+            tmp_url = "plucker:/~special~/bookmarks"
+            bookdoc = PluckerDocs.PluckerBookmarkDocument(tmp_url, bookmarks)
+            self._write_doc (out_dict, bookdoc, tmp_url, 6, verbose)
+
         ## write the URL information, if desired
         if not self._config.get_bool ('no_urlinfo', 0):
             links = self._mapper.build_links()
@@ -497,6 +528,7 @@ class PDBWriter (Writer):
         self._big_icon = config.get_string ('big_icon','')
         self._small_icon = config.get_string ('small_icon','')
         self._config = config
+        self._creator_id = config.get_string('creator_id', 'Plkr')
 
 
     def write (self, verbose, alias_list, mapping=None):
@@ -512,7 +544,8 @@ class PDBWriter (Writer):
         info = self._pdb_file.getDBInfo ()
         info['name'] = self._dbname
         info['version'] = self._dbversion
-        info['creator'] = 'Plkr'
+        info['creator'] = self._creator_id
+        #info['creator'] = 'Plkr'
         info['type'] = 'Data'
         info['createDate'] = int (time.time())
         info['modifyDate'] = info['createDate']
