@@ -1,5 +1,5 @@
 #
-#  $Id: prc.py,v 1.9 2000/07/26 09:34:11 nordstrom Exp $
+#  $Id: prc.py,v 1.13 2002/10/10 21:58:18 janssen Exp $
 #  Original RCSId: prc.py,v 1.6 1999/07/16 05:32:01 rob Exp 
 #
 #  Copyright 1998-1999 Rob Tillotson <robt@debian.org>
@@ -88,6 +88,7 @@ import sys, os, stat, struct
 PI_HDR_SIZE = 78
 PI_RESOURCE_ENT_SIZE = 10
 PI_RECORD_ENT_SIZE = 8
+PI_MAX_DOC_NAME_SIZE = 31
 
 PILOT_TIME_DELTA = 2082844800L
 
@@ -963,21 +964,37 @@ class File(PCache):
         if info.get('flagCopyPrevention',0): flg = flg | flagCopyPrevention
         if info.get('flagLaunchableData',0): flg = flg | flagLaunchableData
         # excludefromsync doesn't actually get stored?
-        hdr = struct.pack('>32shhLLLlll4s4sllh',
-                          pad_null(info.get('name',''), 32),
-                          flg,
-                          info.get('version',0),
-                          info.get('createDate',0L)+PILOT_TIME_DELTA,
-                          info.get('modifyDate',0L)+PILOT_TIME_DELTA,
-                          info.get('backupDate',0L)+PILOT_TIME_DELTA,
-                          info.get('modnum',0),
-                          appinfo_offset, # appinfo
-                          sortinfo_offset, # sortinfo
-                          info.get('type','    '),
-                          info.get('creator','    '),
-                          0, # uid???
-                          0, # nextrec???
-                          len(self.data))
+        name = info.get('name', '')
+        if "_jython" in sys.builtin_module_names:
+            import string
+            # work around bug in Jython implementation of struct.pack
+            nameparts = []
+            while name:
+                partlen = min(len(name),8)
+                nameparts.append(struct.pack('>8s', name[:partlen]))
+                name = name[partlen:]
+            packed_name = string.join(nameparts, '')
+            if len(packed_name) < 32:
+                packed_name = packed_name + (32-len(packed_name))*'\0'
+            elif len(packed_name) > 31:
+                packed_name = packed_name[:31] + '\0'
+        else:
+            nameval = (len(name) > 31 and (name[:31] + '0')) or name
+            packed_name = struct.pack('>32s', name)
+        hdr = packed_name + struct.pack('>hhLLLlll4s4sllh',
+                                        flg,
+                                        info.get('version',0),
+                                        info.get('createDate',0L)+PILOT_TIME_DELTA,
+                                        info.get('modifyDate',0L)+PILOT_TIME_DELTA,
+                                        info.get('backupDate',0L)+PILOT_TIME_DELTA,
+                                        info.get('modnum',0),
+                                        appinfo_offset, # appinfo
+                                        sortinfo_offset, # sortinfo
+                                        info.get('type','    '),
+                                        info.get('creator','    '),
+                                        0, # uid???
+                                        0, # nextrec???
+                                        len(self.data))
 
         f.write(hdr)
 
@@ -999,3 +1016,4 @@ class File(PCache):
         f.write(self.appblock)
         f.write(self.sortblock)
         for x in record_data: f.write(x)
+        f.flush()
