@@ -6,7 +6,7 @@
 // Created:     2001/10/20
 // Copyright:   (c) Robert O'Connor ( rob@medicalmnemonics.com )
 // Licence:     GPL
-// RCS-ID:      $Id: build_progress_dialog.cpp,v 1.7 2003/06/18 03:00:40 robertoconnor Exp $
+// RCS-ID:      $Id: build_progress_dialog.cpp,v 1.9 2004/01/04 00:57:49 robertoconnor Exp $
 
 //----------------------------------------------------------------------------------------
 
@@ -116,6 +116,7 @@ build_progress_dialog::build_progress_dialog( wxWindow* parent,
 #if ( setupUSE_MSW_FORCE_COMMANDLINE_AUTOCLOSE_PROGRESS_CLOSURE )
     m_force_commandline_autoclose = force_commandline_autoclose;
 #endif
+    wxLogDebug( "Finished build_progress_dialog constructor" );
 
 }
 
@@ -153,7 +154,8 @@ bool build_progress_dialog::on_before_execute_commandline_array()
     // If a before_group_command was specified, execute it. TRUE means wait for it to finish
     wxString before_group_command;
     before_group_command = the_configuration->Read( "/PLUCKER_DESKTOP/before_group_command", wxT( "" ) );
-    if ( before_group_command != wxT( "" ) ) {
+    if ( before_group_command != wxT( "" ) ) 
+	{
         get_details_listbox()->append_using_scroll_history( _( "Executing command before spidering channels..." ) );
         ::wxExecute( before_group_command, TRUE );
     }
@@ -202,7 +204,8 @@ bool build_progress_dialog::on_before_execute_commandline_array_item( size_t com
     bool verbosity;
     verbosity = the_configuration->Read( "/" + current_section_string + "/" +
                                          "verbosity", 1L );
-    if ( verbosity == 0 ) {
+    if ( verbosity == 0 ) 
+	{
         get_subtask_statictext()->SetLabel( _( "Processing " ) + doc_name + " (Updating in quiet mode)" );
         if ( get_subtask_gauge()->IsShown() ) {
             get_subtask_gauge()->Show( FALSE );
@@ -211,9 +214,12 @@ bool build_progress_dialog::on_before_execute_commandline_array_item( size_t com
         // program will appear to have frozen.
         set_stream_kind( optionPROCESS_PROGRESS_DIALOG_NONE_STREAM_KIND );
         }
-    } else {
+    } 
+	else 
+	{
         get_subtask_statictext()->SetLabel( _( "Initializing channel: " ) + doc_name + "..." );
-        if ( ! get_subtask_gauge()->IsShown() ) {
+        if ( ! get_subtask_gauge()->IsShown() ) 
+		{
             get_subtask_gauge()->Show( TRUE );
         }
         // Set the stream kind that we want to catch to be stderr (or stdout if
@@ -303,7 +309,8 @@ bool build_progress_dialog::on_after_commandline_array_executions_finished()
     // finish
     wxString after_group_command;
     after_group_command = the_configuration->Read( "/PLUCKER_DESKTOP/after_group_command", wxT( "" ) );
-    if ( after_group_command != wxT( "" ) ) {
+    if ( after_group_command != wxT( "" ) ) 
+	{
         get_details_listbox()->append_using_scroll_history( _( "Executing command after spidering channels..." ) );
         wxExecute( after_group_command, TRUE );
     }
@@ -322,7 +329,8 @@ void build_progress_dialog::on_process_generated_an_output_line( long stream_kin
     get_details_listbox()->append_using_scroll_history( line_text );
 
 
-    if ( line_text.Matches( "*collected, * to do*" ) ) {
+    if ( line_text.Matches( "*collected, * to do*" ) ) 
+	{
 
         wxString    value_string;
         wxString    range_string;        
@@ -343,8 +351,10 @@ void build_progress_dialog::on_process_generated_an_output_line( long stream_kin
         // The range string is now "9"        
 
         // Try to convert these strings to longs (returns false if conversion failed).. 
-        if ( value_string.ToLong( &value_long ) ) {
-            if ( range_string.ToLong( &range_long ) ) {
+        if ( value_string.ToLong( &value_long ) ) 
+		{
+            if ( range_string.ToLong( &range_long ) ) 
+			{
                 // Note that range_long is current still the number remaining. 
                 // Add it to value to get a proper range.
                 range_long = range_long + value_long;
@@ -365,15 +375,21 @@ void build_progress_dialog::on_process_generated_an_output_line( long stream_kin
         }
         
     // If it was a "converting the retrieved files" message...            
-    } else if ( line_text.StartsWith( "Convert" ) ) {
+    } 
+	else if ( line_text.StartsWith( "Convert" ) ) 
+	{
         // ...Update the static text with a message
         get_subtask_statictext()->SetLabel( _( "Converting retrieved files...." ) );
     // If it was a "writing the files into pdb" message...
-    } else if ( line_text.StartsWith( "Wrote " ) ) {
+    } 
+	else if ( line_text.StartsWith( "Wrote " ) ) 
+	{
         // ...Update the static text with a message
         get_subtask_statictext()->SetLabel( _( "Conversion complete. Writing output file..." ) );      
      // It was a "retrieval of files is complete" message:
-    } else if ( line_text.StartsWith( "---- all" ) ) {
+    } 
+	else if ( line_text.StartsWith( "---- all" ) ) 
+	{
         // ...Set the progress bar to max (range is always 
         // plkrFIXED_PERCENTAGE_GAUGE_RANGE to stop flicker, see notes in 
         // plucker_defines.h
@@ -401,7 +417,8 @@ void build_progress_dialog::action_to_do_after_dialog_finishes_closing()
 {
 # if ( setupUSE_MSW_FORCE_COMMANDLINE_AUTOCLOSE_PROGRESS_CLOSURE )
     // Recall we set this when called the wrapper (and hence the build_progress_dialog)
-    if ( m_force_commandline_autoclose == optionFORCE_PROGRAM_TERMINATION_ON_PROGRESS_DIALOG_CLOSE ) {
+    if ( m_force_commandline_autoclose == optionFORCE_PROGRAM_TERMINATION_ON_PROGRESS_DIALOG_CLOSE ) 
+    {
         // Obviously a MSW-only command
         ::PostQuitMessage( 0 );
     }
@@ -430,7 +447,8 @@ void build_progress_dialog::send_to_destinations_and_inform( wxArrayString& sect
     wxString current_section_string;
     for ( size_t current_section_array_index = 0;
           (int) current_section_array_index < (int) sections_to_install.GetCount();
-          current_section_array_index++ ) {
+          current_section_array_index++ ) 
+    {
         current_section_string = sections_to_install.Item( current_section_array_index );
         plucker_controller::get()->set_channel_update_base( current_section_string );
     }
