@@ -6,7 +6,7 @@
 // Created:     2001/10/20
 // Copyright:   (c) Robert O'Connor
 // Licence:     GPL
-// RCS-ID:      $Id: configuration.cpp,v 1.36 2003/08/06 17:35:48 robertoconnor Exp $
+// RCS-ID:      $Id: configuration.cpp,v 1.38 2004/01/04 00:57:49 robertoconnor Exp $
 //----------------------------------------------------------------------------------------
 
 //----------------------------------------------------------------------------------------
@@ -147,7 +147,8 @@ void init_configuration()
 #ifdef __WXMAC__
     // If no image parser specified in their plucker.ini (which is what happens in a new
     // install), then use pil2:
-    if ( the_configuration->Read( "/DEFAULT/image_parser", "" ) == "" ) {
+    if ( the_configuration->Read( "/DEFAULT/image_parser", "" ) == "" ) 
+    {
         the_configuration->Write( "/DEFAULT/image_parser", "pil2" );
         the_configuration->Flush();
     }    
@@ -159,7 +160,8 @@ void init_configuration()
 // Delete the configuration object when finished program, otherwise memory leak
 void finish_configuration()
 {
-   if ( the_configuration ) {
+   if ( the_configuration ) 
+   {
       wxConfigBase::Set( NULL );
       delete the_configuration;
       the_configuration = NULL;
@@ -228,15 +230,21 @@ wxString get_os_configuration_section()
 // a home_url key.
 bool is_channel_section( wxString section )
 {   
-    if ( is_reserved_section( section ) ) {
+    if ( is_reserved_section( section ) ) 
+    {
         return FALSE;
-    } else {
+    }
+    else 
+    {
         wxString key;
         
         key = section << '/' << "home_url";
-        if ( the_configuration->HasEntry( key ) ) {
+        if ( the_configuration->HasEntry( key ) ) 
+        {
             return TRUE;   
-        } else {
+        } 
+        else 
+        {
             return FALSE;
         }   
     }
@@ -249,9 +257,12 @@ bool is_channel_section( wxString section )
 // some settings for an actual channel.
 bool is_modifier_section( wxString section )
 {
-    if ( is_reserved_section( section ) || is_channel_section( section ) ) {
+    if ( is_reserved_section( section ) || is_channel_section( section ) ) 
+    {
         return FALSE;
-    } else {
+    } 
+    else 
+    {
         return TRUE;
     }   
 }
@@ -265,11 +276,14 @@ bool is_reserved_section( wxString section )
                                                         NULL );
     // Check to see whether the section passed is a member of the list of strings
     // of reserved sections
-    if ( reserved_sections->Member( section ) ) {
+    if ( reserved_sections->Member( section ) ) \
+    {
         // TODO: Do stringlists need to be destroyed, or is it automatic?
         delete reserved_sections;
         return TRUE;
-    } else {
+    } 
+    else 
+    {
         delete reserved_sections;
         return FALSE;
     }
@@ -283,7 +297,8 @@ wxString get_legal_new_channel_section( wxString string_to_convert )
     
     // If it was a string of nothing then set it to plkrUNNAMED_CHANNEL_NAME
     // to avoid any asserts working on NULL strings.
-    if ( string_to_convert == "" ) {
+    if ( string_to_convert == "" ) 
+    {
         string_to_convert = plkrUNNAMED_CHANNEL_NAME;
     }
 
@@ -295,7 +310,8 @@ wxString get_legal_new_channel_section( wxString string_to_convert )
     // If it was a string of nothing but illegal characters, and we now have just
     // an empty string, then set it to plkrUNNAMED_CHANNEL_NAME to avoid any 
     // asserts working on NULL strings.
-    if ( string_to_convert == "" ) {
+    if ( string_to_convert == "" ) 
+    {
         string_to_convert = plkrUNNAMED_CHANNEL_NAME;
     }
     
@@ -304,7 +320,8 @@ wxString get_legal_new_channel_section( wxString string_to_convert )
     string_to_convert.Truncate( plkrMAXIMUM_BASE_SECTION_LENGTH );
     
     // If it is a reserved section name by now, then stick a _ after the last character
-    if ( is_reserved_section( string_to_convert ) ) {
+    if ( is_reserved_section( string_to_convert ) ) 
+    {
         string_to_convert += "_";        
     }               
     
@@ -327,7 +344,8 @@ wxString get_legal_new_channel_section( wxString string_to_convert )
     for ( long n = 1;    
           the_configuration->HasGroup( string_to_convert ) 
           || wxDirExists( get_plucker_directory( CHANNELS ) + '/' + string_to_convert );        
-          n++ ) {
+          n++ ) 
+    {
         // Cut off any number extensions that may have been added in a previous
         // iteration of the loop.
         wxLogDebug( "Before truncate, string_to_convert=" + string_to_convert );
@@ -353,7 +371,8 @@ wxString get_pluckerhome_directory()
 #if defined(__WXGTK__) || defined(__WXX11__) || defined(__WXMOTIF__)
     bool pluckerhome_exists;
     pluckerhome_exists = wxGetEnv( "PLUCKERHOME", &pluckerhome_directory );
-    if ( ! pluckerhome_exists ) {
+    if ( ! pluckerhome_exists ) 
+    {
         pluckerhome_directory = wxGetHomeDir() << "/.plucker";
     }
 #endif  
@@ -361,7 +380,8 @@ wxString get_pluckerhome_directory()
 #ifdef __WXMAC__
     bool pluckerhome_exists;
     pluckerhome_exists = wxGetEnv( "PLUCKERHOME", &pluckerhome_directory );
-    if ( ! pluckerhome_exists ) {
+    if ( ! pluckerhome_exists ) 
+    {
         pluckerhome_directory = wxGetHomeDir() << "/Library/Plucker";
     }
 #endif  
@@ -409,7 +429,8 @@ wxString get_plucker_directory( long directory_name )
     // turned off since would expand everything with a $
     wxString    default_directory;
 
-    switch ( directory_name ) {
+    switch ( directory_name )
+    {
 //--------------------------------------------------------------------------------
         case PLUCKERHOME:
             directory = get_pluckerhome_directory();
@@ -505,7 +526,22 @@ wxString get_plucker_directory( long directory_name )
             directory = the_configuration->Read( "/PLUCKER_DESKTOP/directory_pyplucker",
                                                  default_directory );
 
-            wxLogDebug( "get_plucker_directory( WXWINDOWS_CONFIGURATION )=" + directory );
+            wxLogDebug( "get_plucker_directory( PYPLUCKER )=" + directory );
+            break;
+//--------------------------------------------------------------------------------
+        // Directory holding python.exe. Only for MSW. Mac and Linux are available
+        // anywhere, as they are in the path.
+        case PYTHON_VM:
+#if defined(__WXMSW__)
+            default_directory = get_bundle_runtime_files_directory();
+            default_directory += "/parser/python/vm";
+#else 
+            wxLogError( "Error: You shouldn't have been able to reach code in Plucker Desktop that retrieves PYPLUCKER directory" );
+#endif
+            directory = the_configuration->Read( "/PLUCKER_DESKTOP/directory_python_vm",
+                                                 default_directory );
+
+            wxLogDebug( "get_plucker_directory( PTYHON_VM )=" + directory );
             break;
 //--------------------------------------------------------------------------------
              
@@ -522,7 +558,8 @@ void copy_section( wxString from_section, wxString to_section, bool treat_as_abs
     wxLogDebug( "About to copy " + from_section + " section to " + to_section );
 
     // Error check that they aren't empty
-    if ( from_section.IsEmpty() || to_section.IsEmpty() ) {
+    if ( from_section.IsEmpty() || to_section.IsEmpty() ) 
+    {
         wxLogDebug( "Error: can't copy a section since either source or target is empty." );
         return;
     }
@@ -533,11 +570,14 @@ void copy_section( wxString from_section, wxString to_section, bool treat_as_abs
     // If asked to treat the from_section and to_section as absolute, then convert them:
     // Of course of they already have a '/' at their start, then we don't want to add another
     // one.
-    if ( treat_as_absolute ) {
-        if ( ! to_section.StartsWith( "/", NULL ) ) {
+    if ( treat_as_absolute ) 
+    {
+        if ( ! to_section.StartsWith( "/", NULL ) ) 
+        {
             to_section = '/' + to_section;
         }
-        if ( ! from_section.StartsWith( "/", NULL ) ) {
+        if ( ! from_section.StartsWith( "/", NULL ) ) 
+        {
             from_section = '/' + from_section;
         }
     }
@@ -562,7 +602,8 @@ void copy_section( wxString from_section, wxString to_section, bool treat_as_abs
     wxLogDebug( "Tested whether first key exists. Result=%d", next_key_exists );
     wxLogDebug( "The name of that key=" + key_name );
        
-    while ( next_key_exists ) {        
+    while ( next_key_exists ) 
+    {        
         // Read key_name's value from from_section 
         key_value = the_configuration->Read( key_name, "" );
         
@@ -593,7 +634,8 @@ void import_sections_as_section_names( wxArrayString& selected_sections,
                                        wxFileConfig* to_configuration
                                      )
 {
-    for ( size_t i=0; i < selected_sections.GetCount(); i++ ) {
+    for ( size_t i=0; i < selected_sections.GetCount(); i++ ) 
+    {
           copy_section2( selected_sections.Item( i ),
                          selected_sections.Item( i ),
                          options,
@@ -617,13 +659,15 @@ void copy_section2( const wxString& from_section,
     wxLogDebug( "About to copy " + a_from_section + " section to " + a_to_section );
 
     // Error check that they aren't empty
-    if ( a_from_section.IsEmpty() || a_to_section.IsEmpty() ) {
+    if ( a_from_section.IsEmpty() || a_to_section.IsEmpty() )
+    {
         wxLogDebug( "Error: can't copy a section since either source or target section is empty." );
         return;
     }
 
     // Error check that the configuration objects exist
-    if ( ! ( from_configuration && to_configuration ) ) {
+    if ( ! ( from_configuration && to_configuration ) ) 
+    {
        wxLogDebug( "Error: can't copy a section since either source or target wxFileConfig is NULL." );
        return;
     }
@@ -640,11 +684,14 @@ void copy_section2( const wxString& from_section,
     // If asked to treat the a_from_section and a_to_section as absolute, then convert them:
     // Of course of they already have a '/' at their start, then we don't want to add another
     // one.
-    if ( options & optionCONFIGURATION_COPY_SECTION_TREAT_AS_ABSOLUTE ) {
-        if ( ! a_to_section.StartsWith( "/", NULL ) ) {
+    if ( options & optionCONFIGURATION_COPY_SECTION_TREAT_AS_ABSOLUTE ) 
+    {
+        if ( ! a_to_section.StartsWith( "/", NULL ) ) 
+        {
             a_to_section = '/' + a_to_section;
         }
-        if ( ! a_from_section.StartsWith( "/", NULL ) ) {
+        if ( ! a_from_section.StartsWith( "/", NULL ) ) 
+        {
             a_from_section = '/' + a_from_section;
         }
     }
@@ -667,7 +714,8 @@ void copy_section2( const wxString& from_section,
     wxLogDebug( "Tested whether first key exists. Result=%d", next_key_exists );
     wxLogDebug( "The name of that key=" + key_name );
 
-    while ( next_key_exists ) {
+    while ( next_key_exists ) 
+    {
         // Read key_name's value from a_from_section
         key_value = from_configuration->Read( key_name, "" );
 
@@ -686,7 +734,8 @@ void copy_section2( const wxString& from_section,
     from_configuration->SetPath( old_from_configuration_path );
     to_configuration->SetPath( old_to_configuration_path );
 
-    if ( options & optionCONFIGURATION_COPY_SECTION_FLUSH_AT_END ) {
+    if ( options & optionCONFIGURATION_COPY_SECTION_FLUSH_AT_END )
+    {
         // Flush the toilet when you are done (writes the changes to config file).
         to_configuration->Flush();
     }
@@ -710,11 +759,13 @@ void copy_all_sections( wxFileConfig* from_configuration,
     wxLogDebug( "copy_all_sections: The name of that section=%s", section_name.c_str() );
 
     // If flushing at end of all copies, then don't want to flush after each section.
-    if (  options & optionCONFIGURATION_COPY_SECTION_FLUSH_AT_END ) {
+    if (  options & optionCONFIGURATION_COPY_SECTION_FLUSH_AT_END ) 
+    {
         passed_options = options - optionCONFIGURATION_COPY_SECTION_FLUSH_AT_END;
     }
 
-    while ( next_section_exists ) {
+    while ( next_section_exists ) 
+    {
         wxLogDebug( "copy channel section before=%s", section_name.c_str() );
         // Copy over section (error checks that from_configuration is OK).
         copy_section2( section_name,
@@ -729,7 +780,8 @@ void copy_all_sections( wxFileConfig* from_configuration,
     }
 
     // If told to flush now, then flush the to_configuration
-    if ( options & optionCONFIGURATION_COPY_SECTION_FLUSH_AT_END ) {
+    if ( options & optionCONFIGURATION_COPY_SECTION_FLUSH_AT_END ) 
+    {
         to_configuration->Flush();
     }
 }
@@ -742,8 +794,10 @@ void strip_depreciated_keys( wxString section )
 
     the_configuration->SetPath( section );
 
-    for ( int i=0; i < (int)WXSIZEOF( depreciated_channel_keys ); i++ ) {
-        if ( the_configuration->Exists( depreciated_channel_keys[ i ] ) ) {
+    for ( int i=0; i < (int)WXSIZEOF( depreciated_channel_keys ); i++ )
+    {
+        if ( the_configuration->Exists( depreciated_channel_keys[ i ] ) ) 
+        {
             the_configuration->DeleteEntry( depreciated_channel_keys[ i ], FALSE );
         }
     }
@@ -765,22 +819,26 @@ wxString protocol_filename_to_normal_fullname( const wxString &input_filename )
     prefix_to_strip = get_protocol_prefix( output_fullname );    
 
     // If found one, then remove it, placing rest into output_fullname.
-    if ( prefix_to_strip != wxEmptyString ) {
+    if ( prefix_to_strip != wxEmptyString )
+    {
         output_fullname.StartsWith( prefix_to_strip, &output_fullname );        
         // If found one of plucker protocols, then tell to prepend full path to pluckerdir    
-        if ( prefix_to_strip.Contains( wxT( "plucker" ) ) ) {
+        if ( prefix_to_strip.Contains( wxT( "plucker" ) ) ) 
+        {
             prepend_plucker_path = TRUE;
         }        
     }    
     
     // Also prepend plucker path to relative filenames. Using wxIsAbsolutePath() instead
     // of checking for a leading '/' as MSW/MAC can start an absolute with a drive.
-    if ( ! wxIsAbsolutePath( output_fullname ) ) {
+    if ( ! wxIsAbsolutePath( output_fullname ) )
+    {
         prepend_plucker_path = TRUE;
     }
 
     // Do the prepend if so requested
-    if ( prepend_plucker_path ) {
+    if ( prepend_plucker_path )
+    {
         // ?TODO: Should this be the user's pluckerdir on POSIX. 
         output_fullname = get_plucker_directory( PLUCKERHOME ) + "/" + output_fullname;
     }
@@ -798,7 +856,8 @@ void set_close_on_exit( const wxString& section,
     wxString close_on_exit_key  = "/" + section + "/" + "close_on_exit";
     
     // Write the keys if already exist, or we specified that want to write them anyways.
-    if ( the_configuration->Exists( close_on_exit_key ) || ! only_write_if_key_exists ) {
+    if ( the_configuration->Exists( close_on_exit_key ) || ! only_write_if_key_exists ) 
+    {
         the_configuration->Write( close_on_exit_key, (long)enabled );
     }
     
@@ -817,12 +876,14 @@ void set_close_on_error( const wxString& section,
     wxString close_on_error_key  = "/" + section + "/" + "close_on_error";
     
     // Write the keys if already exist, or we specified that want to write them anyways.
-    if ( the_configuration->Exists( close_on_error_key ) || ! only_write_if_key_exists ) {
+    if ( the_configuration->Exists( close_on_error_key ) || ! only_write_if_key_exists ) 
+    {
         the_configuration->Write( close_on_error_key, (long)enabled );
     }
     
     // Flush changes if so requested
-    if ( flush ) {
+    if ( flush ) 
+    {
         the_configuration->Flush();
     }
 }
@@ -841,19 +902,23 @@ wxString get_protocol_prefix( wxString &input_filename )
                                 wxT( "HTTP:" )
                               };
                           
-    for ( int n=0; n < (int)WXSIZEOF( prefix_bases ); n++ ) {    
+    for ( int n=0; n < (int)WXSIZEOF( prefix_bases ); n++ ) 
+    {    
         // Does it have just the prefix base
-        if ( input_filename.StartsWith( prefix_bases[ n ] ) ) {
+        if ( input_filename.StartsWith( prefix_bases[ n ] ) ) 
+        {
             found_prefix = prefix_bases[ n ];            
         }
         
         // Does it have the prefix base + 1 slash
-        if ( input_filename.StartsWith( prefix_bases[ n ] + wxT( "/" ) ) ) {
+        if ( input_filename.StartsWith( prefix_bases[ n ] + wxT( "/" ) ) ) 
+        {
             found_prefix = prefix_bases[ n ] + wxT( "/" );
         }
         
         // Does it have the prefix base + 2 slashes
-        if ( input_filename.StartsWith( prefix_bases[ n ] + wxT( "//" ) ) ) {
+        if ( input_filename.StartsWith( prefix_bases[ n ] + wxT( "//" ) ) ) 
+        {
             found_prefix = prefix_bases[ n ] + wxT( "//" );
         }
     }
