@@ -6,7 +6,7 @@
 // Created:     2001/10/20
 // Copyright:   (c) Robert O'Connor ( rob@medicalmnemonics.com )
 // Licence:     GPL
-// RCS-ID:      $Id: details_progress_dialog.cpp,v 1.5 2003/03/17 17:43:19 robertoconnor Exp $
+// RCS-ID:      $Id: details_progress_dialog.cpp,v 1.6 2004/01/04 00:57:49 robertoconnor Exp $
 //----------------------------------------------------------------------------------------
 
 //----------------------------------------------------------------------------------------
@@ -209,7 +209,8 @@ void details_progress_dialog::set_flickerfree_gauge_and_range( wxGauge& gauge, i
      // Start at zero, in case range is also zero, then value will become zero.
     int new_value = 0;
     // If not diving by zero...
-    if ( range != 0 ) {
+    if ( range != 0 )
+    {
         // ...Get the new value on a range of 1000, based on value/range that was passed.
         // Needed a double value.
         new_value = ( plkrFIXED_PERCENTAGE_GAUGE_RANGE * value ) / range;
@@ -259,15 +260,16 @@ void details_progress_dialog::move_dialog_to_make_fully_visible_with_shown_listb
     dialog_top = GetPosition().y;
     
     // If the expanded listbox would go past the drawable screen height.
-    if ( screen_height < ( dialog_top + m_dialog_height_with_listbox ) ) {
-        
+    if ( screen_height < ( dialog_top + m_dialog_height_with_listbox ) )
+    {    
         // Start at the bottom and move it up accordingly.
         dialog_top =  screen_height;
         dialog_top -= m_dialog_height_with_listbox;
         // However, if the top would now impinge into a top taskbar, then set it 
         // down to top of the the usable screen area, so the caption is at least visible.
         // and let the bottom part disappear.
-        if ( dialog_top < screen_top ) {
+        if ( dialog_top < screen_top ) 
+        {
             dialog_top = screen_top;
         }
         
@@ -331,9 +333,12 @@ void details_progress_dialog::on_details_button( wxCommandEvent &event )
     // Show a busy cursor for the lifetime of this function
     wxBusyCursor busy_cursor;
     
-    if ( m_details_progress_listbox_is_shown ) {
+    if ( m_details_progress_listbox_is_shown ) 
+    {
         hide_listbox();
-    } else {
+    } 
+    else 
+    {
         show_listbox();
     }
 }
@@ -351,20 +356,23 @@ void details_progress_dialog::on_init_dialog( wxInitDialogEvent& event )
   
     // Only execute this in the first init dialog event (since GTK has a 
     // Show() in update_dialog_size_after_hide_or_show_listbox() ).
-    if ( m_is_initial_init_event ) {
-
+    if ( m_is_initial_init_event ) 
+    {
         // Set the initial init event to FALSE, so we can't run this stuff again. 
         m_is_initial_init_event = FALSE;  
 
         // Look in the configuration and see if they desired to hide it. If so, hide it
         // initially.
-        if ( wxConfigBase::Get( FALSE ) ) {
-            if ( m_show_hide_listbox_configuration_key != wxEmptyString ) {
+        if ( wxConfigBase::Get( FALSE ) ) 
+        {
+            if ( m_show_hide_listbox_configuration_key != wxEmptyString )
+            {
                 bool initially_show_details;
                 initially_show_details = (bool)wxConfigBase::Get()->Read( m_show_hide_listbox_configuration_key,
                                                                           1L );
                 wxLogDebug( "Read value in m_show_hide_listbox_configuration_key=%d", initially_show_details );
-                if ( ! initially_show_details ) {
+                if ( ! initially_show_details ) 
+                {
                     // Dialog is constructed with the listbox showing. Therefore, this call
                     // will hide it.                    
                     hide_listbox();
@@ -387,8 +395,8 @@ void details_progress_dialog::on_paint_dialog( wxPaintEvent& event )
     wxLogDebug( "Entering details_progress_dialog::on_paint_dialog()" );
 
    // If this is the initial paint event (we set it in the constructor to be TRUE)
-   if ( m_is_initial_paint_event ) {         
-               
+   if ( m_is_initial_paint_event ) 
+   {                     
         // Set the initial paint event to FALSE, so we can't run this stuff again. 
         m_is_initial_paint_event = FALSE;       
         
@@ -423,7 +431,8 @@ void details_progress_dialog::all_chores_completed_without_interruption()
     m_stop_button->SetLabel( _( "OK" ) );
 
     // If automatically close when done, then do the close behavior
-    if ( m_automatically_close_when_done ) {
+    if ( m_automatically_close_when_done )
+    {
         // Can't do this: close_dialog(). The program will hang.
         // close_dialog();
 
@@ -454,11 +463,13 @@ void details_progress_dialog::OnCloseWindow( wxCloseEvent& event )
     m_abort_signal_was_entered = TRUE;
 
     // Save whether we want to show the details for next time.
-    if ( wxConfigBase::Get( FALSE ) ) {
-        if ( m_show_hide_listbox_configuration_key != wxEmptyString ) {
-          wxConfigBase::Get()->Write( m_show_hide_listbox_configuration_key,
+    if ( wxConfigBase::Get( FALSE ) ) 
+    {
+        if ( m_show_hide_listbox_configuration_key != wxEmptyString ) 
+        {
+            wxConfigBase::Get()->Write( m_show_hide_listbox_configuration_key,
                                       m_details_progress_listbox_is_shown );
-          wxLogDebug( "Wrote value to m_show_hide_listbox_configuration_key=%d", m_details_progress_listbox_is_shown );
+            wxLogDebug( "Wrote value to m_show_hide_listbox_configuration_key=%d", m_details_progress_listbox_is_shown );
         }
     }
 
