@@ -6,7 +6,7 @@
 // Created:     2001/10/20
 // Copyright:   (c) Robert O'Connor ( rob@medicalmnemonics.com )
 // Licence:     GPL
-// RCS-ID:      $Id: editor_dialog.cpp,v 1.25 2003/06/26 03:27:50 robertoconnor Exp $
+// RCS-ID:      $Id: editor_dialog.cpp,v 1.26 2004/01/04 00:57:49 robertoconnor Exp $
 //----------------------------------------------------------------------------------------
 
 //----------------------------------------------------------------------------------------
@@ -190,10 +190,13 @@ editor_dialog::editor_dialog( wxWindow* parent, wxString filename_to_load )
     // in the XRC (similar to showcase_dialog.cpp--see there for comments).    
     bool use_advanced_toolbars = the_configuration->Read( "/PLUCKER_DESKTOP/editor_use_advanced_toolbars", 0L );
     
-    if ( use_advanced_toolbars ) {    
+    if ( use_advanced_toolbars ) 
+    {    
         wxXmlResource::Get()->AttachUnknownControl( wxT( "editor_dialog_toolbar_panel" ),
             wxXmlResource::Get()->LoadPanel( this, wxT( "editor_dialog_advanced_panel" ) ) );
-    } else {
+    } 
+    else 
+    {
         wxXmlResource::Get()->AttachUnknownControl( wxT( "editor_dialog_toolbar_panel" ),
             wxXmlResource::Get()->LoadPanel( this, wxT( "editor_dialog_basic_panel" ) ) );
     }
@@ -274,7 +277,8 @@ void editor_dialog::stc_or_textctrl_init()
 #if ( setupUSE_STYLED_TEXT_CONTROL )
 
     // Setup the default font, if it is OK
-    if ( default_font.Ok() ) {
+    if ( default_font.Ok() ) 
+    {
         m_editor_stc->StyleSetFont( wxSTC_STYLE_DEFAULT, default_font );
     }
     // Clear any current styles.
@@ -391,14 +395,16 @@ void editor_dialog::stc_or_textctrl_insert_text
     // editor preferences, and it is prudent to do so (ie a newline after a <i> </i> doesn't
     // make sense.
     bool insert_a_linefeed = the_configuration->Read( "/PLUCKER_DESKTOP/editor_tools_insert_linefeeds", 1L); 
-    if ( insert_a_linefeed && tag_can_insert_newline ) {
+    if ( insert_a_linefeed && tag_can_insert_newline ) 
+    {
         ending_text_to_insert += wxT( "\n" );  
     }        
     
 #if ( setupUSE_STYLED_TEXT_CONTROL )
     selected_text = m_editor_stc->GetSelectedText();
     
-    if ( selected_text == "" ) {
+    if ( selected_text == "" )
+    {
         // No text selected, just add the two strings at the insertion point
         int original_caret_position = m_editor_stc->GetCurrentPos();
         m_editor_stc->InsertText( original_caret_position, ending_text_to_insert );
@@ -410,7 +416,9 @@ void editor_dialog::stc_or_textctrl_insert_text
         int new_caret_position = original_caret_position + length_of_starting_text_to_insert;
         m_editor_stc->SetSelectionStart( new_caret_position );
         //m_editor_stc->SetCurrentPos( new_caret_position );
-    } else {
+    } 
+    else 
+    {
         // Some text is selected, so put the ending_text at the end of the selected text,
         // and the starting text before the selected text.
         // First, store the positions of start and end of selection.
@@ -434,7 +442,8 @@ void editor_dialog::stc_or_textctrl_insert_text
                                      &selection_ending_caret_position );
                         
     // If selection starting caret position equals the end, that means was no selection
-    if ( selection_starting_caret_position == selection_ending_caret_position ) {       
+    if ( selection_starting_caret_position == selection_ending_caret_position ) 
+    {       
         long original_caret_position = m_editor_textctrl->GetInsertionPoint();
         // No text selected, just add the two strings at the insertion point 
         m_editor_textctrl->WriteText( starting_text_to_insert );
@@ -445,7 +454,9 @@ void editor_dialog::stc_or_textctrl_insert_text
         long length_of_starting_text_to_insert = (long) starting_text_to_insert.Length();
         long new_caret_position = original_caret_position + length_of_starting_text_to_insert;
         m_editor_textctrl->SetInsertionPoint( new_caret_position );
-    } else {
+    } 
+    else
+    {
         // Some text is selected, so put the ending_text at the end of the selected text,
         // and the starting text before the selected text.
         // Set the insertion point to selection end, then insert ending text
@@ -514,14 +525,16 @@ void editor_dialog::stc_or_textctrl_save_file()
     //       that was modified.
     
     // Only bother saving if the text inside the textbox has been modified.
-    if ( m_editor_textctrl->IsModified() ) {     
+    if ( m_editor_textctrl->IsModified() ) 
+    {     
         // Save the HTML code from the text control, back into the file.
         // Recall "this->m_editted_filename" was the home.html file that we
         // squirreled away when we first opened this dialog.
     
         bool successful = m_editor_textctrl->SaveFile( this->m_editted_filename );        
         // Show a warning message if unable to save.
-        if( ! successful ) {
+        if( ! successful )
+        {
             wxLogError( _( "Unable to save home.html" ) );
         }
     }  
@@ -538,7 +551,8 @@ void editor_dialog::on_notebook_changing( wxNotebookEvent &event )
 { 
     // There is a on_notebook_changing event sent when loading dialog, and this causes a 
     // crash. Therefore we prevent it executing until our notebook is displayed.
-    if ( this->wxWindow::IsShown() ) {
+    if ( this->wxWindow::IsShown() ) 
+    {
         wxLogDebug( "Entering on_notebook_changing_function" );
         wxLogDebug( "Notebook event.GetOldSelection=%d", event.GetOldSelection() ); 
         if ( event.GetOldSelection() == EDITOR_NOTEBOOK_PAGE_INDEX_EDIT ) {   
@@ -553,7 +567,8 @@ void editor_dialog::on_toolbar_hyperlink_button( wxCommandEvent &event )
 {
    hyperlink_dialog a_hyperlink_dialog( this );   
     
-   if ( a_hyperlink_dialog.ShowModal() == wxID_OK ) {
+   if ( a_hyperlink_dialog.ShowModal() == wxID_OK )
+   {
        wxString starting_text;
        wxString ending_text;
        a_hyperlink_dialog.transfer_to( starting_text, ending_text );              
@@ -567,7 +582,8 @@ void editor_dialog::on_toolbar_mailto_button( wxCommandEvent &event )
 {
    email_dialog a_email_dialog( this );   
     
-   if ( a_email_dialog.ShowModal() == wxID_OK ) {
+   if ( a_email_dialog.ShowModal() == wxID_OK ) 
+   {
        wxString starting_text;
        wxString ending_text;
        a_email_dialog.transfer_to( starting_text, ending_text );              
@@ -580,7 +596,8 @@ void editor_dialog::on_toolbar_bookmark_button( wxCommandEvent &event )
 {
    bookmark_dialog a_bookmark_dialog( this );   
     
-   if ( a_bookmark_dialog.ShowModal() == wxID_OK ) {
+   if ( a_bookmark_dialog.ShowModal() == wxID_OK )
+   {
        wxString starting_text;
        wxString ending_text;
        a_bookmark_dialog.transfer_to( starting_text, ending_text );              
@@ -593,7 +610,8 @@ void editor_dialog::on_toolbar_popup_button( wxCommandEvent &event )
 {
    popup_dialog a_popup_dialog( this );   
     
-   if ( a_popup_dialog.ShowModal() == wxID_OK ) {
+   if ( a_popup_dialog.ShowModal() == wxID_OK )
+   {
        wxString starting_text;
        wxString ending_text;
        a_popup_dialog.transfer_to( starting_text, ending_text );              
@@ -641,7 +659,8 @@ void editor_dialog::on_toolbar_image_button( wxCommandEvent &event )
 {
    image_dialog a_image_dialog( this );   
     
-   if ( a_image_dialog.ShowModal() == wxID_OK ) {
+   if ( a_image_dialog.ShowModal() == wxID_OK ) 
+   {
        wxString starting_text;
        wxString ending_text;
        a_image_dialog.transfer_to( starting_text, ending_text );              
@@ -654,7 +673,8 @@ void editor_dialog::on_toolbar_hr_button( wxCommandEvent &event )
 {
    hr_dialog a_hr_dialog( this );   
     
-   if ( a_hr_dialog.ShowModal() == wxID_OK ) {
+   if ( a_hr_dialog.ShowModal() == wxID_OK ) 
+   {
        wxString starting_text;
        wxString ending_text;
        a_hr_dialog.transfer_to( starting_text, ending_text );              
@@ -673,7 +693,8 @@ void editor_dialog::on_toolbar_span_button( wxCommandEvent &event )
 {
    span_dialog a_span_dialog( this );   
     
-   if ( a_span_dialog.ShowModal() == wxID_OK ) {
+   if ( a_span_dialog.ShowModal() == wxID_OK )
+   {
        wxString starting_text;
        wxString ending_text;
        a_span_dialog.transfer_to( starting_text, ending_text );              
@@ -728,7 +749,8 @@ void editor_dialog::on_toolbar_font_button( wxCommandEvent &event )
 {
    font_dialog a_font_dialog( this );   
     
-   if ( a_font_dialog.ShowModal() == wxID_OK ) {
+   if ( a_font_dialog.ShowModal() == wxID_OK )
+   {
        wxString starting_text;
        wxString ending_text;
        a_font_dialog.transfer_to( starting_text, ending_text );              
@@ -849,7 +871,8 @@ void editor_dialog::on_toolbar_table_button( wxCommandEvent &event )
 {
     table_dialog a_table_dialog( this );
 
-    if ( a_table_dialog.ShowModal() == wxID_OK ) {
+    if ( a_table_dialog.ShowModal() == wxID_OK ) 
+    {
         wxString starting_text;
         wxString ending_text;
         a_table_dialog.transfer_to( starting_text, ending_text );
@@ -868,7 +891,8 @@ void editor_dialog::on_toolbar_th_button( wxCommandEvent &event )
 {
     td_th_dialog a_td_th_dialog( this, optionTD_TH_DIALOG_IS_TH );
 
-    if ( a_td_th_dialog.ShowModal() == wxID_OK ) {
+    if ( a_td_th_dialog.ShowModal() == wxID_OK )
+    {
         wxString starting_text;
         wxString ending_text;
         a_td_th_dialog.transfer_to( starting_text, ending_text );
@@ -881,7 +905,8 @@ void editor_dialog::on_toolbar_td_button( wxCommandEvent &event )
 {
     td_th_dialog a_td_th_dialog( this, optionTD_TH_DIALOG_IS_TD );
 
-    if ( a_td_th_dialog.ShowModal() == wxID_OK ) {
+    if ( a_td_th_dialog.ShowModal() == wxID_OK ) 
+    {
         wxString starting_text;
         wxString ending_text;
         a_td_th_dialog.transfer_to( starting_text, ending_text );
@@ -894,7 +919,8 @@ void editor_dialog::on_toolbar_ordered_list_button( wxCommandEvent &event )
 {
     ol_dialog a_ol_dialog( this );
 
-    if ( a_ol_dialog.ShowModal() == wxID_OK ) {
+    if ( a_ol_dialog.ShowModal() == wxID_OK )
+    {
         wxString starting_text;
         wxString ending_text;
         a_ol_dialog.transfer_to( starting_text, ending_text );
@@ -906,7 +932,8 @@ void editor_dialog::on_toolbar_unordered_list_button( wxCommandEvent &event )
 {
     ul_dialog a_ul_dialog( this );
 
-    if ( a_ul_dialog.ShowModal() == wxID_OK ) {
+    if ( a_ul_dialog.ShowModal() == wxID_OK ) 
+    {
         wxString starting_text;
         wxString ending_text;
         a_ul_dialog.transfer_to( starting_text, ending_text );
@@ -960,7 +987,8 @@ void editor_dialog::on_toolbar_body_button( wxCommandEvent &event )
 {
    body_dialog a_body_dialog( this );   
     
-   if ( a_body_dialog.ShowModal() == wxID_OK ) {
+   if ( a_body_dialog.ShowModal() == wxID_OK )
+   {
        wxString starting_text;
        wxString ending_text;
        a_body_dialog.transfer_to( starting_text, ending_text );              
