@@ -6,7 +6,7 @@
 // Created:     2001/10/20
 // Copyright:   (c) Robert O'Connor ( rob@medicalmnemonics.com )
 // Licence:     GPL
-// RCS-ID:      $Id: exclusion_dialog.cpp,v 1.24 2003/03/17 17:43:19 robertoconnor Exp $
+// RCS-ID:      $Id: exclusion_dialog.cpp,v 1.25 2004/01/04 00:57:49 robertoconnor Exp $
 //----------------------------------------------------------------------------------------
 
 //----------------------------------------------------------------------------------------
@@ -113,10 +113,13 @@ exclusion_dialog::exclusion_dialog( wxWindow* parent, wxString configuration_sec
     
     // If configuration_section is DEFAULT, then load the shared exclusion list,
     // otherwise load the one from the channel (or NEW_CHANNEL_TEMPLATE, which is a channel).
-    if ( m_configuration_section == "DEFAULT" ) {
+    if ( m_configuration_section == "DEFAULT" ) 
+    {
         m_exclusion_filename = get_plucker_directory( PLUCKERHOME ) + '/' 
                                + "exclusionlist.txt";
-    } else {
+    } 
+    else
+    {
         m_exclusion_filename = get_plucker_directory( CHANNELS ) + '/' 
                                + m_configuration_section + '/' + "exclusionlist.txt";
     }
@@ -189,25 +192,32 @@ void exclusion_dialog::listctrl_load_rows()
     bool textfile_is_empty = false;  // Whether textfile is empty.
     
     // Check to see if file exists before try to open this file (saves a warning dialog)
-    if ( wxFileExists( m_exclusion_filename ) ) {
+    if ( wxFileExists( m_exclusion_filename ) ) 
+    {
         successful = textfile.Open( m_exclusion_filename );
-        if ( successful ) {
+        if ( successful ) 
+        {
             // Now check to make sure that there is at least some characters in it, else
             // Loading rows will crash plucker desktop.
-            if ( (int)textfile.GetLineCount() == 0 ) {
+            if ( (int)textfile.GetLineCount() == 0 ) 
+            {
                 textfile_is_empty = true;
             }
         }
-    } else { 
+    } 
+    else
+    { 
         successful = textfile.Create( m_exclusion_filename );
-        if ( successful ) {            
+        if ( successful ) 
+        {            
             textfile_is_empty = true;
         }
     }
     // If an empty text file, then need to add some default entries.
     // Will cause an error if try to read the strings from an empty textfile.
     // Adding two demonstration strings (an extension and url) into the file.
-    if ( textfile_is_empty ) {
+    if ( textfile_is_empty ) 
+    {
         textfile.AddLine( wxT( "0:-:.*\\.mp3$" ) );
         textfile.AddLine( wxT( "0:-:http://.*ad\\.doubleclick\\.net/.*" ) );
         // Write is like wxConfigBase::Flush(). It saves it to file.
@@ -223,7 +233,8 @@ void exclusion_dialog::listctrl_load_rows()
     // loop is over. 
     for ( string_to_parse = textfile.GetFirstLine();
           ! textfile.Eof();
-          string_to_parse = textfile.GetNextLine() ) {
+          string_to_parse = textfile.GetNextLine() ) 
+    {
         // Parse this string that was read from the textfile. 
         listctrl_parse_single_loaded_row( string_to_parse );
     }
@@ -244,7 +255,8 @@ void exclusion_dialog::listctrl_parse_single_loaded_row( wxString string_to_pars
     wxString priority;               // The priority of the excluded item
 
     // Ignore lines that are empty or start with a '#' as they are comments
-    if ( ! ( string_to_parse.StartsWith( "#", NULL ) || ( string_to_parse == wxT( "" ) ) ) ) {        
+    if ( ! ( string_to_parse.StartsWith( "#", NULL ) || ( string_to_parse == wxT( "" ) ) ) ) 
+    {        
         // Entries are of the format
         // 0:-:.*\.mp3$
         // Priority : Include(+) or Exclude(-) : Item to exclude (can contain a ':')
@@ -256,9 +268,12 @@ void exclusion_dialog::listctrl_parse_single_loaded_row( wxString string_to_pars
         action = string_to_parse.BeforeFirst(':');        
         exclusion = string_to_parse.AfterFirst(':');   
         // More descriptive to have "Include" / "Exclude" that just a +/-, so convert them
-        if ( action == '-') {
+        if ( action == '-') 
+        {
             action = _( "Exclude" );
-        } else {
+        } 
+        else
+        {
             action = _( "Include" );
         }
         
@@ -266,13 +281,16 @@ void exclusion_dialog::listctrl_parse_single_loaded_row( wxString string_to_pars
         // and stick it into the file extension listctrl. TODO-OPTIONAL: If people are putting
         // non-http URLS in there, then need to have a commented section divider in the 
         // file, like #--------BEGIN URLS----------.
-        if ( ! exclusion.Contains( "http" ) ) {
+        if ( ! exclusion.Contains( "http" ) ) 
+        {
             m_extensions_exclusion_listctrl->InsertItem( extensions_row_number, exclusion, 0 );
             m_extensions_exclusion_listctrl->SetItem( extensions_row_number, ACTION_COLUMN, action );
             m_extensions_exclusion_listctrl->SetItem( extensions_row_number, PRIORITY_COLUMN, priority );              
             extensions_row_number++; 
         // Else put in in the url tab.
-        } else {
+        } 
+        else 
+        {
             m_urls_exclusion_listctrl->InsertItem( urls_row_number, exclusion, 0 );
             m_urls_exclusion_listctrl->SetItem( urls_row_number, ACTION_COLUMN, action );
             m_urls_exclusion_listctrl->SetItem( urls_row_number, PRIORITY_COLUMN, priority );
@@ -295,14 +313,17 @@ void exclusion_dialog::listctrl_save_rows( exclusion_listctrl* listctrl )
     wxLogDebug( "Entering save rows function" );   
     
     // Recommended loop for listctrl.                                 
-    for ( ;; ) {
+    for ( ;; ) 
+    {
         item = listctrl->GetNextItem( item, wxLIST_NEXT_ALL );
         
         wxLogDebug( "Value of item is %ld", item );  
         
         // No more rows in the item, so break loop.
-        if ( item == -1 )
+        if ( item == -1 ) 
+        {
             break; 
+        }
 
         // Look up the properties of wxListItem--first thing is to set the id number
         // (m_itemId) to our current item so we know what item we are talking about
@@ -317,16 +338,20 @@ void exclusion_dialog::listctrl_save_rows( exclusion_listctrl* listctrl )
         listctrl->GetItem( info );
         exclusion_string = info.m_text;
         // Abort this interation of the loop if this exclusion is blank
-        if ( exclusion_string == "" ) {
+        if ( exclusion_string == "" ) 
+        {
             continue;
         }
         
         // Same strategy for the action column
         info.m_col = ACTION_COLUMN;
         listctrl->GetItem( info );        
-        if ( info.m_text == _( "Include" ) ) {
+        if ( info.m_text == _( "Include" ) )
+        {
             action_string = "+";
-        } else {
+        } 
+        else 
+        {
             action_string = "-";
         }
 
@@ -482,7 +507,8 @@ void exclusion_dialog::OnOK( wxCommandEvent& event )
     // Save the file. Only save if either of the listctrls is dirty (they set it when 
     // added/edited/deleted an entry.
     if ( m_extensions_exclusion_listctrl->get_is_listctrl_dirty() ||
-         m_urls_exclusion_listctrl->get_is_listctrl_dirty() )  {
+         m_urls_exclusion_listctrl->get_is_listctrl_dirty() )  
+    {
         save_exclusion_file();
     }
     // Get rid of the modal dialog. Not transferring any info from this modal's control
