@@ -3,7 +3,7 @@
 // Purpose:     A control to allow install to Palm handheld Usernames or
 //              directories.
 // Author:      Robert O'Connor
-// RCS-ID:      $Id: hhictrl.cpp,v 1.19 2003/07/27 21:57:53 robertoconnor Exp $
+// RCS-ID:      $Id: hhictrl.cpp,v 1.20 2004/01/04 00:57:49 robertoconnor Exp $
 // Copyright:   Robert O'Connor
 // Licence:     GPL
 //----------------------------------------------------------------------------------------
@@ -213,23 +213,34 @@ void wxHandheldInstallCtrl::InsertUserDestinations( handheld_dest_array_type& ha
     wxArrayString   row_cells_contents;
     wxArrayInt      matching_row_numbers;
     // Insert them
-    for ( size_t n = 0; n < handheld_dest_array.GetCount(); n++ ) {
+    for ( size_t n = 0; n < handheld_dest_array.GetCount(); n++ ) 
+    {
         row_to_insert =  m_usersListCtrl->GetItemCount();
             // Convert our handheld_target_storage_mode to the string in the column
-            if( handheld_dest_array[ n ]->handheld_target_storage_mode == plkrHANDHELD_TARGET_STORAGE_MODE_SD_CARD ) {
+            if( handheld_dest_array[ n ]->handheld_target_storage_mode == plkrHANDHELD_TARGET_STORAGE_MODE_SD_CARD )
+            {
                 ram_or_card_string = _( "SD Card" );
-            } else if( handheld_dest_array[ n ]->handheld_target_storage_mode == plkrHANDHELD_TARGET_STORAGE_MODE_MEMORY_STICK ) {
+            } 
+            else if( handheld_dest_array[ n ]->handheld_target_storage_mode == plkrHANDHELD_TARGET_STORAGE_MODE_MEMORY_STICK ) 
+            {
                 ram_or_card_string = _( "M.Stick" );
-            } else if( handheld_dest_array[ n ]->handheld_target_storage_mode == plkrHANDHELD_TARGET_STORAGE_MODE_COMPACT_FLASH ) {
+            } 
+            else if( handheld_dest_array[ n ]->handheld_target_storage_mode == plkrHANDHELD_TARGET_STORAGE_MODE_COMPACT_FLASH )
+            {
                 ram_or_card_string = _( "CF Card" );
-            } else {
+            } 
+            else 
+            {
                 // All others are RAM.
                 ram_or_card_string = _( "RAM" );
             }
             // Convert our is_usb_pause to the string in the column
-            if( handheld_dest_array[ n ]->is_usb_pause == 1 ) {
+            if( handheld_dest_array[ n ]->is_usb_pause == 1 )
+            {
                 is_usb_pause_string = _( "Yes" );
-            } else {
+            } 
+            else 
+            {
                 is_usb_pause_string = _( "No" ); 
             }
         // Only insert it if that user/RAM combo doesn't doesn't already exists in the
@@ -247,7 +258,8 @@ void wxHandheldInstallCtrl::InsertUserDestinations( handheld_dest_array_type& ha
         // Look to see if there is matching rows...
         m_usersListCtrl->find_matching_rows( row_cells_contents, &matching_row_numbers );
         // ...and if there isn't any matches...
-        if ( matching_row_numbers.IsEmpty() ) {
+        if ( matching_row_numbers.IsEmpty() ) 
+        {
             // ..then insert row:
             m_usersListCtrl->InsertItem( row_to_insert, handheld_dest_array[ n ]->user_name, m_usersListCtrlImageId );
             m_usersListCtrl->SetItem( row_to_insert, RAM_OR_CARD_COLUMN, ram_or_card_string );
@@ -268,12 +280,14 @@ void wxHandheldInstallCtrl::GetUserDestinations( handheld_dest_array_type& handh
 
     long selected_row_number = -1; // -1 includes first row
 
-    for ( ;; ) {
+    for ( ;; ) 
+    {
         // Default flags of all directions, and don't care the state.
         selected_row_number = m_usersListCtrl->GetNextItem( selected_row_number );
 
         // No more rows that fit the flag of being in a selected state, so break loop.
-        if ( selected_row_number == -1 ) {
+        if ( selected_row_number == -1 ) 
+        {
             wxLogDebug( "No more items fitting flags so breaking GetUserDestinations()" );
             break;
         }
@@ -284,21 +298,31 @@ void wxHandheldInstallCtrl::GetUserDestinations( handheld_dest_array_type& handh
         handheld_dest_type* handheld_dest    = new handheld_dest_type();
 
         handheld_dest->user_name = m_usersListCtrl->get_cell_contents_string( selected_row_number, USER_NAME_COLUMN );
-        if ( m_usersListCtrl->get_cell_contents_string( selected_row_number, RAM_OR_CARD_COLUMN ) == _( "RAM" ) ) {
+        if ( m_usersListCtrl->get_cell_contents_string( selected_row_number, RAM_OR_CARD_COLUMN ) == _( "RAM" ) ) 
+        {
             handheld_dest->handheld_target_storage_mode = plkrHANDHELD_TARGET_STORAGE_MODE_RAM;
-        } else if ( m_usersListCtrl->get_cell_contents_string( selected_row_number, RAM_OR_CARD_COLUMN ) == _( "SD Card" ) ) {
+        } 
+        else if ( m_usersListCtrl->get_cell_contents_string( selected_row_number, RAM_OR_CARD_COLUMN ) == _( "SD Card" ) )
+        {
             handheld_dest->handheld_target_storage_mode = plkrHANDHELD_TARGET_STORAGE_MODE_SD_CARD;
-        } else if ( m_usersListCtrl->get_cell_contents_string( selected_row_number, RAM_OR_CARD_COLUMN ) == _( "CF Card" ) ) {
+        } 
+        else if ( m_usersListCtrl->get_cell_contents_string( selected_row_number, RAM_OR_CARD_COLUMN ) == _( "CF Card" ) ) 
+        {
             handheld_dest->handheld_target_storage_mode = plkrHANDHELD_TARGET_STORAGE_MODE_COMPACT_FLASH;
-        } else {
+        } 
+        else 
+        {
             // Memory Stick
             handheld_dest->handheld_target_storage_mode = plkrHANDHELD_TARGET_STORAGE_MODE_MEMORY_STICK;
         }
         // row_directory_on_card would be similar
 
-        if ( m_usersListCtrl->get_cell_contents_string( selected_row_number, IS_USB_PAUSE_COLUMN ) == _( "Yes" ) ) {
+        if ( m_usersListCtrl->get_cell_contents_string( selected_row_number, IS_USB_PAUSE_COLUMN ) == _( "Yes" ) ) 
+        {
             handheld_dest->is_usb_pause = 1;
-        } else {
+        } 
+        else 
+        {
             handheld_dest->is_usb_pause = 0;
         }
 
@@ -314,7 +338,8 @@ void wxHandheldInstallCtrl::GetUserDestinations( handheld_dest_array_type& handh
 
 void wxHandheldInstallCtrl::InsertDirectoryDestinations( wxArrayString& directories )
 {
-    if ( directories.IsEmpty() ) {
+    if ( directories.IsEmpty() ) 
+    {
         return;
     }
     // Insert by rows, starting at index zero. Achieve by getting number
@@ -322,11 +347,13 @@ void wxHandheldInstallCtrl::InsertDirectoryDestinations( wxArrayString& director
     int rowToInsert;
     
     // Insert them
-    for ( size_t n = 0; n < directories.GetCount(); n++ ) {
+    for ( size_t n = 0; n < directories.GetCount(); n++ ) 
+    {
         rowToInsert =  m_directoriesListCtrl->GetItemCount();
         // Only insert it if the directoryname already exists in the list 
         // -1 starts from start, looks for string, and returns -1 if not found
-        if ( m_directoriesListCtrl->FindItem(-1, directories[n]) == -1 ) {
+        if ( m_directoriesListCtrl->FindItem(-1, directories[n]) == -1 )
+        {
             m_directoriesListCtrl->InsertItem(rowToInsert, directories[n], m_directoriesListCtrlImageId);
         }
     }
@@ -337,12 +364,14 @@ void wxHandheldInstallCtrl::GetDirectoryDestinations( wxArrayString& directoryDe
     long            selectedRowNumber       = -1; // -1 includes first row
     wxListItem      rowInfo;
 
-    for ( ;; ) {
+    for ( ;; )
+    {
         // Default flags of all directions, and don't care the state.
         selectedRowNumber = m_directoriesListCtrl->GetNextItem( selectedRowNumber );
                                                                 
         // No more rows that fit the flag of being in a selected state, so break loop.
-        if ( selectedRowNumber == -1 ) {
+        if ( selectedRowNumber == -1 ) 
+        {
             break; 
         }    
         rowInfo.m_itemId = selectedRowNumber;
@@ -356,7 +385,8 @@ void wxHandheldInstallCtrl::GetDirectoryDestinations( wxArrayString& directoryDe
 
 bool wxHandheldInstallCtrl::InstallFilesToDestinations( wxArrayString& install_fullnames )
 {
-    if ( install_fullnames.IsEmpty() ) {
+    if ( install_fullnames.IsEmpty() )
+    {
         return false;
     }
 
@@ -367,13 +397,15 @@ bool wxHandheldInstallCtrl::InstallFilesToDestinations( wxArrayString& install_f
     GetUserDestinations( handheld_dest_array );
     GetDirectoryDestinations( directory_destinations );    
 
-    for ( int n=0; n < (int)handheld_dest_array.GetCount(); n++ ) {
+    for ( int n=0; n < (int)handheld_dest_array.GetCount(); n++ )
+    {
         wxLogDebug( wxT( "user_array index is %d" ), n );          
         m_palmInstaller->install_files_by_handheld_dest( *handheld_dest_array.Item( n ),
                                                          install_fullnames );
     }
 
-    for ( int i=0; i < (int)directory_destinations.GetCount(); i++ ) {
+    for ( int i=0; i < (int)directory_destinations.GetCount(); i++ ) 
+    {
         wxLogDebug( wxT( "directory_array index is %d" ), i );
         m_palmInstaller->install_files_by_directory( directory_destinations.Item( i ),
                                                      install_fullnames );
@@ -401,7 +433,8 @@ bool wxHandheldInstallCtrl::IsAllDestinationsEmpty()
     GetUserDestinations( handheld_dest_array );
     GetDirectoryDestinations( sync_directories_array );
 
-    if ( handheld_dest_array.IsEmpty() && sync_directories_array.IsEmpty() ) {
+    if ( handheld_dest_array.IsEmpty() && sync_directories_array.IsEmpty() ) 
+    {
         wxLogDebug( "Both hhictrl array's returning empty." );
         is_empty = TRUE;
     }
@@ -436,16 +469,20 @@ void wxHandheldInstallCtrl::OnRemoveButtons(wxCommandEvent &event)
 {
     // Recycle this function for both listctrls, depending on which button was hit
     wxListCtrl* listCtrl;    
-    if ( event.GetId() == wxID_HANDHELDINSTALLCTRL_USERS_REMOVE_BUTTON ) {
+    if ( event.GetId() == wxID_HANDHELDINSTALLCTRL_USERS_REMOVE_BUTTON )
+    {
         listCtrl = m_usersListCtrl;
-    } else {
+    } 
+    else 
+    {
         listCtrl = m_directoriesListCtrl;
     }
 
     long  selectedRowNumber = -1;  
     wxArrayInt selectedRowNumbersArray;
         
-    for ( ;; ) {
+    for ( ;; ) 
+    {
         // for( ;; ) with this next line is the recommended way for iterating selected rows.
         // selectedRowNumber was initialized at -1 to allow inclusion of first selected
         // row.
@@ -454,7 +491,8 @@ void wxHandheldInstallCtrl::OnRemoveButtons(wxCommandEvent &event)
                                                    wxLIST_STATE_SELECTED );
                                                                 
         // No more rows that fit the flag of being in a selected state, so break loop.
-        if ( selectedRowNumber == -1 ) {
+        if ( selectedRowNumber == -1 ) 
+        {
             break; 
         }    
         // Assemble a list of indexes so we can loop though them backwards to delete them
@@ -463,8 +501,10 @@ void wxHandheldInstallCtrl::OnRemoveButtons(wxCommandEvent &event)
     
     // Loop through the row numbers in reverse, since once remove one, the rest will
     // move up, but if remove them bottom to top, then this doesn't matter.
-    if ( ! selectedRowNumbersArray.IsEmpty() ) {
-        for ( long i=(selectedRowNumbersArray.GetCount()-1); 0 <= i; i-- ) {
+    if ( ! selectedRowNumbersArray.IsEmpty() )
+    {
+        for ( long i=(selectedRowNumbersArray.GetCount()-1); 0 <= i; i-- ) 
+        {
             listCtrl->DeleteItem( selectedRowNumbersArray[i] );
         }
     }
@@ -506,7 +546,8 @@ void wxHandheldInstallCtrl::OnUsersAddButton()
         all_available_user_names );
 
     // .. and they clicked the OK box...
-    if ( a_handheld_dest_dialog.ShowModal() == wxID_OK ) {
+    if ( a_handheld_dest_dialog.ShowModal() == wxID_OK ) 
+    {
         a_handheld_dest_dialog.transfer_to( handheld_dest );
         // Insert the destinations:
         handheld_dest_array.Add( &handheld_dest );
@@ -550,7 +591,8 @@ void wxHandheldInstallCtrl::OnDirectoriesAddButton()
                               _( "Select directory in which to place the output" ),
                               "" );
   
-    if ( directoryDlg.ShowModal() == wxID_OK ) {
+    if ( directoryDlg.ShowModal() == wxID_OK )
+    {
         selectedDirectoryName.Add(directoryDlg.GetPath()); 
         InsertDirectoryDestinations(selectedDirectoryName);
     }     
