@@ -6,7 +6,7 @@
 // Created:     2001/10/20
 // Copyright:   (c) Robert O'Connor ( rob@medicalmnemonics.com )
 // Licence:     GPL
-// RCS-ID:      $Id: main_listctrl.cpp,v 1.32 2003/03/17 17:43:20 robertoconnor Exp $
+// RCS-ID:      $Id: main_listctrl.cpp,v 1.33 2004/01/04 00:57:49 robertoconnor Exp $
 //----------------------------------------------------------------------------------------
 
 //----------------------------------------------------------------------------------------
@@ -236,7 +236,8 @@ void main_listctrl::gui_add_channel()
     // that it generated.
     
     // Only do it if not an Empty String returned (ie didn't hit cancel)
-    if ( new_channel_name == wxEmptyString ) {    
+    if ( new_channel_name == wxEmptyString ) 
+    {    
         return;
     }
        
@@ -265,13 +266,17 @@ void main_listctrl::gui_add_channel_wizard()
                 &generated_channel_section, &is_launch_channel_dialog_requested );
     
     // If they clicked the finish button...
-    if ( clicked_finish_button ) {
+    if ( clicked_finish_button )
+    {
         // ...and they want to launch a channel dialog...
-        if ( is_launch_channel_dialog_requested ) {
+        if ( is_launch_channel_dialog_requested )
+        {
             // Launch a dialog for the new channel
             launch_dialog_for_new_channel( generated_channel_section );
         // ..or if they didn't want to launch a channel dialog...
-        } else {
+        } 
+        else
+        {
             // ...just insert the row
             insert_row( generated_channel_section );
         }
@@ -294,7 +299,8 @@ void main_listctrl::gui_configure_channel()
                                        wxLIST_STATE_SELECTED );
         
     // No rows that fit the flag of being in a selected state, so abort
-    if ( selected_row_number == -1 ) {
+    if ( selected_row_number == -1 ) 
+    {
         plucker_controller::get()->set_is_new_update_execution_allowed( TRUE );
         return; 
     }
@@ -308,7 +314,8 @@ void main_listctrl::gui_configure_channel()
     wxLogDebug( "Showed a channel dialog using m_parent as parent" );
     
     // Show modal, and if clicked OK, then refresh the row, which will repaint as required.
-    if ( a_channel_dialog.ShowModal() == wxID_OK ) {
+    if ( a_channel_dialog.ShowModal() == wxID_OK ) 
+    {
         refresh_row( selected_row_number );
     }            
     
@@ -322,22 +329,24 @@ void main_listctrl::gui_delete_channels( bool prompt_first )
  
     bool prompt_returned_ok = FALSE;
 
-    if ( prompt_first ) {
+    if ( prompt_first ) 
+    {
         wxMessageDialog confirm_delete_messagedialog( this,
                   _( "Are you sure you wish to permanently delete the selected channels?" ),
                   _( "Confirm channel delete" ),
                   wxYES | wxNO | wxYES_DEFAULT | wxICON_QUESTION );
 
         // Show the dialog, and if the YES button was pressed, then proceed to delete
-        if ( confirm_delete_messagedialog.ShowModal() == wxID_YES ) {
+        if ( confirm_delete_messagedialog.ShowModal() == wxID_YES ) 
+        {
             prompt_returned_ok = TRUE;  
             wxLogDebug( "Prompt: delete channels returned TRUE" );
         }      
     } 
     
     // If said okay, or not prompting first, then do the deletion.
-    if ( prompt_returned_ok || ! prompt_first ) {
-    
+    if ( prompt_returned_ok || ! prompt_first )
+    {
         wxLogDebug( "Either no prompt, or prompt returned okay, so starting delete loop" );
         
         // Make a list of the section columns that are currently selected in listctrl
@@ -354,8 +363,10 @@ void main_listctrl::gui_delete_channels( bool prompt_first )
         // Delete in reverse order (as list row numbers will slide up as each is 
         // deleted, which causes them to renumber. But if delete bottom to top, this no
         // longer is a concern).
-        if ( ! selected_row_numbers.IsEmpty() ) {
-            for ( long n = ( selected_row_numbers.GetCount() -1 ); 0 <= n; n-- ) {
+        if ( ! selected_row_numbers.IsEmpty() ) 
+        {
+            for ( long n = ( selected_row_numbers.GetCount() -1 ); 0 <= n; n-- ) 
+            {
                 // Remove it from the listctrl
                 DeleteItem( selected_row_numbers[ n ] );
                 // Remove it from our array of section names
@@ -387,9 +398,12 @@ void  main_listctrl::gui_update_selected_channels()
     spidering_display_mode = the_configuration->Read( "/PLUCKER_DESKTOP/spidering_display_mode",    
                                                       "dialog" );
 
-    if ( spidering_display_mode == "console" ) {
+    if ( spidering_display_mode == "console" ) 
+    {
         plucker_controller::get()->update_channels( channel_sections );
-    } else {
+    } 
+    else 
+    {
         build_progress_wrappers::launch_build_progress_dialog( m_parent, channel_sections );
     }
 
@@ -413,8 +427,8 @@ void  main_listctrl::gui_update_due_channels()
     plucker_controller::get()->get_due_channels( channel_sections );
 
     // Update if some are due, else show a message that none are due
-    if ( ! channel_sections.IsEmpty() ) {
-
+    if ( ! channel_sections.IsEmpty() ) 
+    {
         // Ensure that there is destinations provided for all the channels
         ensure_destinations_provided( channel_sections );
         
@@ -423,9 +437,12 @@ void  main_listctrl::gui_update_due_channels()
         wxString spidering_display_mode;
         spidering_display_mode = the_configuration->Read( "/PLUCKER_DESKTOP/spidering_display_mode",
                                                           "dialog" );
-        if ( spidering_display_mode == "console" ) {
+        if ( spidering_display_mode == "console" ) 
+        {
             plucker_controller::get()->update_due_channels();
-        } else {
+        } 
+        else 
+        {
             wxArrayString update_sections_array;
             plucker_controller::get()->get_due_channels( update_sections_array );
             build_progress_wrappers::launch_build_progress_dialog( m_parent, update_sections_array );
@@ -435,7 +452,9 @@ void  main_listctrl::gui_update_due_channels()
     // Refresh the rows. Have to update name too, in case they changed it in the
     // channel dialog when changing the destinations.
         refresh_rows( channel_sections );
-    } else {
+    } 
+    else
+    {
         wxMessageDialog no_due_messagedialog ( this,
           _( "No channels are currently due at this time." ),
           _( "All due channels are updated" ),
@@ -467,28 +486,35 @@ void main_listctrl::gui_update_all_channels()
     
     // If there is no channels, then just show a message box instead of 
     // starting up an empty progress dialog.
-    if ( update_sections_array.GetCount() == 0 ) {
+    if ( update_sections_array.GetCount() == 0 ) 
+    {
           // Make a message dialog
           wxMessageDialog no_channels_messagedialog ( this,
           _( "You currently have no channels in your list. Would you like to add a channel now?" ),
           _( "No channels to update" ),
           wxYES | wxNO | wxYES_DEFAULT | wxICON_QUESTION );        
         // Show the dialog, and if the YES button was pressed, then proceed to delete
-        if ( no_channels_messagedialog.ShowModal() == wxID_YES ) {        
+        if ( no_channels_messagedialog.ShowModal() == wxID_YES ) 
+        {        
 #if ( setupUSE_WIZARDS )        
             gui_add_channel_wizard();
 #else   //setupUSE_WIZARDS
             gui_add_channel();
 #endif  //setupUSE_WIZARDS
-        } else {
+        }
+        else 
+        {
             // Abort, to stop a progress dialog/console from occurring.
             return;
         }
     }
 
-    if ( spidering_display_mode == "console" ) {
+    if ( spidering_display_mode == "console" ) 
+    {
         plucker_controller::get()->update_all_channels();   
-    } else {    
+    } 
+    else 
+    {    
         build_progress_wrappers::launch_build_progress_dialog( m_parent, update_sections_array );     
     }
     
@@ -504,7 +530,8 @@ void main_listctrl::gui_update_all_channels()
 
 void main_listctrl::launch_dialog_for_new_channel( const wxString& new_channel_section )
 {
-    if ( new_channel_section.IsEmpty() ) {
+    if ( new_channel_section.IsEmpty() ) 
+    {
         wxLogError( "Error: can't open a channel dialog for an empty section." );
         return;
     }
@@ -519,10 +546,13 @@ void main_listctrl::launch_dialog_for_new_channel( const wxString& new_channel_s
     channel_dialog a_channel_dialog( m_parent, channel_section );
     
     // Show modal, and if clicked OK, then refresh the row, which will repaint as required.
-    if ( a_channel_dialog.ShowModal() == wxID_OK ) {    
+    if ( a_channel_dialog.ShowModal() == wxID_OK ) 
+    {    
         // insert_row also adds it to our array
         insert_row( channel_section );
-    } else {
+    } 
+    else
+    {
         // Cancel button on channel dialog was clicked
         wxLogDebug( "main_listctrl::launch_dialog_for_new_channel != wxID_OK so flushing..." );
      
@@ -578,7 +608,8 @@ void main_listctrl::load_rows()
     
     // Make a "channels" directory to hold home.html, cache and other future stuff
     // for each entry
-    if ( ! wxDirExists( get_plucker_directory( CHANNELS ) ) ) {
+    if ( ! wxDirExists( get_plucker_directory( CHANNELS ) ) ) 
+    {
         wxMkdir( get_plucker_directory( CHANNELS ), 0777 );
     }
     wxLogDebug( "Finished testing/trying to make channel directory" );
@@ -587,12 +618,14 @@ void main_listctrl::load_rows()
     
     found_section = the_configuration->GetFirstGroup( channel_section,
                                                       index );
-    while ( found_section ) {
+    while ( found_section ) 
+    {
         // Load up the each channel section (group) from the
         // plucker.ini/pluckerrc file
         wxLogDebug( "Config section=%s", channel_section.c_str() );
 
-       if ( is_channel_section( channel_section ) ) {
+       if ( is_channel_section( channel_section ) )
+       {
             // TODO: check for illegal characters in 
             // section name, and rename the section in the config file
             // as necessary before loading it into listctrl.
@@ -636,7 +669,8 @@ void main_listctrl::load_rows()
                 // If it was a filename key, then strip off path and the 
                 // extension, so only basename left
                 if ( doc_name == channel_section + "/doc_file" ||
-                     doc_name == channel_section + "/db_file" ) {
+                     doc_name == channel_section + "/db_file" ) 
+                {
                     wxSplitPath( channel_name.c_str(), NULL, &channel_name, NULL ); 
                 }
                 // Insert a new row (item in wxWindows) in the listctrl using
@@ -653,7 +687,8 @@ void main_listctrl::load_rows()
                 due_datetime   = plucker_controller::get()->get_channel_due_datetime( channel_section, FALSE );    
                 due_string     = due_datetime.Format( wxT( plkrPRETTY_COMPACT_DATE_TIME_FORMAT ) ).c_str();                   
 
-                if ( ! plucker_controller::get()->is_channel_update_enabled( channel_section ) ) {
+                if ( ! plucker_controller::get()->is_channel_update_enabled( channel_section ) )
+                {
                     due_string = _( "Never" );
                 }                                               
                 
@@ -684,7 +719,8 @@ void main_listctrl::get_selected_channel_sections( wxArrayString* channel_sectio
     
     wxLogDebug( "Entering selected channels function" );                                   
         
-    for ( ;; ) {
+    for ( ;; ) 
+    {
         // for( ;; ) with this next line is the recommended way for iterating selected rows.
         // selected_row_number was initialized at -1 to allow inclusion of first selected
         // row.
@@ -692,7 +728,8 @@ void main_listctrl::get_selected_channel_sections( wxArrayString* channel_sectio
                                            wxLIST_STATE_SELECTED );
         
         // No more rows that fit the flag of being in a selected state, so break loop.
-        if ( selected_row_number == -1 ) {
+        if ( selected_row_number == -1 ) 
+        {
             break; 
         }
         
@@ -712,16 +749,20 @@ void main_listctrl::get_selected_row_numbers( wxArrayInt* row_numbers )
     
     wxLogDebug( "Entering selected row numbers function" );                                   
         
-    for ( ;; ) {
+    for ( ;; )
+    {
         // for( ;; ) with this next line is the recommended way for iterating selected rows.
         // selected_row_number was initialized at -1 to allow inclusion of first selected
         // row.
         selected_row_number = GetNextItem( selected_row_number, wxLIST_NEXT_ALL,
                                            wxLIST_STATE_SELECTED );
         
-        if ( selected_row_number == -1 ) {
+        if ( selected_row_number == -1 ) 
+        {
             break;           
-        } else {
+        }
+        else 
+        {
             row_numbers->Add( selected_row_number );        
             wxLogDebug( "Appended row %ld to selected rows", selected_row_number );
         }            
@@ -773,17 +814,20 @@ void main_listctrl::refresh_row( long row_number )
     due_datetime = plucker_controller::get()->get_channel_due_datetime( channel_section );
     due_string   = due_datetime.Format( wxT( plkrPRETTY_COMPACT_DATE_TIME_FORMAT ) ).c_str();
 
-    if ( ! plucker_controller::get()->is_channel_update_enabled( channel_section ) ) {
+    if ( ! plucker_controller::get()->is_channel_update_enabled( channel_section ) ) 
+    {
         due_string = _( "Never" );
     }
 
     // Only repaint each if they are different from current (prevents unneeded flicker)
-    if ( channel_name != get_cell_contents_string( row_number, NAME_COLUMN ) ) {
+    if ( channel_name != get_cell_contents_string( row_number, NAME_COLUMN ) )
+    {
         SetItem( row_number, NAME_COLUMN, channel_name,
                  SMALL_IMAGE_LIST_ID_MAIN_FRAME_LISTCTRL );
     }
 
-    if ( due_string != get_cell_contents_string ( row_number, DUE_COLUMN ) ) {
+    if ( due_string != get_cell_contents_string ( row_number, DUE_COLUMN ) )
+    {
         SetItem( row_number, DUE_COLUMN, due_string );
     }
 
@@ -796,21 +840,25 @@ void main_listctrl::refresh_rows( wxArrayString& channel_sections )
     wxString    channel_section;  
     long        row_number = -1;  // '-1' needed to include the first selected row.                                 
 
-    if ( number_of_sections == 0 ) {
+    if ( number_of_sections == 0 ) 
+    {
         return;
     }     
         
-    for ( ;; ) {
+    for ( ;; ) 
+    {
         row_number = GetNextItem( row_number, wxLIST_NEXT_ALL );
         
-        if ( row_number == -1 ) {
+        if ( row_number == -1 )
+        {
             break; 
         }
         
         channel_section = get_row_channel_section( row_number );
         
         // Only proceed if the current channel section was in our array
-        if ( channel_sections.Index( channel_section ) != wxNOT_FOUND ) {                
+        if ( channel_sections.Index( channel_section ) != wxNOT_FOUND )
+        {                
             refresh_row( row_number );
         }    
     }
@@ -902,7 +950,8 @@ void main_listctrl::popup_menu( wxMouseEvent& event )
     a_menu.Append( PU_UPDATE_ALL_CHANNELS, _( "Update all channels" ) );
     
     // If no listctrl rows selected, then disable the menu items that require selection
-    if ( GetSelectedItemCount() == 0 ) {
+    if ( GetSelectedItemCount() == 0 ) 
+    {
         a_menu.Enable( PU_CONFIGURE_CHANNEL, FALSE );
         a_menu.Enable( PU_DELETE_CHANNELS, FALSE );
         a_menu.Enable( PU_UPDATE_SELECTED_CHANNELS, FALSE );
@@ -931,7 +980,8 @@ void main_listctrl::on_paste_url_as_new_channel_menuitem()
 #if ( setupUSE_DRAG_AND_DROP ) 
     wxString url_to_paste;
     // Only continue if returned successful extraction of text from clipboard
-    if ( utils_clipboard::get_text_from_clipboard( url_to_paste ) ) {
+    if ( utils_clipboard::get_text_from_clipboard( url_to_paste ) ) 
+    {
         paste_text_url_as_new_channel( url_to_paste ); 
     }   
 #endif  // setupUSE_DRAG_AND_DROP 
@@ -943,7 +993,8 @@ void main_listctrl::on_paste_fullnames_as_new_separate_channels_menuitem()
 #if ( setupUSE_DRAG_AND_DROP ) 
     wxArrayString fullname_array;
     // Only continue if returned successful extraction of fullnames from clipboard
-    if ( utils_clipboard::get_fullname_array_from_clipboard( fullname_array ) ) {
+    if ( utils_clipboard::get_fullname_array_from_clipboard( fullname_array ) )
+    {
         paste_fullnames_as_new_separate_channels( fullname_array );
     }
     fullname_array.Clear();
@@ -965,7 +1016,8 @@ void main_listctrl::paste_text_url_as_new_channel( const wxString& url )
 
     // It could well be empty, since using the "add channel" get text from user
     // call, which returns an empty string when press cancel.
-    if ( channel.doc_name != wxEmptyString ) {
+    if ( channel.doc_name != wxEmptyString ) 
+    {
         generated_channel_section   = plucker_controller::get()->add_channel( channel );           
         launch_dialog_for_new_channel( generated_channel_section );
     }    
@@ -984,13 +1036,15 @@ void main_listctrl::paste_fullnames_as_new_separate_channels( const wxArrayStrin
     size_t      current_fullname_array_index;
     wxString    current_fullname;
     
-    if ( number_of_fullnames == 0 ) {
+    if ( number_of_fullnames == 0 ) 
+    {
         return;
     } 
     
     for ( current_fullname_array_index = 0; 
           (int)current_fullname_array_index < (int)number_of_fullnames;
-          current_fullname_array_index++ ) {  
+          current_fullname_array_index++ ) 
+    {  
           // Clear the string from the previous loop iteration
           current_fullname.Empty();
           // Append file:// to the front of the file
@@ -1051,20 +1105,21 @@ bool main_listctrl::ensure_destinations_provided( const wxArrayString& channel_s
     wxString        dialog_message_string;
     wxString        channel_name;
     
-    if ( number_of_sections == 0 ) {
+    if ( number_of_sections == 0 ) 
+    {
         return FALSE;
     }
 
     // Do our loop of updating
     for ( current_section_array_index = 0;
           (int)current_section_array_index < (int)number_of_sections;
-          current_section_array_index++ ) {
-
+          current_section_array_index++ ) 
+    {
         current_section_string = channel_sections.Item( current_section_array_index );
 
         if ( ( the_configuration->Read( '/' + current_section_string + '/' + "user", "" ) == "" ) &&
-            ( the_configuration->Read( '/' + current_section_string + '/' + "copy_to_dir", "" ) == "" ) ) {
-
+            ( the_configuration->Read( '/' + current_section_string + '/' + "copy_to_dir", "" ) == "" ) ) 
+        {
             // Flick the switch to let know that at least one destination was added.
             destination_was_modified = TRUE;
 
@@ -1075,18 +1130,17 @@ bool main_listctrl::ensure_destinations_provided( const wxArrayString& channel_s
                 << channel_name << _( "\'. Add some destinations now?" );
 
             // ...and if want to show the warnings....
-            if ( the_configuration->Read( show_message_path_key, (long)default_checked ) == (long)default_checked ) {
-
+            if ( the_configuration->Read( show_message_path_key, (long)default_checked ) == (long)default_checked ) 
+            {
                 checkbox_message_dialog a_checkbox_message_dialog( this,
                     dialog_message_string,
                     _( "No destinations selected" ), wxYES | wxNO|wxCENTRE|wxICON_QUESTION,
                     wxDefaultPosition, _( "Always perform this check when about to update channels" ),
                     show_message_path_key,
                     default_checked );
-
                 // .. and they clicked the OK box in the warning...
-                if ( a_checkbox_message_dialog.ShowModal() == wxID_YES ) {
-
+                if ( a_checkbox_message_dialog.ShowModal() == wxID_YES ) 
+                {
                     // Create a channel dialog, jumping straight to the destinations tab
                     channel_dialog a_channel_dialog( m_parent, current_section_string,
                                                      CHANNEL_NOTEBOOK_PAGE_INDEX_DESTINATION );
