@@ -6,7 +6,7 @@
 // Created:     2003/05/24
 // Copyright:   (c) Robert O'Connor ( rob@medicalmnemonics.com )
 // Licence:     GPL
-// RCS-ID:      $Id: palm_installer_mac.cpp,v 1.9 2003/10/06 21:57:17 robertoconnor Exp $
+// RCS-ID:      $Id: palm_installer_mac.cpp,v 1.11 2004/02/01 17:57:07 robertoconnor Exp $
 //----------------------------------------------------------------------------------------
 
 //----------------------------------------------------------------------------------------
@@ -85,7 +85,8 @@ public:
 
     virtual wxDirTraverseResult OnDir( const wxString& directory_name )
     {
-        if ( directory_name.Contains( m_filter ) ) {
+        if ( directory_name.Contains( m_filter ) ) 
+        {
             m_subdirectories.Add( directory_name );
         }
         // Using wxDIR_STOP not wxDIR_CONTINUE, since only want one directory.
@@ -165,7 +166,8 @@ bool palm_installer::install_files_by_handheld_dest( handheld_dest_type& handhel
     wxLogDebug( wxT( "Entering palm_installer::install_files_by_handheld_dest function" ) );
 
     // If wxArrayString is empty, then abort
-    if ( install_fullnames.IsEmpty() ) {
+    if ( install_fullnames.IsEmpty() ) 
+    {
         wxLogDebug( "Error: no filenames sent to install_files_by_handheld_dest. Aborting..." );
         return false;
     }
@@ -173,13 +175,14 @@ bool palm_installer::install_files_by_handheld_dest( handheld_dest_type& handhel
     // Look up the user_index of this user_name
     user_index = get_user_index_from_user_name( handheld_dest.user_name );
 
-    if ( get_number_of_users() > 0 ) {
-
+    if ( get_number_of_users() > 0 ) 
+    {
         destination_rootpath = get_palm_desktop_path() + "/"
                                + get_user_subdirectory( user_index );
 
         // If installing to card, use that path...
-        switch (  handheld_dest.handheld_target_storage_mode ) {
+        switch (  handheld_dest.handheld_target_storage_mode )
+        {
             case plkrHANDHELD_TARGET_STORAGE_MODE_SD_CARD:
              destination_path = get_translated_handheld_destination_path( destination_rootpath,
                                                                           "SecureDigital (SD) Card",
@@ -204,12 +207,14 @@ bool palm_installer::install_files_by_handheld_dest( handheld_dest_type& handhel
 
         // If this is a brand new user, won't have an "Install" subdirectory
         // in his/her directory yet, so create it, if not there yet.
-        if ( ! wxDirExists( destination_path ) ) {
+        if ( ! wxDirExists( destination_path ) ) 
+        {
             wxLogDebug( "destination_path not exist, so making..." );
 
             // If the "Files to Install" doesn't exist, then need to make that, before can
             // make a dir such as "Memory Stick" inside of that.
-            if ( ! wxDirExists ( destination_rootpath + "/Files to Install" ) ) {
+            if ( ! wxDirExists ( destination_rootpath + "/Files to Install" ) ) 
+            {
             	wxMkdir( destination_rootpath + "/Files to Install", 0777 );
             }
 
@@ -218,15 +223,15 @@ bool palm_installer::install_files_by_handheld_dest( handheld_dest_type& handhel
             wxMkdir( destination_path, 0777 );
         }
 
-        for ( size_t n = 0; n < install_fullnames.GetCount(); n++ ) {
-
+        for ( size_t n = 0; n < install_fullnames.GetCount(); n++ )
+        {
             wxLogDebug( "Starting loop to install" );
             install_fullname = install_fullnames.Item( n );
             wxLogDebug( "install_fullname=" + install_fullname );
             install_fullname_exists = wxFileExists( install_fullname );
 
-            if ( install_fullname_exists ) {
-
+            if ( install_fullname_exists ) 
+            {
                 // Get the basename and extension, so it can have the same basename + extension
                 // in the install directory.
                 wxSplitPath( install_fullname.c_str(), NULL, &destination_basename,
@@ -236,16 +241,20 @@ bool palm_installer::install_files_by_handheld_dest( handheld_dest_type& handhel
                                        + '.' + destination_extension;
 
                 // Copy the file over.
-                if ( wxCopyFile( install_fullname, destination_fullname, TRUE ) ) {
+                if ( wxCopyFile( install_fullname, destination_fullname, TRUE ) ) 
+                {
                     successful = TRUE;
-                } else {
+                } 
+                else 
+                {
                     wxLogError( "Error: couldn't copy " + install_fullname + " to " + destination_fullname );
                 }
 
-            } else {
+            }
+            else 
+            {
                 wxLogDebug( "Error: install_fullname " + install_fullname + " doesn't exist" );
             }
-
         }
     }
 
@@ -305,30 +314,12 @@ void palm_installer::load_user_data()
     wxLogDebug( "Looking for Palm Users dat file=" + dat_fullname );
 
     // Make sure the file exists
-    if ( ! wxFileExists( dat_fullname ) ) {
-/*
-// Had to cut the checkbox message dialog, because crashed when opening config dialog
-// if it was active.
-#if ( setupUSE_CHECKBOX_MESSAGE_DIALOG )
-            bool default_checked = TRUE;
-            wxString show_message_path_key = ( "/PLUCKER_DESKTOP/message_dialog_enabled_palm_desktop" );
-            if ( the_configuration->Read( show_message_path_key, (long)default_checked ) == default_checked ) {
-                // Unfortuately using a NULL for window parent
-                checkbox_message_dialog a_checkbox_message_dialog( NULL, 
-                _( "Unable to access Palm Desktop software.\nYou will need to choose directories, not HotSync handhelds, as the destinations for channels." ),
-                _( "Palm Desktop detection warning" ), wxOK|wxCENTRE|wxICON_EXCLAMATION, 
-                wxDefaultPosition, _( "Always perform this check in the future" ),
-                show_message_path_key,
-                default_checked );
-                
-                a_checkbox_message_dialog.ShowModal();
-            }      
-#else 
-            wxLogError( _( "Unable to access Palm Desktop software.\nYou will need to choose directories, not HotSync handhelds, as the destinations for channels." ) );
-#endif     
-*/              
-            // Abort now. Can't read configuration from Registry.
-            return;
+    if ( ! wxFileExists( dat_fullname ) ) 
+    {
+        // Note: a checkbox message dialog, casuses a crash when opening config dialog
+        // if it was active. Therefore, no checkbox message dialog           
+        // Abort now. Can't read configuration from Registry.
+        return;
     }
 
     // Create the object
@@ -336,7 +327,8 @@ void palm_installer::load_user_data()
 
     // Make sure that file was opened properly. I am not going to make a checkbox removable 
     // dialog since there is almost zero chance that it won't open.
-    if ( ! dat_file.IsOpened() ) {
+    if ( ! dat_file.IsOpened() ) 
+    {
         wxLogError( "Error: unable to load file named " + dat_fullname );
         // Need to return here or else will get in trouble later.
         return;
@@ -371,15 +363,15 @@ void palm_installer::load_user_data()
     m_number_of_users = data_input.Read8();
     wxLogDebug( wxT("Read m_number_of_users=%d"), (int)m_number_of_users );
 
-    if ( 0 < m_number_of_users ) {
-
+    if ( 0 < m_number_of_users ) 
+    {
         // Fast forward through 4 unneeded bytes by reading a UInt32 
         // This is since I don't care if NUser (user) or a CUser (profile).
         unused = data_input.Read32();
 
         // Execute a loop enough times to query information about each user.
-        for ( count = 0; count < m_number_of_users; count++ ) {
-
+        for ( count = 0; count < m_number_of_users; count++ ) 
+        {
             // Add a new member to our array of users.
             // The "new" we create here, gets "delete"ed in the base class's destructor
             // as part of the WX_CLEAR_ARRAY macro
@@ -399,7 +391,8 @@ void palm_installer::load_user_data()
             string_length = data_input.Read32();
             wxLogDebug( "Read string_length=%u", string_length );
 
-            for ( int j = 0; j < string_length; j++ ) {
+            for ( int j = 0; j < string_length; j++ ) 
+            {
                 single_character = data_input.Read8();
                 m_users_array[ count ]->name += (wxChar)single_character;
             }
@@ -420,29 +413,35 @@ void palm_installer::load_user_data()
                         m_users_array[ count ]->subdirectory.c_str() );
 
             // Zip through rest of crap until find the next user entry
-            if ( count < m_number_of_users - 1 ) {
+            if ( count < m_number_of_users - 1 ) 
+            {
                 wxLogDebug( "Entering search for next user..." );
                 found_user = FALSE;
                 // A signal for the next user/profile entry is either the sequence:
                 // 0x4E 0x55 0x73 0x72 (78 85 115 114 which spells NUsr), or
 				// 0x43 0x55 0x73 0x72 (67 85 115 114 which spells CUsr)
-                while ( ! found_user ) {                 
+                while ( ! found_user ) 
+                {                 
                     // Read the next character
                     single_character = data_input.Read8();                    
                     // If the single character is 78 or 67....  
-                    if ( ( single_character == 78 ) || ( single_character == 67 ) ) {
+                    if ( ( single_character == 78 ) || ( single_character == 67 ) ) 
+                    {
                         // Read the next character
                         single_character = data_input.Read8();
                         // ...and it is followed by a 85...
-                        if ( single_character == 85 ) {
+                        if ( single_character == 85 ) 
+                        {
                             // Read the next character
                             single_character = data_input.Read8();
                             // ...and it is followed by a 85...
-                            if ( single_character == 115 ) {
+                            if ( single_character == 115 )
+                            {
                             	// Read the next character
                                 single_character = data_input.Read8();
                                 // ...and it is followed by a 85...
-                                if ( single_character == 114 ) {
+                                if ( single_character == 114 ) 
+                                {
                                     wxLogDebug( "Found next user. Restarting loop to query info" );
                                     // ...Got a user. break.   
                                     found_user = TRUE;  
@@ -475,33 +474,41 @@ wxString palm_installer::get_translated_handheld_destination_path( const wxStrin
     
     // Try the translated card directory first, but only if the translation doesn't match the 
     // untranslated one
-    if ( input_card_subdirectory != _( input_card_subdirectory ) ) {  
+    if ( input_card_subdirectory != _( input_card_subdirectory ) ) 
+    {  
         i = found_subdirectories_array.Index( input_destination_rootpath + "/Files to Install/" + _( input_card_subdirectory ) );
-        if ( i != wxNOT_FOUND ) {
+        if ( i != wxNOT_FOUND )
+        {
             output_path = found_subdirectories_array.Item( i );
         }
     }
     
     // Then try the guess. Note that whichever is the last one in the dir will be the winner      
-    if ( output_path == wxEmptyString ) {
-        for ( i=0; i < found_subdirectories_array.GetCount(); i++ ) {
+    if ( output_path == wxEmptyString ) 
+    {
+        for ( i=0; i < found_subdirectories_array.GetCount(); i++ ) 
+        {
             wxLogDebug( "subdir is=" +  found_subdirectories_array.Item( i ) );
-            if ( found_subdirectories_array.Item( i ).Contains( input_card_subdirectory_guess_filter  ) ) { 
+            if ( found_subdirectories_array.Item( i ).Contains( input_card_subdirectory_guess_filter  ) ) 
+            { 
                 output_path = found_subdirectories_array.Item( i );
             }
         }
     }
     
     // Then try the English one. Note the lack of _() around the input_card_subdirectory.
-    if ( output_path == wxEmptyString ) {
+    if ( output_path == wxEmptyString ) 
+    {
         i = found_subdirectories_array.Index( input_destination_rootpath + "/Files to Install/" + input_card_subdirectory );
-        if ( i != wxNOT_FOUND ) {
+        if ( i != wxNOT_FOUND ) 
+        {
             output_path = found_subdirectories_array.Item( i );
         }
     }
     
     // If still none, then just create one, in a translated name (which will be English if is no translation)
-    if ( output_path == wxEmptyString ) {
+    if ( output_path == wxEmptyString ) 
+    {
         output_path = input_destination_rootpath + "/Files to Install/" + _( input_card_subdirectory );
     }
     
