@@ -1,12 +1,12 @@
 //----------------------------------------------------------------------------------------
 // Name:        process_progress_dialog.cpp
-// Purpose:     Desribes process_progress_dialog
+// Purpose:     Describes process_progress_dialog
 // Author:      Robert O'Connor
 // Modified by:
 // Created:     2001/10/20
 // Copyright:   (c) Robert O'Connor ( rob@medicalmnemonics.com )
 // Licence:     GPL
-// RCS-ID:      $Id: process_progress_dialog.cpp,v 1.5 2003/03/17 17:43:20 robertoconnor Exp $
+// RCS-ID:      $Id: process_progress_dialog.cpp,v 1.9 2004/01/04 03:07:24 robertoconnor Exp $
 //----------------------------------------------------------------------------------------
 
 //----------------------------------------------------------------------------------------
@@ -132,6 +132,7 @@ process_progress_dialog::process_progress_dialog( wxWindow* parent,
     // constructor, and then call SetOwner to set it to 'this'.
     wxLogDebug( "Setting owner of m_idle_timer" );
     m_idle_timer.SetOwner( this, ID_PROCESS_PROGRESS_TIMER );
+    wxLogDebug( "Set owner of m_idle_timer" );
 #endif
 
 }
@@ -160,13 +161,15 @@ void process_progress_dialog::action_to_do_when_dialog_is_ready()
 {
     wxLogDebug( "Entering process_progress_dialog::action_to_do_when_dialog_is_ready()" );
 
-    if ( m_commandline_array.GetCount() == 0 ) {
+    if ( m_commandline_array.GetCount() == 0 )
+    {
         wxLogDebug( "process_progress_dialog::action_to_do_when_dialog_is_ready(): m_commandline_array empty. Aborting." );
         return;
     }
 
     // Abort function if a desire to close dialog was entered.
-    if ( get_abort_signal_was_entered() ) {
+    if ( get_abort_signal_was_entered() )
+    {
         return;
     }
 
@@ -174,7 +177,8 @@ void process_progress_dialog::action_to_do_when_dialog_is_ready()
     on_before_execute_commandline_array();
 
     // Abort function if a desire to close dialog was entered.
-    if ( get_abort_signal_was_entered() ) {
+    if ( get_abort_signal_was_entered() ) 
+    {
         return;
     }
 
@@ -199,7 +203,8 @@ void process_progress_dialog::action_to_do_before_dialog_finishes_closing()
 
 #if ( setupUSE_PROCESS_PROGRESS_DIALOG_IDLE_TIMER )
     // If the timer is running...
-    if ( m_idle_timer.IsRunning() ) {
+    if ( m_idle_timer.IsRunning() ) 
+    {
         // ...stop it
         wxLogDebug( "About to stop process_progress_dialog's m_idle_timer" );
         m_idle_timer.Stop();
@@ -249,8 +254,8 @@ void process_progress_dialog::detach_and_kill_all_running_processes()
     
     for ( size_t running_process_array_index = 0; 
           running_process_array_index < m_running_process_array.GetCount();
-          running_process_array_index++ ) {
-
+          running_process_array_index++ ) 
+    {
         int pid_to_kill = m_running_process_array[ running_process_array_index ]->get_process_pid();
         
         wxLogDebug( "pid to detach and kill=%d", pid_to_kill );               
@@ -317,7 +322,8 @@ void process_progress_dialog::execute_commandline_array_item( size_t commandline
     on_before_execute_commandline_array_item( m_current_commandline_array_index );
 
     // Abort fucntion if a desire to close dialog was entered.
-    if ( get_abort_signal_was_entered() ) {
+    if ( get_abort_signal_was_entered() ) 
+    {
         return;
     }
 
@@ -335,11 +341,14 @@ void process_progress_dialog::execute_commandline_array_item( size_t commandline
     int newly_created_pid = ::wxExecute( commandline, wxEXEC_ASYNC | wxEXEC_MAKE_GROUP_LEADER, a_piped_process );
 
     // The returned pid is 0 if something when wrong.
-    if ( newly_created_pid == 0 ) {
+    if ( newly_created_pid == 0 ) 
+    {
         wxLogDebug( wxT( "Execution of '%s' failed." ), commandline.c_str() );
         // Delete it.
         delete a_piped_process;
-    } else {
+    } 
+    else 
+    {
         // Add to our array of running processes.
         m_running_process_array.Add( a_piped_process );
         // Store its pid into it, so that can find it if want to kill it by pid.
@@ -360,12 +369,14 @@ void process_progress_dialog::on_idle( wxIdleEvent& event )
 
     for ( size_t running_process_array_index = 0;
           running_process_array_index < m_running_process_array.GetCount();
-          running_process_array_index++ ) {
+          running_process_array_index++ ) 
+    {
 
         // If there wasn't an abort signal entered [This is FUNDAMENTAL]....
         if ( ! get_abort_signal_was_entered() &&
             /// ...suck some process output returning whether or not there is more input queued...
-            m_running_process_array[ running_process_array_index ]->suck_process_output( FALSE ) ) {
+            m_running_process_array[ running_process_array_index ]->suck_process_output( FALSE ) )
+        {
             // ...and if there is, then suck it again.
             event.RequestMore();
         }
@@ -387,12 +398,14 @@ void process_progress_dialog::on_process_terminated( wxProcessEvent& event )
 
     for ( size_t running_process_array_index = 0; 
           running_process_array_index < m_running_process_array.GetCount();
-          running_process_array_index++ ) {   
+          running_process_array_index++ )
+    {   
                    
         // Do a while loop show the rest of the output (will stop when it returns
         // false, that ther is no more input.
         while ( m_running_process_array[ running_process_array_index ]->suck_process_output( TRUE ) )
-            ;
+        
+            ; // \todo Safe to put brackets around here?
 
         // we're not needed any more
         wxLogDebug( "process_progress_dialog::on_process_terminated: About to delete process." );
@@ -408,7 +421,8 @@ void process_progress_dialog::on_process_terminated( wxProcessEvent& event )
     
     // Abort function (and hence further process executions, since the terminate 
     // events are chained to the start of the next execution.
-    if ( get_abort_signal_was_entered() ) {
+    if ( get_abort_signal_was_entered() ) 
+    {
         return;
     }
         
@@ -418,7 +432,8 @@ void process_progress_dialog::on_process_terminated( wxProcessEvent& event )
           
     // Abort function (and hence further process executions, since the terminate
     // events are chained to the start of the next execution.
-    if ( get_abort_signal_was_entered() ) {
+    if ( get_abort_signal_was_entered() ) 
+    {
         return;
     }
 
@@ -430,7 +445,9 @@ void process_progress_dialog::on_process_terminated( wxProcessEvent& event )
         m_current_commandline_array_index++;
         // ..and then execute this commandline.
         execute_commandline_array_item( m_current_commandline_array_index );
-    } else {
+    } 
+    else 
+    {
         // The before array is done, the before process / process / after processes are
         // all done, all that is left is the stuff aftet complete the array:
         on_after_commandline_array_executions_finished();
