@@ -6,7 +6,7 @@
 // Created:     2001/10/26
 // Copyright:   (c) Robert O'Connor ( rob@medicalmnemonics.com )
 // Licence:     GPL
-// RCS-ID:      $Id: showcase_dialog.cpp,v 1.26 2003/03/17 17:43:20 robertoconnor Exp $
+// RCS-ID:      $Id: showcase_dialog.cpp,v 1.27 2004/01/04 03:07:24 robertoconnor Exp $
 //----------------------------------------------------------------------------------------
 
 //----------------------------------------------------------------------------------------
@@ -370,7 +370,8 @@ void showcase_dialog::transfer_to(
     
     info.m_itemId = selected_row_number;
     
-    for ( int i = 0; i < (int) m_showcase_field_names.GetCount(); i++ ) {
+    for ( int i = 0; i < (int) m_showcase_field_names.GetCount(); i++ )
+    {
         // Set the column of the cell to look for information
         info.m_col = i;
         // Set text mask (don't know why that is here, but seems to be always used).
@@ -666,10 +667,13 @@ void showcase_dialog::listctrl_init()
     // the pane width if split with a vertical line, otherwise stretch to the full size of
     // the splitterwindow.    
     int maximum_listctrl_width;
-    if ( m_showcase_splitterwindow->GetSplitMode() == wxSPLIT_VERTICAL ) {
+    if ( m_showcase_splitterwindow->GetSplitMode() == wxSPLIT_VERTICAL )
+    {
         maximum_listctrl_width = XRCCTRL( *this, "showcase_dialog_main_splitterwindow", wxSplitterWindow )
                                         ->GetSashPosition();
-    } else {
+    }
+    else 
+    {
         maximum_listctrl_width = m_showcase_splitterwindow->GetSize().x;
     }
     m_showcase_listctrl->set_column_widths( maximum_listctrl_width );
@@ -694,9 +698,11 @@ void showcase_dialog::load_details_htmlwindow_content( long item_id )
     
     // Get what row was selected, by looking at the event's properties
     info.m_itemId = item_id;
-    for ( int i = 0; i < (int)m_showcase_field_names.GetCount(); i++ ) {
+    for ( int i = 0; i < (int)m_showcase_field_names.GetCount(); i++ ) 
+    {
         // Check to see if the current column index is in our list from config file, of columns to show.
-        if ( columns_to_display.Index( i ) != wxNOT_FOUND ) {
+        if ( columns_to_display.Index( i ) != wxNOT_FOUND ) 
+        {
             // Set the column of the cell to look for information
             info.m_col = i;
             // Set text mask (don't know why that is here, but seems to be always used).
@@ -775,7 +781,8 @@ void showcase_dialog::load_preview_htmlwindow_content( wxString url )
                                                                           proxy );
     
     // Let user know if was a problem and then abort
-    if ( ! downloaded_successfully ) {    
+    if ( ! downloaded_successfully ) 
+    {    
         preview_htmlwindow->SetPage( _( "Channel preview unavailable." ) );
         wxLogDebug( "Unable to successfully download remote url to local file." );
         return;
@@ -789,7 +796,8 @@ void showcase_dialog::load_preview_htmlwindow_content( wxString url )
     wxString content_string;                  // A string to hold the contnets of the file.
 
     // Check to see that file exists.
-    if ( ! downloaded_file.Exists( local_filename ) ) {
+    if ( ! downloaded_file.Exists( local_filename ) )
+    {
         preview_htmlwindow->SetPage( _( "Channel preview unavailable." ) );
         wxLogDebug( "Couldn't make a file object of file '%s'", local_filename.c_str() ); 
         return;
@@ -806,7 +814,8 @@ void showcase_dialog::load_preview_htmlwindow_content( wxString url )
     
     bool show_images = the_configuration->Read( "/PLUCKER_DESKTOP/showcase_dialog_show_images", 1L );        
     
-    if ( show_images ) {
+    if ( show_images ) 
+    {
         // This is the workhorse. Replace all the href= and src= parameters with
         // their respective absolute urls [see utils_internet.cpp]. Works except 
         // mobile.theonion.com (2.3.1, cygwinb20, 256 colors) crashes program for some reason.
@@ -843,10 +852,13 @@ void showcase_dialog::on_splitter_sash_position_changed( int sash_position )
 {
     wxLogDebug( "Entering showcase_DIALOG::on_splitter_sash_position_changed" );    
         // It if is a vertical split...
-    if ( m_showcase_splitterwindow->GetSplitMode() == wxSPLIT_VERTICAL  ) {                
+    if ( m_showcase_splitterwindow->GetSplitMode() == wxSPLIT_VERTICAL ) 
+    {                
         // ...set listctrl control column widths to the sash
         m_showcase_listctrl->set_column_widths( sash_position );
-    } else {
+    } 
+    else 
+    {
         // Otherwise, set listctrl control columns to the full width of the window.  
         int listctrl_pane_width = m_showcase_splitterwindow->GetWindow1()->GetSize().x;
         m_showcase_listctrl->set_column_widths( listctrl_pane_width );
@@ -882,7 +894,8 @@ void showcase_dialog::on_listctrl_item_selected( wxListEvent& event )
 #endif
 
     // Load up the details pane, if so requested by user
-    if ( the_configuration->Read( "/PLUCKER_DESKTOP/showcase_dialog_show_details", 1L ) ) {
+    if ( the_configuration->Read( "/PLUCKER_DESKTOP/showcase_dialog_show_details", 1L ) ) 
+    {
         load_details_htmlwindow_content( item_id );
     }
     
@@ -891,7 +904,8 @@ void showcase_dialog::on_listctrl_item_selected( wxListEvent& event )
     // wxDialupManager::IsOnline, but can only have 1 object at a time, so need to 
     // coordinate that with program as a whole.
 
-    if ( the_configuration->Read( "/PLUCKER_DESKTOP/showcase_dialog_show_preview", 1L ) ) {
+    if ( the_configuration->Read( "/PLUCKER_DESKTOP/showcase_dialog_show_preview", 1L ) ) 
+    {
         wxListItem info;        // Stores the properties to both find a cell in the
                                 // listctrl, or to get the cell properties out of that cell
     
@@ -917,9 +931,12 @@ void showcase_dialog::OnOK( wxCommandEvent& event )
     long selected_row_number = m_showcase_listctrl->GetNextItem( -1, wxLIST_NEXT_ALL, 
                                                           wxLIST_STATE_SELECTED );
     // If no selected row, then end as a cancel. If a selected, then end with an OK..
-    if ( selected_row_number == -1 ) {
+    if ( selected_row_number == -1 )
+    {
         EndModal( wxID_CANCEL ); 
-    } else {
+    } 
+    else 
+    {
         EndModal( wxID_OK );        
     }
 }
@@ -928,13 +945,17 @@ void showcase_dialog::OnOK( wxCommandEvent& event )
 void showcase_dialog::OnSize( wxSizeEvent &event )
 {
     /// I don't want this stuff to fire, if dialog isn't shown, just in case.
-    if ( IsShown() ) {
+    if ( IsShown() ) 
+    {
         wxLogDebug( "Entering showcase_DIALOG::OnSize" );    
             // It if is a vertical split...
-        if ( m_showcase_splitterwindow->GetSplitMode() == wxSPLIT_VERTICAL  ) {                
+        if ( m_showcase_splitterwindow->GetSplitMode() == wxSPLIT_VERTICAL ) 
+        {                
             // ...set listctrl control column widths to the sash
             m_showcase_listctrl->set_column_widths( m_showcase_splitterwindow->GetSashPosition() );
-        } else {
+        } 
+        else 
+        {
             // Otherwise, set listctrl control columns to the full width of the window.  
             int listctrl_pane_width = m_showcase_splitterwindow->GetWindow1()->GetSize().x;
             m_showcase_listctrl->set_column_widths( listctrl_pane_width );
