@@ -6,7 +6,7 @@
 // Created:     2001/10/20
 // Copyright:   (c) Robert O'Connor ( rob@medicalmnemonics.com )
 // Licence:     GPL
-// RCS-ID:      $Id: utils_controls.cpp,v 1.16 2003/06/18 03:00:40 robertoconnor Exp $
+// RCS-ID:      $Id: utils_controls.cpp,v 1.17 2004/01/04 03:07:24 robertoconnor Exp $
 //----------------------------------------------------------------------------------------
 
 //----------------------------------------------------------------------------------------
@@ -72,7 +72,8 @@
 bool utils_controls::splitterwindow_fix_sash_out_of_range( wxSplitterWindow* splitterwindow )
 {   
     // If its not split, impossible to be out of range, so abort
-    if ( ! splitterwindow->IsSplit() ) {
+    if ( ! splitterwindow->IsSplit() ) 
+    {
         wxLogDebug( "Window not split, so sash fix aborting." );
         return FALSE;
     }   
@@ -84,9 +85,12 @@ bool utils_controls::splitterwindow_fix_sash_out_of_range( wxSplitterWindow* spl
     int split_mode = splitterwindow->GetSplitMode();
     
     int window_maximum;
-    if ( split_mode == wxSPLIT_VERTICAL ) {
+    if ( split_mode == wxSPLIT_VERTICAL ) 
+    {
         window_maximum = ctrl_width;
-    } else {
+    } 
+    else
+    {
         window_maximum = ctrl_height;
     }
     
@@ -94,11 +98,14 @@ bool utils_controls::splitterwindow_fix_sash_out_of_range( wxSplitterWindow* spl
     // Minimum pane size is zero if was never set.
     int minimum_pane_size = splitterwindow->GetMinimumPaneSize();
     // Check to see if the sash position is legal, ie, within the minimum pane size.    
-    if ( sash_position < ( window_maximum - minimum_pane_size ) ) {
+    if ( sash_position < ( window_maximum - minimum_pane_size ) )
+    {
         // It's okay, so abort and return FALSE.
         wxLogDebug( "Sash within allowable parameters, so sash fix aborting." );
         return FALSE;
-    } else {
+    } 
+    else 
+    {
         // Ilegal, so set the sash back so its within a minimum pane width.
         // TRUE redraws the sash.
         splitterwindow->SetSashPosition( ( window_maximum - minimum_pane_size ), TRUE );
@@ -123,11 +130,14 @@ bool utils_controls::textctrl_hex_color_selection( wxTextCtrl* target_color_text
     // 1. Extract the old hex value
     wxString old_hex_color_string = target_color_textctrl->GetValue();    
     // 2. Check to see if valid
-    if ( utils_string::is_valid_hex_color_string( old_hex_color_string ) ) {
+    if ( utils_string::is_valid_hex_color_string( old_hex_color_string ) ) 
+    {
         wxLogDebug( "Valid old_hex_color_string=" + old_hex_color_string );
         // Convert textboxes previous color from its hex value [utils_string.cpp]
         col = utils_string::hex_string_to_color( old_hex_color_string );
-    } else {
+    } 
+    else 
+    {
         // MSW's Cygwin,BCC (at least) need to have an initialization to something other
         // than black in order to get the matrix part of the dialog going properly.
         col = wxColour( "MEDIUM BLUE" );
@@ -145,7 +155,8 @@ bool utils_controls::textctrl_hex_color_selection( wxTextCtrl* target_color_text
     // Set the title
     color_dialog.SetTitle( _( "Choose the colour" ) );
     // Only do the action if returned hitting an OK button.
-    if ( color_dialog.ShowModal() == wxID_OK ) {
+    if ( color_dialog.ShowModal() == wxID_OK ) 
+    {
         wxColourData returned_color_data = color_dialog.GetColourData();
         col = returned_color_data.GetColour();
         // Convert to hex [utils_string.cpp]
@@ -177,7 +188,8 @@ bool utils_controls::textctrl_filename_selection
 {
     // Extract the old filename value to override default file and default directory
     wxString old_filename_string = target_filename_textctrl->GetValue();   
-    if ( allow_multiple == TRUE ) {
+    if ( allow_multiple == TRUE ) 
+    {
         // Can only use the first entry to set dir/file.
         old_filename_string = old_filename_string.BeforeFirst ( multiple_selection_delimiter );
     }
@@ -188,11 +200,16 @@ bool utils_controls::textctrl_filename_selection
     // Look for/strip off a "file://" first, then if not one, then look for/strip off 
     // a "file:" prefix next. If no file: either, then keep going. Keep the 
     // file_protocol_prefix so we can tack it on again at the end.
-    if ( uses_file_protocol ) {
-        if ( old_filename_string.StartsWith( wxT( "file://" ), &old_filename_string ) ) {
+    if ( uses_file_protocol ) 
+    {
+        if ( old_filename_string.StartsWith( wxT( "file://" ), &old_filename_string ) ) 
+        {
             file_protocol_prefix = wxT( "file://" );
-        } else {
-            if ( old_filename_string.StartsWith( wxT( "file:" ), &old_filename_string ) ) {
+        } 
+        else 
+        {
+            if ( old_filename_string.StartsWith( wxT( "file:" ), &old_filename_string ) )
+            {
                 file_protocol_prefix = wxT( "file:" );
             }
         }    
@@ -204,17 +221,20 @@ bool utils_controls::textctrl_filename_selection
     wxString old_extension;
     
     // Must check for non-null string before calling wxSplitPath
-    if ( old_filename_string != "" ) {
+    if ( old_filename_string != "" )
+    {
         wxSplitPath( old_filename_string, &old_directory, &old_base, &old_extension );
     }
     
     // If there was a directory part, make that default directory
-    if ( old_directory != "" ) {
+    if ( old_directory != "" ) 
+    {
         default_directory = old_directory;
     }
     
     // If was a base, make that default filename
-    if ( old_base != "" ) {
+    if ( old_base != "" ) 
+    {
         default_file = old_base;
         // If was an extension, add a '.' and the extension to filename
         if ( old_extension != "" ) {
@@ -236,15 +256,18 @@ bool utils_controls::textctrl_filename_selection
                               wildcard );
 
     // Set multiple style if multiple
-    if ( allow_multiple == TRUE ) {
+    if ( allow_multiple == TRUE )
+    {
         a_file_dialog.SetStyle( wxMULTIPLE );
     }    
                                   
     wxString output_string;
     // Show the filename dialog modally, and only do the action if returned
     // hitting an OK button.
-    if ( a_file_dialog.ShowModal() == wxID_OK ) {
-        if ( allow_multiple == TRUE ) {
+    if ( a_file_dialog.ShowModal() == wxID_OK )
+    {
+        if ( allow_multiple == TRUE )
+        {
             // Get the array of selections, and loop through writing to output_string 
             // and a delimiter at the end of each.
             wxArrayString selected_paths;
@@ -255,12 +278,15 @@ bool utils_controls::textctrl_filename_selection
             selected_paths.Clear(); 
             // Remove the last delimiter
             output_string.RemoveLast();                 
-        } else {
+        } 
+        else 
+        {
             output_string = a_file_dialog.GetPath();
         }
         
         // If flag set to load/save file:// extension, then bolt it back on.
-        if ( uses_file_protocol ) {
+        if ( uses_file_protocol ) 
+        {
             output_string = file_protocol_prefix + output_string;
         }
         
@@ -293,7 +319,8 @@ bool utils_controls::textctrl_directory_selection
 {
     // Extract the old filename value to override default file and default directory
     wxString old_directory_string = target_filename_textctrl->GetValue();   
-    if ( allow_multiple == TRUE ) {
+    if ( allow_multiple == TRUE ) 
+    {
         // Can only use the first entry to set the directory
         old_directory_string = old_directory_string.BeforeFirst ( multiple_selection_delimiter );
     }
@@ -302,7 +329,8 @@ bool utils_controls::textctrl_directory_selection
     // If there was a directory in the text control, make that default directory,
     // otherwise we will be using the default directory that we were passed (to be used
     // when there is no directory currently in the textctrl.
-    if ( old_directory_string != "" ) {
+    if ( old_directory_string != "" ) 
+    {
         default_directory = old_directory_string;
     }
 
@@ -313,7 +341,8 @@ bool utils_controls::textctrl_directory_selection
     wxString output_string;
     // Show the filename dialog modally, and only do the action if returned
     // hitting an OK button.
-    if ( directory_dialog.ShowModal() == wxID_OK ) {
+    if ( directory_dialog.ShowModal() == wxID_OK ) 
+    {
         output_string = directory_dialog.GetPath();
         target_filename_textctrl->SetValue( output_string );
     }
@@ -337,14 +366,15 @@ void utils_controls::set_html_window_text_in_gui_style( wxHtmlWindow* target_htm
 
     // Set padding from htmlwindow edges to it contents to be zero, so looks same as
     // a normal control.    
-    if ( options & optionSET_HTML_WINDOW_INTERNAL_PADDING_TO_ZERO ) {    
-    
+    if ( options & optionSET_HTML_WINDOW_INTERNAL_PADDING_TO_ZERO )
+    {    
         target_html_window->SetBorders( 0 );
     
     }
  
     // Set the font face and size of the HTML window to match the GUI's font.
-    if ( options & optionSET_HTML_FONT_FACE_AND_POINTSIZE_TO_MATCH_GUI ) { 
+    if ( options & optionSET_HTML_FONT_FACE_AND_POINTSIZE_TO_MATCH_GUI )
+    { 
         // Get face and point size of the font used for GUI controls.
         wxFont system_gui_font( *wxNORMAL_FONT ); 
         system_gui_font = wxSystemSettings::GetFont( wxSYS_DEFAULT_GUI_FONT );
@@ -384,8 +414,8 @@ void utils_controls::set_html_window_text_in_gui_style( wxHtmlWindow* target_htm
     } 
     
         
-    if ( options & optionSET_HTML_FONT_COLOR_TO_MATCH_GUI ) {
-        
+    if ( options & optionSET_HTML_FONT_COLOR_TO_MATCH_GUI ) 
+    {    
         // Look up the system's color for text in windows. Use this for the text.
         wxColor  system_window_text_color            = wxSystemSettings::GetColour( wxSYS_COLOUR_WINDOWTEXT );
         // Convert to a rgb string
@@ -410,12 +440,15 @@ void utils_controls::set_html_window_text_in_gui_style( wxHtmlWindow* target_htm
     // resets it back to white), so include teh background color inside the body tag,
     // since we are doing the text anyways.
     if ( options & optionSET_HTML_BACKGROUND_TO_MATCH_GUI_PANEL  ||
-         options & optionSET_HTML_BACKGROUND_TO_MATCH_GUI_LISTBOX ) {
-
+         options & optionSET_HTML_BACKGROUND_TO_MATCH_GUI_LISTBOX ) 
+    {
         wxColor desired_background_color;
-        if ( options & optionSET_HTML_BACKGROUND_TO_MATCH_GUI_PANEL ) {
+        if ( options & optionSET_HTML_BACKGROUND_TO_MATCH_GUI_PANEL )
+        {
             desired_background_color = wxSystemSettings::GetSystemColour( wxSYS_COLOUR_BTNFACE );
-        } else {
+        } 
+        else 
+        {
             desired_background_color = wxSystemSettings::GetSystemColour( wxSYS_COLOUR_LISTBOX );
         }
 
@@ -515,7 +548,8 @@ void utils_controls::read_dialog_position_and_size( wxWindow* target_window,
 
     // Now if the new position would be to the left of the width of the viewable area,
     // or below the viewable area...
-    if ( ( screen_lower_left.x < new_position.x ) || ( screen_lower_left.y < new_position.y ) ) {
+    if ( ( screen_lower_left.x < new_position.x ) || ( screen_lower_left.y < new_position.y ) ) 
+    {
         // ...then wind back to the defaults.
         new_position = default_position;
         new_size     = default_size;
@@ -527,7 +561,8 @@ void utils_controls::read_dialog_position_and_size( wxWindow* target_window,
     // Different user-defined behaviour of how want the windows to behave.
     long window_placement_type= the_configuration->Read( "/PLUCKER_DESKTOP/window_placement_type", 0L );
 
-    switch ( window_placement_type ) {
+    switch ( window_placement_type ) 
+    {
         case plkrWINDOW_PLACEMENT_STYLE_CENTER_ON_PARENT:
             target_window->CenterOnParent();
             break;
@@ -577,30 +612,33 @@ void utils_controls::strip_excess_separators_from_menubar( wxMenuBar* target_men
     // The current menuitem, as extracted from the node.
     wxMenuItem*			  current_menuitem;
 
-    for ( current_menu_index = 0; current_menu_index < target_menubar->GetMenuCount(); current_menu_index++ ) {
+    for ( current_menu_index = 0; current_menu_index < target_menubar->GetMenuCount(); current_menu_index++ ) 
+    {
         // Get the current menu.
         current_menu = target_menubar->GetMenu( current_menu_index );
         // Loop through the menuitemnodes on this menu, from bottom to top. Going bottom
         // to top as the menuitem index numbers will shift to smaller numbers as menuitems are deleted.
         wxLogDebug( "Number of menuitems in current_menu is %d", (int)current_menu->GetMenuItemCount() );
         current_menuitem_node = current_menu->GetMenuItems().GetLast();
-        while ( current_menuitem_node ) {
+        while ( current_menuitem_node ) 
+        {
             current_menuitem = current_menuitem_node->GetData();
             // If a separator...
-            if ( current_menuitem->IsSeparator() ) {
+            if ( current_menuitem->IsSeparator() ) 
+            {
                 // ..and it is....
                 // ...the last menuitem in the menu....
                 if ( ! current_menuitem_node->GetNext() 
                     // ...or the first menuitem in the menu...
                     || ! current_menuitem_node->GetPrevious()
                     // ...or the menuitem prior to it in the menu is also a separator... 
-                    || current_menuitem_node->GetPrevious()->GetData()->IsSeparator() ) {
+                    || current_menuitem_node->GetPrevious()->GetData()->IsSeparator() ) 
+                {
                     // ...then delete it.
                     current_menu->Delete( current_menuitem );    
                 }
             }
             current_menuitem_node = current_menuitem_node->GetPrevious();
         }
-     }
-     
+     }     
 }
