@@ -6,7 +6,7 @@
 // Created:     2001/10/20
 // Copyright:   (c) Robert O'Connor ( rob@medicalmnemonics.com )
 // Licence:     GPL
-// RCS-ID:      $Id: viewer_wizardpage.cpp,v 1.7 2003/06/18 03:00:40 robertoconnor Exp $
+// RCS-ID:      $Id: viewer_wizardpage.cpp,v 1.11 2004/03/30 06:51:19 robertoconnor Exp $
 //----------------------------------------------------------------------------------------
 
 //----------------------------------------------------------------------------------------
@@ -74,12 +74,16 @@ END_EVENT_TABLE()
 // Internal constants
 //----------------------------------------------------------------------------------------
 
+/*
+const wxString viewer_lores_basename = the_configuration->Read("/PLUCKER_DESKTOP/viewer_lores_basename", "_");
+const wxString viewer_hires_basename = the_configuration->Read("/PLUCKER_DESKTOP/viewer_hires_basename", "hires_");
+
 // The name of the prc file before the iso code and the file extension.
 // Note, the get_is_hires_device() selected looks for hires in the filename here. 
-const wxString device_prefix_basename_array[] = { wxT( "viewer_" ),
-                                                  wxT( "viewer_hires_" )
+const wxString device_prefix_basename_array[] = { viewer_lores_basename,
+                                                  viewer_hires_basename
                                                 };
-                                                
+*/                                                
 const wxString bpp_key_array[] = { wxT( "1bpp" ),
                                    wxT( "2bpp" ),
                                    wxT( "4bpp" ),
@@ -87,7 +91,8 @@ const wxString bpp_key_array[] = { wxT( "1bpp" ),
                                    wxT( "16bpp" )
                                  };
 
-const wxString viewer_language_iso_codes_array[] = { wxT( "cz" ),
+const wxString viewer_language_iso_codes_array[] = { wxT( "ca" ),
+													 wxT( "cz" ),
                                                      wxT( "da" ),
                                                      wxT( "de" ),
                                                      wxT( "en" ),
@@ -135,7 +140,8 @@ viewer_wizardpage::viewer_wizardpage( wxWizard *parent,
                                              };
 
     // The order of languages and their iso-codes obviously have to match
-    const wxString viewer_languages_descriptive_array[] = { _( "Czech" ),
+    const wxString viewer_languages_descriptive_array[] = { _( "Catalan" ),
+															_( "Czech" ),
                                                             _( "Danish" ),
                                                             _( "German" ),
                                                             _( "English" ),
@@ -256,12 +262,12 @@ viewer_wizardpage::~viewer_wizardpage()
 void viewer_wizardpage::wizardpage_initalize()
 {
     // Set up the initial value of the controls
-    m_device_choice->SetSelection( 0 );
+    m_device_choice->SetSelection( 1 );
     // \todo need better ways of calling the defaults here instead of just "3".
     // I think that it is okay to set a selection beyond the range (for example if
     // just 2 entries, okay to set selection to the translated English.
     m_language_choice->SetStringSelection( _( "English" ) );
-    m_bpp_choice->SetSelection( 2 );
+    m_bpp_choice->SetSelection( 4 );
     m_install_viewer_checkbox->SetValue( TRUE );
     m_install_zlib_checkbox->SetValue( TRUE );
     m_install_manual_checkbox->SetValue( TRUE );
@@ -277,25 +283,41 @@ void viewer_wizardpage::get_fullnames_to_install( wxArrayString* fullnames_to_in
     wxString        zlib_fullname;
     wxString        userguide_fullname;
 
+	wxString viewer_lores_basename = the_configuration->Read("/PLUCKER_DESKTOP/viewer_lores_basename", "viewer_");
+	wxString viewer_hires_basename = the_configuration->Read("/PLUCKER_DESKTOP/viewer_hires_basename", "viewer_hires_");
+
+	// The name of the prc file before the iso code and the file extension.
+	// Note, the get_is_hires_viewer_selected() function looks for 2nd viewer in the dropdown. 
+	const wxString device_prefix_basename_array[] = { 
+													  viewer_lores_basename,
+			                                          viewer_hires_basename
+													};
+	
     language_index      = m_language_choice->GetSelection();
     device_index        = m_device_choice->GetSelection();
 
-    if ( device_index != -1 ) {
+    if ( device_index != -1 )
+    {
         viewer_fullname     = get_plucker_directory( HANDHELD_VIEWER_SOFTWARE ) + '/'
                               + device_prefix_basename_array[ device_index ]
                               + viewer_language_iso_codes_array[ language_index ]
                               + ".prc";
         wxLogDebug( "viewer_wizardpage: viewer_fullname=" + viewer_fullname );
-    } else {
+    }
+    else
+    {
         wxLogDebug( "Error: no viewers on system. Can't install" );
     }
 
 #if defined(__WXMSW__) || defined(__WXMAC__)
     // We install a SysZLib_hires.prc on Mac and MSW bundles, if needed but not on GTK yet.
-    if ( get_is_hires_viewer_selected() ) {
+    if ( get_is_hires_viewer_selected() ) 
+    {
         zlib_fullname     = get_plucker_directory( HANDHELD_VIEWER_SOFTWARE ) + '/'
                           + "SysZLib_hires.prc";        
-    } else {
+    } 
+    else 
+    {
         zlib_fullname     = get_plucker_directory( HANDHELD_VIEWER_SOFTWARE ) + '/'
                           + "SysZLib.prc";
     }
@@ -310,14 +332,17 @@ void viewer_wizardpage::get_fullnames_to_install( wxArrayString* fullnames_to_in
                           + "PluckerUserGuide.pdb";
     wxLogDebug( "viewer_wizardpage: userguid_fullname=" + userguide_fullname );
 
-    if ( m_install_viewer_checkbox->GetValue() == TRUE ) {
+    if ( m_install_viewer_checkbox->GetValue() == TRUE )
+    {
         fullnames_to_install->Add( viewer_fullname );
     }
 
-    if ( m_install_zlib_checkbox->GetValue() == TRUE ) {
+    if ( m_install_zlib_checkbox->GetValue() == TRUE ) 
+    {
         fullnames_to_install->Add( zlib_fullname );
     }
-    if ( m_install_manual_checkbox->GetValue() == TRUE ) {
+    if ( m_install_manual_checkbox->GetValue() == TRUE )
+    {
         fullnames_to_install->Add( userguide_fullname );
     }
 }
@@ -327,7 +352,8 @@ long viewer_wizardpage::get_bpp()
 {
     long bpp = 0;
 
-    switch ( m_bpp_choice->GetSelection() ) {
+    switch ( m_bpp_choice->GetSelection() ) 
+    {
         case 0:
             bpp = 1;
             break;
@@ -352,15 +378,13 @@ bool viewer_wizardpage::get_is_hires_viewer_selected()
 {
 	bool is_hires_viewer_selected = FALSE;
     int device_index;
-    wxString device_basename;
     
     device_index  = m_device_choice->GetSelection();
     
-    if ( device_index != -1 ) {
-        device_basename = device_prefix_basename_array[ device_index ];
-        if ( device_basename.Contains( wxT( "hires" ) ) ) {
-            is_hires_viewer_selected = TRUE;
-        }
+	// device_index == 1 is the hires viewer.
+    if ( device_index == 1 ) 
+    {
+		is_hires_viewer_selected = TRUE;
     }
     
     return is_hires_viewer_selected;
