@@ -1,14 +1,14 @@
 /*
- * $Id: categoryform.c,v 1.31 2003/09/21 20:11:28 prussar Exp $
+ * $Id: categoryform.c,v 1.35 2004/05/08 09:04:53 nordstrom Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -26,6 +26,7 @@
 #include "resourceids.h"
 #include "util.h"
 #include "doclist.h"
+#include "DIA.h"
 
 #include "categoryform.h"
 
@@ -233,12 +234,30 @@ Boolean CategoryNameFormHandleEvent
             handled = true;
             break;
 
+        case winEnterEvent:
+            handled = ResizeHandleWinEnterEvent();
+            break;
+
+        case winDisplayChangedEvent:
+            handled = ResizeHandleWinDisplayChangedEvent();
+            break;
+
+        case winExitEvent:
+            handled = ResizeHandleWinExitEvent();
+            break;
+
         case frmOpenEvent:
+#ifdef HAVE_SILKSCREEN
+            ResizeHandleFrmOpenEvent();
+#endif
             CategoryNameFormInit();
             handled = true;
             break;
 
         case frmCloseEvent:
+#ifdef HAVE_SILKSCREEN
+            ResizeHandleFrmCloseEvent();
+#endif
             handled = false;
             break;
 
@@ -760,10 +779,28 @@ Boolean CategoryFormHandleEvent
             break;
 
         case frmCloseEvent:
+#ifdef HAVE_SILKSCREEN
+            ResizeHandleFrmCloseEvent();
+#endif
             handled = false;
             break;
 
+        case winEnterEvent:
+            handled = ResizeHandleWinEnterEvent();
+            break;
+
+        case winDisplayChangedEvent:
+            handled = ResizeHandleWinDisplayChangedEvent();
+            break;
+
+        case winExitEvent:
+            handled = ResizeHandleWinExitEvent();
+            break;
+
         case frmOpenEvent:
+#ifdef HAVE_SILKSCREEN
+            ResizeHandleFrmOpenEvent();
+#endif
             CategoryFormInit();
             handled = true;
             break;
