@@ -1,4 +1,4 @@
-/* $Id: resourceids.h,v 1.2 2003/08/01 11:56:14 prussar Exp $ */
+/* $Id: resourceids.h,v 1.4 2003/12/13 14:21:26 prussar Exp $ */
 
 #define frmMain             1000
 #define frmPrefsButton      1001
@@ -18,6 +18,11 @@
 #define frmDocListCleared   1016
 #define frmDone             1017
 #define frmWhatIsThis       1018
+#define frmFontCacheRecords 1019
+#define frmFontCacheSize    1020
+#define frmFontCacheButton  1021
+#define frmFontCacheNotFound 1022
+#define frmFontCacheCleared  1023
 
 #define frmAbout            1100
 #define frmAboutOK          1101
@@ -27,7 +32,8 @@
 #define confirmPrefs        3000
 #define confirmTemp         3001
 #define confirmDocList      3002
-#define errCannotDelete     3003
+#define confirmFontCache    3003
+#define errCannotDelete     3004
 
 #define menuMain            4000
 #define menuHelpWhatIsThis  4001
