@@ -1,5 +1,5 @@
 /*
- * $Id: doclist.c,v 1.24 2003/09/24 13:06:43 prussar Exp $
+ * $Id: doclist.c,v 1.31 2004/03/06 19:37:48 nordstrom Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -32,6 +32,7 @@
 #include "genericfile.h"
 #include "libraryform.h"
 #include "document.h"
+#include "font.h"
 
 #include "doclist.h"
 
@@ -221,10 +222,11 @@ static void ReleaseDeactivateList( void )
 static void UpdateDocList( void )
     /* THROWS */
 {
-    /* no automatic update if user has selected to handle it manually */
-    if ( Prefs()->syncPolicy == SYNC_MANUAL )
+    /* no automatic update if user has selected to handle it manually
+       or if we have a low power situation */
+    if ( Prefs()->syncPolicy == SYNC_MANUAL || ! HasEnoughPower() )
         return;
-
+        
     /* create list for documents that have been deleted since the last
        session */
     CreateRemoveList();
@@ -234,8 +236,18 @@ static void UpdateDocList( void )
         EnumerateRAM();
 
         /* ... and on mounted external cards */
-        if ( Prefs()->syncPolicy != SYNC_IGNORE_CARD )
-            EnumerateCards();
+        if ( Prefs()->syncPolicy != SYNC_IGNORE_CARD ) {
+#ifdef SUPPORT_VFS_FONTS
+            if ( ! ScannedVFSFonts() ) {
+                EnumerateCards( ENUMERATECARD_DOCS | ENUMERATECARD_FONTS );
+                PostprocessVFSFonts();
+            }
+            else
+#endif
+            {
+                EnumerateCards( ENUMERATECARD_DOCS );
+            }
+        }
 
         /* clean up */
         DeactivateDocuments();
@@ -271,7 +283,7 @@ static void InitPlkrAppInfo
             NULL, NULL, &appInfoID, NULL, NULL, NULL );
     THROW_IF( err != errNone, err );
 
-    if ( appInfoID == NULL ) {
+    if ( appInfoID == 0 ) {
         handle = DmNewHandle( docRef, sizeof *appInfoP );
         THROW_IF( handle == NULL, dmErrMemError );
 
@@ -862,7 +874,7 @@ void OpenDocList( void )
                     StrCat( metaName, "-" );
                     StrNCat( metaName, dataPtr->name, dmDBNameLength - 6 );
                     metaID = DmFindDatabase( cardNo, metaName );
-                    if ( metaID != NULL ) {
+                    if ( metaID != 0 ) {
                         timestamp = TimGetSeconds();
                     }
                     else {
