@@ -1,5 +1,5 @@
 /*
- * $Id: document.c,v 1.131.4.7 2003/11/03 15:19:18 prussar Exp $
+ * $Id: document.c,v 1.174 2004/04/18 15:34:47 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -34,7 +34,7 @@
 #include "prefsdata.h"
 #include "resourceids.h"
 #include "screen.h"
-#include "silkscreen.h"
+#include "handera.h"
 #include "uncompress.h"
 #include "util.h"
 #include "metadocument.h"
@@ -44,6 +44,7 @@
 #include "table.h"
 #include "fullscreenform.h"
 #include "rotate.h"
+#include "control.h"
 
 #include "document.h"
 
@@ -52,8 +53,9 @@
  *      Internal Constants
  *
  ***********************************************************************/
-#define ESTIMATE_RESOLUTION         50
-
+#define ESTIMATE_RESOLUTION     50
+#define BACKWARDS               true
+#define CONTINUATION            true
 
 
 /***********************************************************************
@@ -72,13 +74,14 @@ static void SetIdle( void ) DOCUMENT_SECTION;
 static void ShowVerticalOffset( void ) DOCUMENT_SECTION;
 static void SetFirstVisibleParagraph( const Int16 paragraph,
                 const YOffset y ) DOCUMENT_SECTION;
-static void InitializeTextContext( TextContext* tContext ) DOCUMENT_SECTION;
+static void InitializeTextContext( TextContext* tContext,
+                const WriteModeType mode ) DOCUMENT_SECTION;
 static void AdjustVerticalOffset( MemHandle recordHandle,
                 YOffset adjustment ) DOCUMENT_SECTION;
 static YOffset CalculatePosition( Header* record, Int16 pOffset,
                 Int16 characterPosition, YOffset* verticalOffsetPtr,
                 Boolean recalculateHeights, Coord yPosition,
-                Boolean backwards, Boolean draw )
+                Boolean backwards, WriteModeType mode )
                 DOCUMENT_SECTION;
 static Boolean ViewPHTML( Header* record, Boolean newPage,
                 Int16 pOffset, Int16 cOffset, Coord yPosition,
@@ -94,8 +97,6 @@ static void ActivateSavedBounds( void ) DOCUMENT_SECTION;
 static void RenderRecord( Header* record, WriteModeType mode,
             Boolean continuation, RectangleType* clipRect ) DOCUMENT_SECTION;
 static void EstimateHeights( void ) DOCUMENT_SECTION;
-static MemHandle GetSequentialRecordHandle ( UInt16* recordNumPtr,
-            DirectionType direction ) DOCUMENT_SECTION;
 
 
 /***********************************************************************
@@ -113,13 +114,17 @@ static RectangleType    savedExtents;
 static RectangleType    savedViewportBounds;
 static RectangleType    oldViewportBounds;
 static Char             offsetFormat[ 5 ];
-static Int32          scrollbarScale;
+static Int32            scrollbarScale;
 static YOffset          trueMaxScrollbar;
 static Int16            scaledMaxScrollbar;
 static UInt16           currentRecordId = NO_RECORD;
 #ifdef HAVE_ROTATE
 static DirectionType    scrollbarDirection;
 #endif
+#ifdef SUPPORT_WORD_LOOKUP
+static Coord            findPixelX = -1;
+static Coord            findPixelY = -1;
+#endif
 
 #ifdef ENABLE_SCROLL_TO_BOTTOM
 static Boolean          leftOff = false;
@@ -132,6 +137,17 @@ static Int16 (*OldExtentX)( void );
 static Int16 (*OldExtentY)( void );
 static UInt16 oldCoordSys;
 
+static Boolean haveLastUncompressedRecordId;
+static UInt16  lastUncompressedRecordId;
+
+
+
+/* Call whenever the last uncompressed PHTML record might be overwritten */
+void ResetLastUncompressedPHTML( void )
+{
+    haveLastUncompressedRecordId = false;
+}
+
 
 
 /* Initialize format of percentage indicator */
@@ -184,6 +200,9 @@ static void UpdateScrollbar
 {
     YOffset position;
     UInt16  prevCoordSys;
+    FormType* mainForm;
+
+    mainForm = FrmGetFormPtr( GetMainFormId() );
 
     if ( Prefs()->scrollbar == SCROLLBAR_NONE )
         return;
@@ -191,6 +210,9 @@ static void UpdateScrollbar
     prevCoordSys   = PalmSetCoordinateSystem( STANDARD );
 
     if ( viewportBounds.extent.y < GetSequenceHeight() ) {
+        FrmShowObject( mainForm, FrmGetObjectIndex( mainForm, 
+            frmMainScrollBar ) );
+
         trueMaxScrollbar   = GetSequenceHeight() - viewportBounds.extent.y;
         position           = GetSequenceOffset();
         scrollbarScale     = 1 + ( trueMaxScrollbar / 32000 );
@@ -210,11 +232,8 @@ static void UpdateScrollbar
 
     }
     else {
-        FormType* mainForm;
-
-        mainForm = FrmGetFormPtr( GetMainFormId() );
-
-        FrmHideObject( mainForm, FrmGetObjectIndex( mainForm, frmMainScrollBar ) );
+        FrmHideObject( mainForm, FrmGetObjectIndex( mainForm, 
+            frmMainScrollBar ) );
     }
 
     PalmSetCoordinateSystem( prevCoordSys );
@@ -305,6 +324,9 @@ void SetWorking( void )
         FrmHideObject( mainForm, FrmGetObjectIndex( mainForm, bmpLeft ) );
         FrmHideObject( mainForm, FrmGetObjectIndex( mainForm, bmpRight ) );
         FrmHideObject( mainForm, FrmGetObjectIndex( mainForm, bmpHome ) );
+#ifdef SUPPORT_WORD_LOOKUP
+        FrmHideObject( mainForm, FrmGetObjectIndex( mainForm, bmpLookup ) );
+#endif
 
         FrmShowObject( mainForm, FrmGetObjectIndex( mainForm, bmpWait ) );
     }
@@ -325,7 +347,7 @@ static void SetIdle( void )
     mainForm = FrmGetFormPtr( GetMainFormId() );
     prevCoordSys = PalmSetCoordinateSystem( STANDARD );
     if ( Prefs()->scrollbar != SCROLLBAR_NONE )
-        FrmShowObject( mainForm, FrmGetObjectIndex( mainForm, 
+        FrmShowObject( mainForm, FrmGetObjectIndex( mainForm,
                                      frmMainScrollBar ) );
 
     if ( Prefs()->toolbar == TOOLBAR_SILK ) {
@@ -333,6 +355,9 @@ static void SetIdle( void )
     }
     else {
         FrmHideObject( mainForm, FrmGetObjectIndex( mainForm, bmpWait ) );
+#ifdef SUPPORT_WORD_LOOKUP
+        FrmHideObject( mainForm, FrmGetObjectIndex( mainForm, bmpLookup ) );
+#endif
 
         FrmShowObject( mainForm, FrmGetObjectIndex( mainForm, bmpLeft ) );
         FrmShowObject( mainForm, FrmGetObjectIndex( mainForm, bmpRight ) );
@@ -354,7 +379,7 @@ void UpdateVerticalOffset
     StrPrintF( offText, offsetFormat, percent );
 
     if ( Prefs()->toolbar == TOOLBAR_SILK ) {
-        UpdateSilkVerticalOffset( offText );
+        HanderaUpdateSilkVerticalOffset( offText );
     }
     else if ( Prefs()->toolbar != TOOLBAR_NONE ) {
         UInt16 prevCoordSys;
@@ -383,7 +408,7 @@ static void ShowVerticalOffset( void )
     if ( 0 < meta->height ) {
         percent  = GetSequenceOffset() + RotExtentY();
         totalHeight = GetSequenceHeight();
-        if ( totalHeight > 100000l ) {
+        if ( 100000l < totalHeight ) {
             percent /= ( totalHeight / 100 );
         }
         else
@@ -428,14 +453,14 @@ static void SetFirstVisibleParagraph
 /* Initialize text context */
 static void InitializeTextContext
     (
-    TextContext* tContext   /* pointer to text context */
+    TextContext*        tContext,  /* pointer to text context */
+    const WriteModeType mode
     )
 {
     tContext->cursorX       = viewportBounds.topLeft.x;
     tContext->cursorY       = viewportBounds.topLeft.y;
-    tContext->writeMode     = DrawChar;
+    tContext->writeMode     = mode;
     tContext->activeAnchor  = false;
-    tContext->patternOffset = -1;
     tContext->findCharacterPosition = -1;
 }
 
@@ -469,8 +494,13 @@ static void RenderRecord
 
     SaveExtents();
 
-    InitializeTextContext( &thisContext );
-    thisContext.writeMode = mode;
+    InitializeTextContext( &thisContext, mode );
+#ifdef SUPPORT_WORD_LOOKUP
+    if ( mode == WRITEMODE_FIND_SELECTED_WORD ) {
+        thisContext.findPixelX = findPixelX;
+        thisContext.findPixelY = findPixelY;
+    }
+#endif
 
     paragraph_index     = meta->firstVisibleParagraph;
     numOfParagraphs     = record->paragraphs - paragraph_index;
@@ -489,31 +519,51 @@ static void RenderRecord
 
     needNewParagraph    = false;
 
-    if ( ! continuation )
+    if ( ! continuation && DO_ANCHORS( mode ) )
         AnchorListInit();
 
     /* If depth > 1, then save colors' state for a later restore */
     if ( 1 < Prefs()->screenDepth )
         SaveDrawState();
 
-    if ( clipRect == NULL )
-        clipRect = &viewportBounds;
-    if ( IsHiResTypeSony( HiResType() ) )
-        /* There is some problem with hi-res clipping on Sony units */
-        RotSetClip( &viewportBounds );
-    else
-        RotSetClip( clipRect );
-    RotCharClipRange( clipRect->topLeft.y, clipRect->extent.y );
+    if ( mode == WRITEMODE_DRAW_CHAR ) {
+        if ( clipRect == NULL )
+            clipRect = &viewportBounds;
+        if ( ( clipRect->extent.y <= 0 || clipRect->extent.x <= 0 ) ) {
+            mode = WRITEMODE_LAYOUT_ANCHORS;
+        }
+        else {
+            if ( IsHiResTypeSony( HiResType() ) )
+                /* There is some problem with hi-res clipping on Sony units */
+                RotSetClip( &viewportBounds );
+            else
+                RotSetClip( clipRect );
+            RotCharClipRange( clipRect->topLeft.y, clipRect->extent.y );
+        }
+    }    
 
     while ( numOfParagraphs-- ) {
         lastCursor = cursor;
 
         paragraphNum = paragraph - firstParagraph;
 
-        if ( paragraphNum < GetLineCacheNeedToRenderFrom() )
+        if ( paragraphNum < GetLineCacheNeedToRenderFrom()
+#ifdef SUPPORT_WORD_LOOKUP
+             || ( mode == WRITEMODE_FIND_SELECTED_WORD &&
+                  ( thisContext.findPixelY < thisContext.cursorY ||
+                    thisContext.cursorY + metaParagraph->height <=
+                        thisContext.findPixelY ) )
+#endif
+           ) {
             thisContext.cursorY += metaParagraph->height;
-        else
+        }
+        else {
             DrawParagraph( &thisContext, paragraph, record );
+#ifdef SUPPORT_WORD_LOOKUP
+            if ( mode == WRITEMODE_FIND_SELECTED_WORD && SelectedWordDone() )
+                break;
+#endif
+        }
 
         cursor += metaParagraph->height;
 
@@ -539,7 +589,7 @@ static void RenderRecord
         paragraph_index++;
     }
 
-    if ( 0 <= thisContext.characterPosition ) {
+    if ( 0 <= thisContext.characterPosition && DO_LAYOUT( mode ) ) {
         DmWrite( meta, OFFSETOF( MetaRecord, characterPosition ),
             &thisContext.characterPosition, sizeof( Int16 ) );
         SetHistoryCharacterPosition( thisContext.characterPosition );
@@ -552,7 +602,8 @@ static void RenderRecord
     if ( 1 < Prefs()->screenDepth )
         RestoreDrawState();
 
-    WinResetClip();
+    if ( mode == WRITEMODE_DRAW_CHAR )
+        WinResetClip();
     RotCharClearClipRange();
 }
 
@@ -597,10 +648,10 @@ static void AdjustVerticalOffset
     continuation = false;
     /* Adjust the record */
     newRecord = *meta;
-    /* bottom of page */
 
-    /** FIX ME: This code assumes that no non-final fragment can be less
-        than one screenful long. **/
+    /** FIX ME: The following code assumes that no non-final fragment can be 
+        less than one screenful long. **/
+    /* bottom of page */
     if ( newRecord.verticalOffset + adjustment < minVerticalOffset ) {
         if ( adjustment < 0 ) {
             /* at the bottom of the page (and beyond) */
@@ -626,8 +677,9 @@ static void AdjustVerticalOffset
 #endif
             }
         }
-        else
+        else {
             continuation = true;
+        }
     }
     if ( nextRecordId == NO_RECORD &&
          0 < newRecord.verticalOffset + adjustment ) {
@@ -650,15 +702,14 @@ static void AdjustVerticalOffset
                 }
             }
         }
-        else
+        else {
             continuation = true;
+        }
     }
-
     if ( adjustment == 0 ) {
         MemHandleUnlock( metaRecord );
         return;
     }
-
     adjustAmount                = abs( adjustment );
     newRecord.verticalOffset   += adjustment;
     SetHistoryVerticalOffset( newRecord.verticalOffset );
@@ -685,7 +736,7 @@ static void AdjustVerticalOffset
 
     record = MemHandleLock( recordHandle );
 
-    RenderRecord( record, DrawChar, continuation, &clipRectangle );
+    RenderRecord( record, WRITEMODE_DRAW_CHAR, continuation, &clipRectangle );
 
     MemHandleUnlock( recordHandle );
 
@@ -706,8 +757,8 @@ static void AdjustVerticalOffset
 #endif
 
     if ( nextRecordId != NO_RECORD ) {
-        ViewRecordAt( nextRecordId, false, 0, 0, yPosition, backwards, true,
-            DrawChar );
+        ViewRecordAt( nextRecordId, FROM_HISTORY, 0, 0, yPosition, backwards,
+            CONTINUATION, WRITEMODE_DRAW_CHAR );
     }
 }
 
@@ -728,7 +779,7 @@ static YOffset CalculatePosition
     Coord    yPosition,         /* start drawing here */
     Boolean  backwards,         /* if backwards is set, yPosition indicates
                                    where the end of the record is to be */
-    Boolean  draw               /* do we do any actual drawing? */
+    WriteModeType mode          /* do we do any actual drawing? */
     )
 {
     Paragraph*  paragraph;
@@ -750,9 +801,8 @@ static YOffset CalculatePosition
     metaParagraph = GET_METAPARAGRAPH( meta, 0 );
 
 
-    InitializeTextContext( &thisContext );
-    if ( ! draw )
-        thisContext.writeMode = NoDraw;
+    InitializeTextContext( &thisContext, mode );
+
     numOfParagraphs = record->paragraphs;
     drawnHeight     = 0;
 
@@ -786,9 +836,9 @@ static YOffset CalculatePosition
             thisContext.cursorY               = yPosition;
             thisContext.findCharacterPosition = characterPosition;
             thisContext.foundYPosition        = -1;
-            thisContext.writeMode             = NoDraw;
+            thisContext.writeMode             = WRITEMODE_NO_DRAW;
             DrawParagraph( &thisContext, paragraph, record );
-            if ( 0 < thisContext.foundYPosition ) {
+            if ( 0 <= thisContext.foundYPosition ) {
                 offsetInParagraph = yPosition - thisContext.foundYPosition;
                 pOffset           = i;
             }
@@ -806,9 +856,7 @@ static YOffset CalculatePosition
 
         paragraph     = GET_PARAGRAPH( record, pOffset );
 
-        InitializeTextContext( &thisContext );
-        if ( ! draw )
-            thisContext.writeMode = NoDraw;
+        InitializeTextContext( &thisContext, mode );
         thisContext.cursorY += yPosition + offsetInParagraph;
 
         for ( i = pOffset ; i < numOfParagraphs ; i++ ) {
@@ -848,8 +896,7 @@ static YOffset CalculatePosition
               0 <= i && RotTopLeftY() < currentY;
               i-- ) {
             if ( recalculateHeights ) {
-                InitializeTextContext( &thisContext );
-                thisContext.writeMode = NoDraw;
+                InitializeTextContext( &thisContext, WRITEMODE_NO_DRAW );
                 DrawParagraph( &thisContext, paragraph, record );
                 paragraphHeight = thisContext.cursorY - RotTopLeftY();
                 DmWrite( meta, sizeof( MetaRecord ) +
@@ -860,9 +907,8 @@ static YOffset CalculatePosition
                 paragraphHeight = metaParagraph[ i ].height;
             }
             currentY -= paragraphHeight;
-            InitializeTextContext( &thisContext );
-            if ( ! draw )
-                thisContext.writeMode = NoDraw;
+            InitializeTextContext( &thisContext, mode );
+
             thisContext.cursorY = currentY;
             DrawParagraph( &thisContext, paragraph, record );
             drawnHeight += paragraphHeight;
@@ -876,8 +922,7 @@ static YOffset CalculatePosition
     WinResetClip();
     RotCharClearClipRange();
 
-    InitializeTextContext( &thisContext );
-    thisContext.writeMode = NoDraw;
+    InitializeTextContext( &thisContext, WRITEMODE_NO_DRAW );
 
     paragraph             = GET_PARAGRAPH( record, 0 );
 
@@ -940,26 +985,30 @@ static Boolean ViewPHTML
     Boolean     didDraw;
     Boolean     lineCache;
 
+    lineCache = IsLineCacheActive();
+    LineCacheDeactivate();
+
     /* Re-validate coordinate system as NATIVE even though we just did in
        other functions. After viewing a fullscreen image, we seem to
        mysteriously get pushed into STANDARD mode by this point. Resetting
        it to NATIVE ensures everything displays properly. */
-
-    lineCache = IsLineCacheActive();
-    LineCacheDeactivate();
-
     prevCoordSys = PalmSetCoordinateSystem( NATIVE );
 
     if ( cOffset != NO_OFFSET )
         pOffset = NO_OFFSET;
 
-    didDraw      = false;
+    didDraw = false;
 
-    if ( mode == DrawChar )
+    if ( mode == WRITEMODE_DRAW_CHAR )
         SetWorking();
 
-    if ( record->type == DATATYPE_PHTML_COMPRESSED )
+    if ( record->type == DATATYPE_PHTML_COMPRESSED &&
+         ( ! haveLastUncompressedRecordId ||
+           lastUncompressedRecordId != record->uid ) ) {
+        haveLastUncompressedRecordId = true;
+        lastUncompressedRecordId     = record->uid;
         Uncompress( record );
+    }
 
     meta       = MemHandleLock( metaRecord );
     recordData = *meta;
@@ -977,21 +1026,21 @@ static Boolean ViewPHTML
         SetFirstVisibleParagraph( 0, 0 );
     }
     if ( recordData.height == 0 ) {
-        if ( yPosition == 0 )
+        if ( yPosition == 0 && DO_ANCHORS( mode ) )
             AnchorListInit();
 
         if ( cOffset == NO_OFFSET )
             recordData.height = CalculatePosition( record, pOffset,
                 recordData.characterPosition, &recordData.verticalOffset,
-                true, yPosition, backwards, mode == DrawChar );
+                true, yPosition, backwards, mode );
         else
             recordData.height = CalculatePosition( record, pOffset,
                 cOffset, &recordData.verticalOffset, true, yPosition,
-                backwards, mode == DrawChar );
+                backwards, mode );
 
         didDraw = true;
 
-        if ( yPosition == 0 )
+        if ( yPosition == 0 && DO_ANCHORS( mode ) )
             AnchorListRelease();
 
         DmWrite( meta, OFFSETOF( MetaRecord, height ), &recordData.height,
@@ -1000,7 +1049,7 @@ static Boolean ViewPHTML
     if ( ( backwards || cOffset != NO_OFFSET || pOffset != NO_OFFSET ) &&
          ! didDraw ) {
         CalculatePosition( record, pOffset, cOffset, &recordData.verticalOffset,
-            false, yPosition, backwards, mode == DrawChar );
+            false, yPosition, backwards, mode );
         DmWrite( meta, OFFSETOF( MetaRecord, verticalOffset ),
             &recordData.verticalOffset, sizeof( YOffset ) );
         didDraw = true;
@@ -1009,8 +1058,9 @@ static Boolean ViewPHTML
        the record. */
 #ifdef ENABLE_SCROLL_TO_BOTTOM
     if ( yPosition == 0 && ! backwards &&
-         NO_RECORD == GetSequentialRecordId( DIRECTION_DOWN ) &&
-         recordData.height < -recordData.verticalOffset + RotExtentY() ) {
+         recordData.height < -recordData.verticalOffset + RotExtentY() &&
+         NO_RECORD == GetSequentialRecordId( DIRECTION_DOWN )
+         ) {
         recordData.verticalOffset = RotExtentY() - recordData.height;
         if ( 0 < recordData.verticalOffset )
             recordData.verticalOffset = 0;
@@ -1018,7 +1068,7 @@ static Boolean ViewPHTML
             &recordData.verticalOffset, sizeof( YOffset ) );
         SetHistoryVerticalOffset( recordData.verticalOffset );
         SetHistoryRecordId( currentRecordId );
-        if ( didDraw )
+        if ( didDraw && mode == WRITEMODE_DRAW_CHAR )
             RotEraseRectangle( &viewportBounds, 0 );
     }
 #endif
@@ -1032,7 +1082,7 @@ static Boolean ViewPHTML
         LineCacheActivate();
     RenderRecord( record, mode, yPosition != 0, NULL );
 
-    if ( mode == DrawChar )
+    if ( mode == WRITEMODE_DRAW_CHAR )
         SetIdle();
 
     PalmSetCoordinateSystem( prevCoordSys );
@@ -1046,7 +1096,7 @@ static Boolean ViewPHTML
 void DoPageMove
     (
     const YOffset pixels  /* number of pixels to move ( positive values move
-                           up, negative down ) */
+                             up, negative down ) */
     )
 {
     UInt16    prevCoordSys;
@@ -1056,6 +1106,8 @@ void DoPageMove
     YOffset   abspixels;
     Boolean   useFullScreen;
 
+    UnselectActualAnchor();
+
     prevCoordSys = PalmSetCoordinateSystem( NATIVE );
 
     abspixels = ( pixels < 0 ) ? -pixels : pixels;
@@ -1064,7 +1116,7 @@ void DoPageMove
         recordId  = DeltaToRecordId( pixels, &cOffset );
         if ( recordId != currentRecordId ) {
             PalmSetCoordinateSystem( prevCoordSys );
-            ViewRecord( recordId, false, NO_OFFSET, cOffset );
+            ViewRecord( recordId, false, NO_OFFSET, cOffset, WRITEMODE_DRAW_CHAR );
             return;
         }
     }
@@ -1091,7 +1143,7 @@ void DoPageMove
         FsAdjustVerticalOffset( pixels );
     else
         AdjustVerticalOffset( currentRecord, pixels );
-        
+
     PalmSetCoordinateSystem( prevCoordSys );
 }
 
@@ -1150,7 +1202,8 @@ Boolean ViewRecord
     Boolean         newPage,    /* indicates if the page is not from the
                                    history */
     Int16           pOffset,    /* offset to first paragraph */
-    Int16           cOffset     /* offset to first character */
+    Int16           cOffset,    /* offset to first character */
+    WriteModeType   mode
     )
 {
     UInt16  prevCoordSys;
@@ -1162,10 +1215,11 @@ Boolean ViewRecord
 
     InitViewportBounds();
     WinResetClip();  /* FIXME: This shouldn't really be needed! */
-    RotEraseRectangle( &viewportBounds, 0 );
+    if ( mode == WRITEMODE_DRAW_CHAR )
+        RotEraseRectangle( &viewportBounds, 0 );
 
     retValue = ViewRecordAt( recordId, newPage, pOffset, cOffset, 0,
-               false, false, DrawChar );
+               ! BACKWARDS, ! CONTINUATION, mode );
 
     PalmSetCoordinateSystem( prevCoordSys );
 
@@ -1190,7 +1244,7 @@ static Boolean ViewRecordAt
     )
 {
     UInt16  prevCoordSys;
-    Boolean status;
+    Boolean success;
     Header* record;
     UInt16  nextRecordId;
     static  Boolean haveLastRecordId;
@@ -1224,9 +1278,12 @@ static Boolean ViewRecordAt
         MSG( _( "record #%d\n", recordId ) );
 
         if ( ! continuation ) {
-            AnchorListRelease();
-            if ( mode == DrawChar )
-                DeleteTableWindow();
+            if ( DO_ANCHORS( mode ) )
+                AnchorListRelease();
+            if ( mode == WRITEMODE_DRAW_CHAR ) {
+                ReleaseFsTableHandle();
+                ReleaseFsImageHandle();
+            }
         }
 
         if ( currentRecordId != recordId ) {
@@ -1235,6 +1292,8 @@ static Boolean ViewRecordAt
             ErrTry {
                 currentRecord   = GetRecordHandle( recordId );
                 currentRecordId = recordId;
+                metaRecord      = GetMetaHandle( recordId, true );
+
             }
             ErrCatch( UNUSED_PARAM( err ) ) {
                 SetLinkIndex( recordId );
@@ -1252,8 +1311,8 @@ static Boolean ViewRecordAt
             haveLastRecordId = true;
         }
 
-        status = false;
-        record = MemHandleLock( currentRecord );
+        success = false;
+        record  = MemHandleLock( currentRecord );
         ErrTry {
             switch ( record->type ) {
                 case DATATYPE_PHTML:
@@ -1261,15 +1320,14 @@ static Boolean ViewRecordAt
 
                     textRecord = true;
 
-                    metaRecord = GetMetaHandle( recordId, true );
-
                     if ( newPage )
                         AddToHistory( recordId );
                     else
                         SetHistoryRecordId( recordId );
-                    status     = ViewPHTML( record, continuation || newPage,
-                                     pOffset, cOffset, yPosition, backwards,
-                                     mode );
+
+                    success = ViewPHTML( record, continuation || newPage,
+                                pOffset, cOffset, yPosition, backwards,
+                                mode );
 
                     if ( ( backwards || ! continuation ) &&
                          GetVerticalOffset() < 0 ) {
@@ -1301,9 +1359,8 @@ static Boolean ViewRecordAt
                     if ( newPage )
                         AddToHistory( recordId );
 
-                    metaRecord  = GetMetaHandle( recordId, true );
-                    status      = LoadFullScreenImage( record, newPage );
-                    if ( status == true )
+                    success     = LoadFullScreenImage( record, newPage );
+                    if ( success )
                         FrmPopupForm( frmFullscreen );
                     break;
 
@@ -1312,24 +1369,23 @@ static Boolean ViewRecordAt
                     if ( newPage )
                         AddToHistory( recordId );
 
-                    metaRecord  = GetMetaHandle( recordId, true );
-                    status      = LoadTable( record, newPage );
-                    if ( status == true )
+                    success     = LoadTable( record, newPage );
+                    if ( success )
                         FrmPopupForm( frmFullscreen );
                     break;
-    
+
                 case DATATYPE_MAILTO:
                     SetMailto( recordId );
                     FrmGotoForm( frmEmail );
-    
-                    status = true;
+
+                    success = true;
                     break;
 
                 case DATATYPE_LINK_INDEX:
                     SetLinkIndex( recordId );
                     FrmGotoForm( frmExternalLinks );
 
-                    status = true;
+                    success = true;
                     break;
 
                 default:
@@ -1340,7 +1396,7 @@ static Boolean ViewRecordAt
         ErrCatch( err ) {
             switch ( err ) {
             case errZLibMemError:
-                status = false;
+                success = false;
                 break;
 
             default:
@@ -1356,7 +1412,7 @@ static Boolean ViewRecordAt
 
     PalmSetCoordinateSystem( prevCoordSys );
 
-    if ( textRecord && mode == DrawChar ) {
+    if ( textRecord && mode == WRITEMODE_DRAW_CHAR ) {
         MetaRecord* meta;
         EstimateHeights();
         ShowVerticalOffset();
@@ -1365,7 +1421,7 @@ static Boolean ViewRecordAt
         MemHandleUnlock( metaRecord );
     }
 
-    return status;
+    return success;
 }
 
 
@@ -1425,7 +1481,7 @@ MemHandle GetMetaRecord( void )
 /* Return vertical offset, negated */
 YOffset GetVerticalOffset( void )
 {
-    YOffset       verticalOffset;
+    YOffset     verticalOffset;
     MetaRecord* meta;
 
     meta = MemHandleLock( metaRecord );
@@ -1492,15 +1548,15 @@ UInt16 GetFirstVisibleRecordId( void )
 
 
 
-/* wrapper for RenderRecord. Replace function set and execute */
-void CopyRecord( Header* record, Char* buf, Int16 bufsize )
+/* Wrapper for RenderRecord. Replace function set and execute */
+void CopyRecord( Char* buf, Int16 bufsize )
 {
     SetReRenderCheck( false );
     ActivateSavedBounds();
     SetCopyBuffer( buf, bufsize );
     StartCopyMode();
-    ViewRecordAt( GetFirstVisibleRecordId(),
-            false, NO_OFFSET, NO_OFFSET, 0, false, false, CopyChar );
+    ViewRecordAt( GetFirstVisibleRecordId(), FROM_HISTORY, NO_OFFSET,
+        NO_OFFSET, 0, ! BACKWARDS, ! CONTINUATION, WRITEMODE_COPY_CHAR );
     StopCopyMode();
     RestoreBounds();
     SetReRenderCheck( true );
@@ -1508,20 +1564,51 @@ void CopyRecord( Header* record, Char* buf, Int16 bufsize )
 
 
 
-/*wrapper for RenderRecord. Replace function set and execute.*/
-void TextifyRecord( Header* record, Char* buf, Int16 bufsize )
+/* Wrapper for RenderRecord. Replace function set and execute. */
+void TextifyRecord( Char* buf, Int16 bufsize )
 {
+    if ( IsFullscreenformActive() ) {
+        if ( IsLargeTable() )
+            StrPrintF( buf, "[TABLE %u]", currentRecordId );
+        else
+            StrPrintF( buf, "[IMG %u]", currentRecordId );
+        return;
+    }
     SetReRenderCheck( false );
     ActivateSavedBounds();
     SetCopyBuffer( buf, bufsize );
-    ViewRecordAt( GetFirstVisibleRecordId(),
-            false, NO_OFFSET, NO_OFFSET, 0, false, false, CopyChar );
+    ViewRecordAt( GetFirstVisibleRecordId(), FROM_HISTORY, NO_OFFSET,
+        NO_OFFSET, 0, ! BACKWARDS, ! CONTINUATION, WRITEMODE_COPY_CHAR );
     RestoreBounds();
     SetReRenderCheck( true );
 }
 
 
 
+#ifdef SUPPORT_WORD_LOOKUP
+/* Wrapper for RenderRecord. Replace function set and execute. */
+void DoFindSelectedWord
+    (
+    Coord x,
+    Coord y
+    )
+{
+    SelectedWordReset();
+    SetReRenderCheck( false );
+    ActivateSavedBounds();
+    RotFromScreenXY( &x, &y );
+    findPixelX = x;
+    findPixelY = y;
+    ViewRecordAt( GetFirstVisibleRecordId(), FROM_HISTORY, NO_OFFSET,
+        NO_OFFSET, 0, ! BACKWARDS, ! CONTINUATION,
+        WRITEMODE_FIND_SELECTED_WORD );
+    RestoreBounds();
+    SetReRenderCheck( true );
+}
+#endif
+
+
+
 static void SaveExtents( void )
 {
     savedViewportBounds = viewportBounds;
@@ -1598,10 +1685,11 @@ static void RestoreBounds( void )
 
 /* Get the first/last record in the sequence */
 UInt16 GetSequenceBoundaryRecordId
-        (
-        UInt16        recordId,
-        DirectionType direction
-        )
+    (
+    UInt16        recordId,
+    DirectionType direction,
+    UInt16*       recordNumPtr
+    )
 {
     UInt16    recordNum;
     MemHandle handle;
@@ -1611,10 +1699,12 @@ UInt16 GetSequenceBoundaryRecordId
         return NO_RECORD;
     else
         FreeRecordHandle( &handle );
+    if ( recordNumPtr != NULL )
+        *recordNumPtr = recordNum;
 
     for (;;) {
-          Header*   record;
-          MemHandle handle;
+         Header*   record;
+         MemHandle handle;
 
          handle = GetSequentialRecordHandle( &recordNum, direction );
          if ( handle != NULL ) {
@@ -1622,9 +1712,12 @@ UInt16 GetSequenceBoundaryRecordId
              recordId = record->uid;
              MemHandleUnlock( handle );
              FreeRecordHandle( &handle );
+             if ( recordNumPtr != NULL )
+                 *recordNumPtr = recordNum;
          }
-         else
+         else {
              return recordId;
+         }
     }
 }
 
@@ -1632,11 +1725,11 @@ UInt16 GetSequenceBoundaryRecordId
 
 
 /* Get the next record in the current sequence */
-static MemHandle GetSequentialRecordHandle
-              (
-              UInt16*       recordNumPtr,
-              DirectionType direction
-              )
+MemHandle GetSequentialRecordHandle
+    (
+    UInt16*       recordNumPtr,
+    DirectionType direction
+    )
 {
     MemHandle handle;
     Header*   record;
@@ -1659,7 +1752,7 @@ static MemHandle GetSequentialRecordHandle
     }
 
     do {
-         FreeRecordHandle( &handle );
+        FreeRecordHandle( &handle );
         if ( direction == DIRECTION_UP ) {
             if ( recordNum < 2 )
                 return NULL;
@@ -1692,9 +1785,9 @@ static MemHandle GetSequentialRecordHandle
 
 /* Get the next record ID in a natural sequence */
 UInt16 GetSequentialRecordId
-              (
-              DirectionType direction
-              )
+    (
+    DirectionType direction
+    )
 {
     MemHandle handle;
     Header*   record;
