@@ -1,5 +1,5 @@
 /*
- * $Id: font.c,v 1.25 2003/09/24 13:02:03 prussar Exp $
+ * $Id: font.c,v 1.48 2004/02/29 20:31:58 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -31,6 +31,10 @@
 
 #include "font.h"
 
+#ifdef SUPPORT_VFS_FONTS
+#include <VFSMgr.h>
+#include "vfsfile.h"
+#endif
 
 /***********************************************************************
  *
@@ -40,6 +44,25 @@
 #define EOL 0xFF
 
 
+
+
+/***********************************************************************
+ *
+ *      Local functions
+ *
+ ***********************************************************************/
+/* This should be called whenever the font package is changed */
+static void ClearUserFontNameList( void );
+static void SortAndMakeUserFontNameList( void );
+#ifdef SUPPORT_VFS_FONTS
+static void VFSFontCacheAddEntry ( const Char* name, const Char* volumeLabel,
+       const Char* path, Int16 cache );
+static Boolean VFSFontCacheFindEntry ( const Char* name );
+#endif       
+
+
+
+
 /***********************************************************************
  *
  *      Internal Types
@@ -64,6 +87,9 @@ typedef struct
     UInt16     cardNo;
     LocalID    dbID;
     Char       name[ dmDBNameLength ];
+    Boolean    onVFS;
+    Char*      path;
+    Char*      volumeLabel;
 }
 DBEntryType;
 
@@ -151,7 +177,7 @@ static FontID palmHiResLargeStyles[ MAXSTYLES ] = {
 };
 
 static FontID palmHiResUserStyles[ MAXSTYLES ] = {
-    userStdFont_palm, userLargeBoldFont_palm, userLargeBoldFont_palm, 
+    userStdFont_palm, userLargeBoldFont_palm, userLargeBoldFont_palm,
     userLargeFont_palm,
     userLargeFont_palm, userBoldFont_palm, userBoldFont_palm, userBoldFont_palm,
     userFixedFont_palm, userStdFont_palm, userNarrowFont_palm,
@@ -208,18 +234,23 @@ static FontID handeraHiResLargeStyles[ MAXSTYLES ] = {};
 
 #endif /* HAVE_HIRES */
 
+static Boolean haveItalic[ NUM_ITALIC_USER_FONTS ];
+
 
 /***********************************************************************
  *
  *      Private variables
  *
  ***********************************************************************/
-static LinkedList   customFontList      = NULL;
 static FontID*      StylesLibrary;
 static FontID*      StylesMain;
 static Int16        currentStyle        = DEFAULTSTYLE;
 static Int16        currentStyleFont    = stdFont;
 static Int16        prevFontHeight      = -1;
+#ifdef SUPPORT_VFS_FONTS
+static Char         vfsFontCacheName[]  = "PlkrVFSFontCache";
+static Boolean      scannedVFS          = false;
+#endif
 
 
 static CustomFontType ListOfCustomStandardFonts[] = {
@@ -232,6 +263,18 @@ static CustomFontType ListOfCustomStandardFonts[] = {
     { 'NFNT', userLargeBoldFontID, userLargeBoldFont, largeBoldFont },
     { 'NFNT', userFixedFontID, userFixedFont, boldFont },
     { 'NFNT', userNarrowFontID, userNarrowFont, stdFont },
+    { 'NFNT', userStdFontID | ITALIC_FONTID_MASK,
+                  userStdFont + ITALIC_FONT_DELTA, stdFont },
+    { 'NFNT', userBoldFontID | ITALIC_FONTID_MASK,
+                  userBoldFont + ITALIC_FONT_DELTA, boldFont },
+    { 'NFNT', userLargeFontID | ITALIC_FONTID_MASK,
+                  userLargeFont + ITALIC_FONT_DELTA, largeFont },
+    { 'NFNT', userLargeBoldFontID | ITALIC_FONTID_MASK,
+                  userLargeBoldFont + ITALIC_FONT_DELTA, largeBoldFont },
+    { 'NFNT', userFixedFontID | ITALIC_FONTID_MASK,
+                  userFixedFont + ITALIC_FONT_DELTA, narrowFixedFont },
+    { 'NFNT', userNarrowFontID | ITALIC_FONTID_MASK,
+                  userNarrowFont + ITALIC_FONT_DELTA, narrowFont },
     { EOL, NULL, NULL }
 };
 
@@ -246,8 +289,20 @@ static CustomFontType ListOfCustomPalmHiResFonts[] = {
     { 'nfnt', userBoldFontID_palm, userBoldFont_palm, boldFont },
     { 'nfnt', userLargeFontID_palm, userLargeFont_palm, largeFont },
     { 'nfnt', userLargeBoldFontID_palm, userLargeBoldFont_palm, largeBoldFont },
-    { 'nfnt', userFixedFontID_palm, userFixedFont_palm, boldFont },
+    { 'nfnt', userFixedFontID_palm, userFixedFont_palm, stdFixedFont_palm },
     { 'nfnt', userNarrowFontID_palm, userNarrowFont_palm, stdFont },
+    { 'nfnt', userStdFontID_palm | ITALIC_FONTID_MASK,
+                  userStdFont_palm + ITALIC_FONT_DELTA, stdFont },
+    { 'nfnt', userBoldFontID_palm | ITALIC_FONTID_MASK,
+                  userBoldFont_palm + ITALIC_FONT_DELTA, boldFont },
+    { 'nfnt', userLargeFontID_palm | ITALIC_FONTID_MASK,
+                  userLargeFont_palm + ITALIC_FONT_DELTA, largeFont },
+    { 'nfnt', userLargeBoldFontID_palm | ITALIC_FONTID_MASK,
+                  userLargeBoldFont_palm + ITALIC_FONT_DELTA, largeBoldFont },
+    { 'nfnt', userFixedFontID_palm | ITALIC_FONTID_MASK,
+                  userFixedFont_palm + ITALIC_FONT_DELTA, stdFixedFont_palm },
+    { 'nfnt', userNarrowFontID_palm | ITALIC_FONTID_MASK,
+                  userNarrowFont_palm + ITALIC_FONT_DELTA, stdFont },
     { EOL, NULL, NULL }
 };
 
@@ -258,8 +313,20 @@ static CustomFontType ListOfCustomSonyHiResFonts[] = {
     { 'NFNT', userBoldFontID_sony, userBoldFont_sony, hrBoldFont },
     { 'NFNT', userLargeFontID_sony, userLargeFont_sony, hrLargeFont },
     { 'NFNT', userLargeBoldFontID_sony, userLargeBoldFont_sony, hrLargeBoldFont },
-    { 'NFNT', userFixedFontID_sony, userFixedFont_sony, hrBoldFont },
+    { 'NFNT', userFixedFontID_sony, userFixedFont_sony, stdFixedFont_sony },
     { 'NFNT', userNarrowFontID_sony, userNarrowFont_sony, hrStdFont },
+    { 'NFNT', userStdFontID_sony | ITALIC_FONTID_MASK,
+                  userStdFont_sony + ITALIC_FONT_DELTA, hrStdFont },
+    { 'NFNT', userBoldFontID_sony | ITALIC_FONTID_MASK,
+                  userBoldFont_sony + ITALIC_FONT_DELTA, hrBoldFont },
+    { 'NFNT', userLargeFontID_sony | ITALIC_FONTID_MASK,
+                  userLargeFont_sony + ITALIC_FONT_DELTA, hrLargeFont },
+    { 'NFNT', userLargeBoldFontID_sony | ITALIC_FONTID_MASK,
+                  userLargeBoldFont_sony + ITALIC_FONT_DELTA, hrLargeBoldFont },
+    { 'NFNT', userFixedFontID_sony | ITALIC_FONTID_MASK,
+                  userFixedFont_sony + ITALIC_FONT_DELTA, stdFixedFont_sony },
+    { 'NFNT', userNarrowFontID_sony | ITALIC_FONTID_MASK,
+                  userNarrowFont_sony + ITALIC_FONT_DELTA, hrStdFont },
     { EOL, NULL, NULL }
 };
 #endif /* HAVE_SONY_SDK */
@@ -321,12 +388,16 @@ static StyleType ListOfFontStylesHandera[] = {
 static LinkedList userFontDBList          = NULL;
 static Int16      numberOfUserFonts       = 0;
 static Char**     userFontNames           = NULL;
-static Char*      emptyUserFontNameList[] = { 
-    "--" 
+static Char*      emptyUserFontNameList[] = {
+    "--"
 };
 static DmOpenRef  currentUserFontDBRef;
 static Int16      currentUserFontNumber   = NO_SUCH_USER_FONT;
 static DeviceFontType  currentDeviceFont;
+#ifdef SUPPORT_VFS_FONTS
+static Boolean    currentFontIsOnVFS = false;
+static FileRef    currentVFSFontRef;
+#endif
 
 
 /* Load custom fonts into memory */
@@ -336,15 +407,13 @@ void LoadCustomFonts
     )
 {
     CustomFontType* fontList;
-    MemHandle*      handleNode;
     FontType*       font;
 
     currentDeviceFont = deviceFont;
-    
+
     GrayFntClearSubstitutionList();
 
-    if ( customFontList == NULL )
-        customFontList = ListCreate();
+    ReleaseCustomFonts();
 
     switch ( deviceFont ) {
         case STANDARD_FONTS:
@@ -365,18 +434,29 @@ void LoadCustomFonts
     }
 
     for ( ; fontList->resourceType != EOL; fontList++ ) {
-        handleNode = SafeMemPtrNew( sizeof( MemHandle ) );
-        *handleNode = DmGetResource( fontList->resourceType,
-                                     fontList->resourceID );
-        if ( *handleNode != NULL ) {
-            font   = MemHandleLock( *handleNode );
+        MemHandle        fontHandle;
+        FontResourceKind kind;
+        fontHandle = GetFontResource( fontList->resourceType,
+                                      fontList->resourceID,
+                                      &kind );
+        if ( fontHandle != NULL ) {
+            font   = MemHandleLock( fontHandle );
             FntDefineFont( fontList->fontID, font );
-            ListAppend( customFontList, handleNode );
+            MemHandleUnlock( fontHandle );
+            if ( FONT_IS_ITALIC( fontList->fontID ) ) {
+                haveItalic[ fontList->fontID - START_ITALIC_USER_FONTS ] = true;
+            }
         }
         else {
-            SafeMemPtrFree( handleNode );
-            GrayFntSubstitute( fontList->fontID, fontList->substituteID );
+            if ( FONT_IS_ITALIC( fontList->fontID ) ) {
+                haveItalic[ fontList->fontID - START_ITALIC_USER_FONTS ] =
+                                                                          false;
+            }
+            else {
+                GrayFntSubstitute( fontList->fontID, fontList->substituteID );
+            }
         }
+        ReleaseFontResource( fontHandle, kind );
     }
 }
 
@@ -385,25 +465,6 @@ void LoadCustomFonts
 /* Release custom fonts from memory */
 void ReleaseCustomFonts( void )
 {
-    MemHandle* handleNode;
-    MemHandle* nextHandleNode;
-
-    if ( customFontList == NULL )
-        return;
-
-    handleNode = ListFirst( customFontList );
-    while ( handleNode != NULL ) {
-        if ( *handleNode != NULL ) {
-            MemHandleUnlock( *handleNode );
-            DmReleaseResource( *handleNode );
-        }
-        nextHandleNode = ListNext( customFontList, handleNode );
-        ListTakeOut( customFontList, handleNode );
-        SafeMemPtrFree( handleNode );
-        handleNode = nextHandleNode;
-    }
-    ListRelease( customFontList );
-    customFontList = NULL;
 }
 
 
@@ -411,13 +472,12 @@ void ReleaseCustomFonts( void )
 /* Refresh custom fonts in memory */
 void RefreshCustomFonts( void )
 {
-    if ( ! Support30() )
-        return;
-    ReleaseCustomFonts();
-    LoadCustomFonts( currentDeviceFont );
+    if ( Support30() )
+        LoadCustomFonts( currentDeviceFont );
 }
 
 
+
 /* Return the current font style */
 Int16 GetCurrentStyle( void )
 {
@@ -663,6 +723,37 @@ void SetFontStylesHandera( void )
 
 
 
+/* Change the current font to an italic style if available.  Return true
+   if successful. */
+Boolean SetFontItalic( void )
+{
+    FontID  font;
+    font = FntGetFont();
+    if ( FONT_IS_ITALIC( font ) )
+        return true;
+    if ( FONT_IS_USER( font ) && haveItalic[ font - START_USER_FONTS ] ) {
+        FntSetFont( font + ITALIC_FONT_DELTA );
+        return true;
+    }
+    return false;
+}
+
+
+
+/* Change the current font back to upright (non-italic) style.  Return 
+   true if we were in an italic style */
+Boolean EndFontItalic( void )
+{
+    FontID  font;
+    font = FntGetFont();
+    if ( ! FONT_IS_ITALIC( font ) )
+        return false;
+    FntSetFont( font - ITALIC_FONT_DELTA );
+    return true;
+}
+
+
+
 /* Display the example font character centred in the given object ID */
 void DisplayChar
     (
@@ -719,6 +810,148 @@ extern Int16 FontLineHeight( void )
 }
 
 
+
+#ifdef SUPPORT_VFS_FONTS
+/* Add a font in VFS */
+void AddVFSFont
+     (
+     const Char* name,
+     Char*       volumeLabel,
+     Char*       path
+     )
+{
+    if ( userFontDBList == NULL )
+        return;
+
+    if ( GetUserFontNumber( name, false, fontCacheOff ) == NO_SUCH_USER_FONT ) {
+        DBEntryType* dbListEntry;
+
+        dbListEntry                 = SafeMemPtrNew( sizeof( DBEntryType ) );
+        dbListEntry->volumeLabel    = SafeMemPtrNew( StrLen( volumeLabel ) + 1 );
+        dbListEntry->path           = SafeMemPtrNew( StrLen( path ) + 1 );
+        dbListEntry->onVFS          = true;
+        StrCopy( dbListEntry->name, name );
+        StrCopy( dbListEntry->volumeLabel, volumeLabel );
+        StrCopy( dbListEntry->path, path );
+
+        ListAppend( userFontDBList, dbListEntry );
+    }
+}
+#endif
+
+
+
+/* Get a font resource in the current user font */
+MemHandle GetFontResource
+     (
+     UInt32             resourceType,
+     UInt32             resourceID,
+     FontResourceKind*  resourceKindP
+     )
+{
+#ifdef SUPPORT_VFS_FONTS
+    if ( currentFontIsOnVFS ) {
+        Err         err;
+        MemHandle   handle;
+
+        err = VFSFileDBGetResource( currentVFSFontRef, resourceType, resourceID,
+                  &handle );
+        if ( err == errNone ) {
+            *resourceKindP = fontResourceVFS;
+            return handle;
+        }
+    }
+#endif
+    *resourceKindP = fontResourceStorage;
+    return DmGetResource( resourceType, resourceID );
+}
+
+
+
+
+/* Release a font resource */
+void ReleaseFontResource
+     (
+     MemHandle        handle,
+     FontResourceKind kind
+     )
+{
+     if ( handle == NULL )
+         return;
+#ifdef SUPPORT_VFS_FONTS
+     if ( kind == fontResourceVFS ) {
+         MemHandleFree( handle );
+     }
+     else
+#endif
+     {
+         DmReleaseResource( handle );
+     }
+     return;
+}
+
+
+
+static void ClearUserFontNameList( void )
+{
+    if ( userFontNames == NULL )
+        return;
+
+    SafeMemPtrFree( userFontNames );
+    userFontNames = NULL;
+}
+
+
+
+static Int16 DBEntryCompare
+    (
+    void*  a,
+    void*  b,
+    Int32  other
+    )
+{
+    return StrCompare( ( *( DBEntryType** )a )->name,
+                       ( *( DBEntryType** )b )->name );
+}
+
+
+
+static void SortAndMakeUserFontNameList( void )
+{
+    UInt16        i;
+    DBEntryType*  dbListEntry;
+    DBEntryType** userFontArray;
+
+    currentUserFontNumber = NO_SUCH_USER_FONT;
+    ClearUserFontNameList();
+    if ( userFontDBList == NULL )
+        return;
+
+    numberOfUserFonts = ListSize( userFontDBList );
+    if ( numberOfUserFonts == 0 )
+        return;
+
+    userFontArray   = SafeMemPtrNew( numberOfUserFonts * sizeof( DBEntryType* ) );
+    userFontNames   = SafeMemPtrNew( numberOfUserFonts * sizeof( Char* ) );
+    dbListEntry     = ListFirst( userFontDBList );
+    for ( i = 0; i < numberOfUserFonts ; i++ ) {
+        userFontArray[ i ] = dbListEntry;
+        dbListEntry        = ListNext( userFontDBList, dbListEntry );
+    }
+    SysQSort( userFontArray, numberOfUserFonts, sizeof( DBEntryType* ),
+        DBEntryCompare, 0 );
+    ListDelete( userFontDBList );
+    userFontDBList = ListCreate();
+    for ( i = 0 ; i < numberOfUserFonts ; i++ ) {
+        ListAppend( userFontDBList, userFontArray[ i ] );
+        userFontNames[ i ] = userFontArray[ i ]->name;
+    }
+    SafeMemPtrFree( userFontArray );
+}
+
+
+
+
 /* Initialize internal lists of user font prcs */
 void InitializeUserFontDBs( void )
 {
@@ -727,18 +960,15 @@ void InitializeUserFontDBs( void )
     LocalID             dbID;
     Err                 err;
     DBEntryType*        dbListEntry;
-    Int16               i;
 
     userFontDBList = ListCreate();
 
-    if ( NULL == userFontDBList )
+    if ( userFontDBList == NULL )
         return;
 
-    numberOfUserFonts = 0;
-
     err = DmGetNextDatabaseByTypeCreator( true, &stateInfo,
-            (UInt32) UserFontResourceType, (UInt32) ViewerAppID,
-            false, &cardNo, &dbID );
+              (UInt32) UserFontResourceType, (UInt32) ViewerAppID,
+              false, &cardNo, &dbID );
 
     while ( err == errNone ) {
         dbListEntry = SafeMemPtrNew( sizeof( DBEntryType ) );
@@ -747,49 +977,110 @@ void InitializeUserFontDBs( void )
         if ( err == errNone ) {
             dbListEntry->cardNo = cardNo;
             dbListEntry->dbID   = dbID;
+            dbListEntry->onVFS  = false;
             ListAppend( userFontDBList, dbListEntry );
-            numberOfUserFonts++;
         }
-        else
+        else {
             SafeMemPtrFree( dbListEntry );
+        }
         err = DmGetNextDatabaseByTypeCreator( false, &stateInfo,
                 (UInt32) UserFontResourceType, (UInt32) ViewerAppID, false,
                 &cardNo, &dbID );
     }
+    SortAndMakeUserFontNameList();
+}
 
-    if ( 0 < numberOfUserFonts ) {
-        dbListEntry = ListFirst( userFontDBList );
-        userFontNames = SafeMemPtrNew( numberOfUserFonts * sizeof( Char* ) );
-        for ( i = 0; i < numberOfUserFonts && NULL != dbListEntry ; i++ ) {
-            userFontNames[ i ] = SafeMemPtrNew( 1 +
-                                     StrLen( dbListEntry->name ) );
-            StrCopy( userFontNames[ i ], dbListEntry->name );
-            dbListEntry = ListNext( userFontDBList, dbListEntry );
+
+
+#ifdef SUPPORT_VFS_FONTS
+Boolean ScannedVFSFonts( void )
+{
+    return scannedVFS;
+}
+
+
+
+void PostprocessVFSFonts( void )
+{
+    SortAndMakeUserFontNameList();
+    scannedVFS = true;
+}
+#endif
+
+
+
+/* Scan VFS for fonts */
+void ScanVFSFonts( void )
+{
+#ifdef SUPPORT_VFS_FONTS
+    if ( ! scannedVFS ) {
+        ErrTry {
+            EnumerateCards( ENUMERATECARD_FONTS );
+            PostprocessVFSFonts();
         }
-        numberOfUserFonts = i;
+        ErrCatch( UNUSED_PARAM( err ) ) {
+        } ErrEndCatch
     }
+#endif
 }
 
 
 
 /* Go from name to position in user font list, or NO_SUCH_USER_FONT */
-Int16 GetUserFontNumber(
-    Char* name    /* name of user font to search for */
+Int16 GetUserFontNumber
+    (
+    const Char* name,      /* name of user font to search for */
+    Boolean     scanVFS,   /* do we scan VFS if we need to? */
+    Int16       cache      /* which cache to add to */
     )
 {
-    Int16 i;
-    if ( numberOfUserFonts == 0 )
+    Int16        i;
+    DBEntryType* dbListEntry;
+    
+    if ( *name == '\0' )
         return NO_SUCH_USER_FONT;
-    for ( i = 0 ; i < numberOfUserFonts ; i++ )
-        if ( ! StrCompare( userFontNames[ i ] , name ) )
+
+    dbListEntry = ListFirst( userFontDBList );
+    i = 0;
+    while( dbListEntry != NULL ) {
+        DBEntryType* nextDBListEntry;
+        nextDBListEntry = ListNext( userFontDBList, dbListEntry );
+        if ( ! StrCompare( dbListEntry->name, name ) ) {
+#ifdef SUPPORT_VFS_FONTS
+            if ( cache != fontCacheOff && dbListEntry->onVFS ) {
+                VFSFontCacheAddEntry( dbListEntry->name,
+                    dbListEntry->volumeLabel,
+                    dbListEntry->path,
+                    cache );
+            }
+#endif
             return i;
+        }
+        dbListEntry = nextDBListEntry;
+        i++;
+    }
+#ifdef SUPPORT_VFS_FONTS
+    if ( ! scannedVFS && scanVFS ) {
+        if ( VFSFontCacheFindEntry( name ) ) {
+            /* we found it and added it in at the end of the list */
+            SortAndMakeUserFontNameList();
+            return GetUserFontNumber( name, true, cache );
+        }
+        else {
+            ScanVFSFonts();
+            return GetUserFontNumber( name, false, cache );
+        }
+    }
+#endif
     return NO_SUCH_USER_FONT;
 }
 
 
 
+
 /* Go from position in user font list to name */
-Char* GetUserFontName(
+Char* GetUserFontName
+    (
     Int16 number    /* number of user font */
     )
 {
@@ -803,12 +1094,14 @@ Char* GetUserFontName(
 
 /* Load a particular user font prc */
 /* Can take NO_SUCH_USER_FONT as an argument */
-void LoadUserFont(
+void LoadUserFont
+    (
     Int16 number  /* number of user font to load */
     )
 {
     DBEntryType*  dbListEntry;
-    CloseUserFont();
+    if ( number == currentUserFontNumber )
+        return;
     if ( numberOfUserFonts == 0 || number == NO_SUCH_USER_FONT ) {
         currentUserFontNumber = NO_SUCH_USER_FONT;
         RefreshCustomFonts();
@@ -816,12 +1109,36 @@ void LoadUserFont(
     }
     dbListEntry = ListGet( userFontDBList, number + 1 );
     if ( dbListEntry != NULL ) {
-        currentUserFontDBRef = DmOpenDatabase( dbListEntry->cardNo,
-                                   dbListEntry->dbID, dmModeReadOnly );
-        currentUserFontNumber = number;
+#ifdef SUPPORT_VFS_FONTS
+        if ( dbListEntry->onVFS ) {
+            Err err;
+            err = VFSFileOpen( FindVolRefNum( dbListEntry->volumeLabel ),
+                      dbListEntry->path, vfsModeRead, &currentVFSFontRef );
+            if ( err == errNone ) {
+                currentUserFontNumber = number;
+                currentFontIsOnVFS    = true;
+            }
+            else {
+                currentUserFontNumber  = NO_SUCH_USER_FONT;
+                currentFontIsOnVFS     = false;
+            }
+        }
+        else
+#endif
+        {
+            currentUserFontDBRef   = DmOpenDatabase( dbListEntry->cardNo,
+                                         dbListEntry->dbID, dmModeReadOnly );
+            currentUserFontNumber  = number;
+#ifdef SUPPORT_VFS_FONTS
+            currentFontIsOnVFS     = false;
+#endif
+        }
     }
     else {
-        currentUserFontNumber = NO_SUCH_USER_FONT;
+        currentUserFontNumber  = NO_SUCH_USER_FONT;
+#ifdef SUPPORT_VFS_FONTS
+        currentFontIsOnVFS     = false;
+#endif
     }
     return;
 }
@@ -832,7 +1149,16 @@ void LoadUserFont(
 void CloseUserFont( void )
 {
   if ( currentUserFontNumber != NO_SUCH_USER_FONT ) {
-        DmCloseDatabase( currentUserFontDBRef );
+#ifdef SUPPORT_VFS_FONTS
+        if ( currentFontIsOnVFS ) {
+            VFSFileClose( currentVFSFontRef );
+            currentFontIsOnVFS = false;
+        }
+        else
+#endif
+        {
+            DmCloseDatabase( currentUserFontDBRef );
+        }
         currentUserFontNumber = NO_SUCH_USER_FONT;
   }
 }
@@ -852,7 +1178,6 @@ void DeinitializeUserFontDBs( void )
 {
     DBEntryType*  dbListEntry;
     DBEntryType*  nextDBListEntry;
-    Int16         i;
 
     CloseUserFont();
 
@@ -862,6 +1187,10 @@ void DeinitializeUserFontDBs( void )
     dbListEntry = ListFirst( userFontDBList );
     while ( dbListEntry != NULL ) {
         nextDBListEntry = ListNext( userFontDBList, dbListEntry );
+        if ( dbListEntry->onVFS ) {
+            SafeMemPtrFree( dbListEntry->volumeLabel );
+            SafeMemPtrFree( dbListEntry->path );
+        }
         SafeMemPtrFree( dbListEntry );
         dbListEntry = nextDBListEntry;
     }
@@ -869,17 +1198,15 @@ void DeinitializeUserFontDBs( void )
     ListDelete( userFontDBList );
     userFontDBList = NULL;
 
-    for ( i = 0 ; i < numberOfUserFonts ; i++ )
-        SafeMemPtrFree( userFontNames[ i ] );
-
-    SafeMemPtrFree( userFontNames );
-    userFontNames = NULL;
+    ClearUserFontNameList();
 }
 
 
 
+
 /* Initialize drop down list of user fonts */
-void InitializeUserFontList(
+void InitializeUserFontList
+    (
     UInt16 listID    /* ID number of list for user font names */
     )
 {
@@ -904,3 +1231,147 @@ Int16 GetNumberOfUserFonts( void )
 {
     return numberOfUserFonts;
 }
+
+
+
+#ifdef SUPPORT_VFS_FONTS
+/* The following functions support the VFS font name cache.
+   This is a Palm database where each records consists of three
+   null-terminated strings.  The first string is the font name.
+   The next is the volume label.  The third is the path. */
+
+/* Look for an entry in the VFS font cache.  If found, add it
+   to the current font list. */
+static Boolean VFSFontCacheFindEntry
+     (
+     const Char*   name
+     )
+{
+    LocalID   cacheID;
+    UInt16    cardNo;
+    DmOpenRef ref;
+    UInt16    numRecords;
+    UInt16    i;
+
+    cardNo     = 0;
+    cacheID    = DmFindDatabase( cardNo, vfsFontCacheName );
+    if ( cacheID == 0 )
+        return false;
+    ref        = DmOpenDatabase( cardNo, cacheID, dmModeReadOnly );
+    numRecords = DmNumRecords( ref );
+
+    for ( i = 0 ; i < numRecords ; i++ ) {
+         MemHandle record;
+         Char*     data;
+         record = DmGetRecord( ref, i );
+
+         if ( record != NULL ) {
+             data = MemHandleLock( record );
+             if ( 3 < MemHandleSize( record ) && ! StrCompare( data, name ) ) {
+                 Char*   volumeLabel;
+                 Char*   path;
+                 Err     err;
+                 FileRef fontRef;
+                 UInt32  type;
+                 UInt32  creatorID;
+                 Boolean found;
+
+                 found       = false;
+                 volumeLabel = data + StrLen( data ) + 1;
+                 path        = volumeLabel + StrLen( volumeLabel ) + 1;
+                 err         = VFSFileOpen( FindVolRefNum( volumeLabel ),
+                                   path, vfsModeRead, &fontRef );
+
+                 if ( err == errNone ) {
+                     Char thisName[ dmDBNameLength + 1 ];
+                     err = VFSFileDBInfo( fontRef, thisName, NULL, NULL, NULL,
+                               NULL, NULL, NULL, NULL, NULL, &type,
+                               &creatorID, NULL );
+                     VFSFileClose( fontRef );
+                     if ( err == errNone && type == UserFontResourceType &&
+                          creatorID == ViewerAppID &&
+                          ! StrCompare( thisName, name ) ) {
+                         AddVFSFont( name, volumeLabel, path );
+                         found = true;
+                     }
+                 }
+                 MemHandleUnlock( record );
+                 DmReleaseRecord( ref, i, false );
+                 DmCloseDatabase( ref );
+                 return found;
+             }
+             MemHandleUnlock( record );
+             DmReleaseRecord( ref, i, false );
+         }
+    }
+    DmCloseDatabase( ref );
+    return false;
+}
+
+
+
+static void VFSFontCacheAddEntry
+     (
+     const Char* name,
+     const Char* volumeLabel,
+     const Char* path,
+     Int16       cache
+     )
+{
+    LocalID   cacheID;
+    UInt16    cardNo;
+    DmOpenRef ref;
+    UInt16    nameLen;
+    UInt16    volumeLabelLen;
+    UInt16    pathLen;
+    UInt16    dataLen;
+    MemHandle record;
+    Char*     recordData;
+
+    if ( cache == fontCacheOff )
+        return;
+
+    cardNo     = 0;
+    cacheID    = DmFindDatabase( cardNo, vfsFontCacheName );
+    if ( cacheID == 0 ) {
+        UInt16 i;
+        Err    err;
+        err    = DmCreateDatabase( cardNo, vfsFontCacheName, ViewerAppID,
+                     PlkrVFSFontCacheType, false );
+        if ( err != errNone )
+            return;
+        cacheID = DmFindDatabase( cardNo, vfsFontCacheName );
+        if ( cacheID == 0 )
+            return;
+        ref = DmOpenDatabase( cardNo, cacheID, dmModeReadWrite );
+        for ( i = 0 ; i < numFontCaches ; i++ ) {
+            UInt16  index;
+            index = i;
+            DmNewRecord( ref, &index, 1 );
+            DmReleaseRecord( ref, index, false );
+        }
+    }
+    else {
+        ref = DmOpenDatabase( cardNo, cacheID, dmModeReadWrite );
+    }
+
+    nameLen        = StrLen( name );
+    volumeLabelLen = StrLen( volumeLabel );
+    pathLen        = StrLen( path );
+    dataLen        = nameLen + volumeLabelLen + pathLen + 3;
+
+    record = DmResizeRecord( ref, cache, dataLen );
+
+    if ( record != NULL ) {
+        recordData = MemHandleLock( record );
+        DmWrite( recordData, 0, name, nameLen + 1 );
+        DmWrite( recordData, nameLen + 1, volumeLabel,
+            volumeLabelLen + 1 );
+        DmWrite( recordData, nameLen + volumeLabelLen + 2, path,
+            pathLen + 1 );
+        MemHandleUnlock( record );
+    }
+    DmReleaseRecord( ref, cache, false );
+    DmCloseDatabase( ref );
+}
+#endif
