@@ -1,5 +1,5 @@
 /*
- * $Id: fontform.c,v 1.30 2003/09/08 02:18:47 chrish Exp $
+ * $Id: fontform.c,v 1.41 2004/05/08 09:04:53 nordstrom Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -31,6 +31,8 @@
 #include "util.h"
 #include "history.h"
 #include "grayfont.h"
+#include "metadocument.h"
+#include "mainform.h"
 
 #include "fontform.h"
 
@@ -209,7 +211,6 @@ static void FontFormInit( void )
     UInt16          i;
     ControlType*    ctl;
     ListType*       list;
-    Char            formTitle[ 30 ];
 
     if ( IsFormMain( Prefs()->lastForm ) ) {
         cameFromLibrary   = false;
@@ -235,6 +236,10 @@ static void FontFormInit( void )
 #endif
     else
         buttonCount = BUTTON_COUNT_OS2;
+        
+#ifdef SUPPORT_VFS_FONTS
+    ScanVFSFonts();
+#endif
 
     form        = FrmGetFormPtr( fontFormID );
     buttonList  = SafeMemPtrNew( buttonCount * sizeof *buttonList );
@@ -336,8 +341,6 @@ static void FontFormInit( void )
         userFontButtonNumber   = 5;
     }
 
-    HanderaConvertForm( form );
-
     if ( cameFromLibrary ) {
         HideFontFormObject( frmFontLineSpacingLabel, false );
         HideFontFormObject( frmFontLineSpacingPopup, true );
@@ -376,10 +379,6 @@ static void FontFormInit( void )
 
     FrmDrawForm( form );
 
-    /* Move the title to the right, to make room for the help icon */
-    StrPrintF( formTitle, "      %s", FrmGetTitle( form ) );
-    FrmSetTitle( form, formTitle );
-
     /* Highlight the pushbutton that reflects the currently set font */
     for ( i = 0; i < buttonCount; i++ ) {
        if ( fontModePref == buttonList[ i ].fontMode ) {
@@ -389,7 +388,8 @@ static void FontFormInit( void )
     }
 
     if ( userFontButtonNumber != NO_USER_FONT_BUTTON ) {
-        userFontSelection = GetUserFontNumber( prefsUserFontName );
+        userFontSelection = GetUserFontNumber( prefsUserFontName, true, 
+                                fontCacheOff );
         if ( userFontSelection == NO_SUCH_USER_FONT )
             userFontSelection = 0;
         list        = GetObjectPtr( frmFontUserFontList );
@@ -446,8 +446,10 @@ void FontDoOK( void )
     updateDoc    = false;
     lineSpacing += MINIMAL_LINE_SPACING;
 
-    if ( userFontButtonNumber != NO_USER_FONT_BUTTON && GetNumberOfUserFonts() != 0 &&
-        userFontSelection != GetUserFontNumber( prefsUserFontName )
+    if ( userFontButtonNumber != NO_USER_FONT_BUTTON && 
+        GetNumberOfUserFonts() != 0 &&
+        userFontSelection != GetUserFontNumber( prefsUserFontName, true, 
+                                 fontCacheOff )
         )
         changeUserFont = true;
     else
@@ -463,8 +465,9 @@ void FontDoOK( void )
         ) {
         if ( ! IsFullscreenformActive() ) {
             if ( ! cameFromLibrary ) {
-                if ( Prefs()->individualDocumentFonts )
+                if ( Prefs()->individualDocumentFonts ) {
                     GetHistoryPtr()->font = buttonList[ selected ].fontMode;
+                }
                 else {
                     Prefs()->fontModeMain = buttonList[ selected ].fontMode;
                     Prefs()->defaultFontModeMain = Prefs()->fontModeMain;
@@ -519,11 +522,12 @@ void FontDoOK( void )
     }
 
     if ( IsFullscreenformActive() ) {
+        FrmUpdateForm( frmFullscreen, frmRedrawUpdateCode );
+        ReRenderAllIfNeeded();
         FrmReturnToForm( frmFullscreen );
     } else {
         if ( ! cameFromLibrary && Prefs()->individualDocumentFonts &&
-            CtlGetValue( GetObjectPtr( frmFontAsDefault ) )
-          ) {
+             CtlGetValue( GetObjectPtr( frmFontAsDefault ) ) ) {
             Prefs()->defaultFontModeMain     = GetHistoryPtr()->font;
             Prefs()->defaultLineSpacing      = GetHistoryPtr()->lineSpacing;
             Prefs()->defaultParagraphSpacing =
@@ -536,8 +540,12 @@ void FontDoOK( void )
         }
 
         FrmReturnToForm( Prefs()->lastForm );
-        if ( updateDoc )
-            FrmUpdateForm( Prefs()->lastForm, frmRedrawUpdateCode );
+        if ( updateDoc ) {
+            if ( IsFormMain( Prefs()->lastForm ) )
+                SetMainFormUpdate();
+            else
+                FrmUpdateForm( Prefs()->lastForm, frmRedrawUpdateCode );
+        }
     }
 
     SafeMemPtrFree( buttonList );
@@ -622,6 +630,7 @@ Boolean FontFormHandleEvent
             }
             break;
         }
+
         case ctlSelectEvent:
             switch ( event->data.ctlEnter.controlID ) {
                 case frmFont1:
@@ -669,12 +678,31 @@ Boolean FontFormHandleEvent
                     break;
             }
             break;
+
+        case winEnterEvent:
+            handled = ResizeHandleWinEnterEvent();
+            break;
+
+        case winDisplayChangedEvent:
+            handled = ResizeHandleWinDisplayChangedEvent();
+            break;
+
+        case winExitEvent:
+            handled = ResizeHandleWinExitEvent();
+            break;
+
         case frmOpenEvent:
+#ifdef HAVE_SILKSCREEN
+            ResizeHandleFrmOpenEvent();
+#endif
             FontFormInit();
             handled = true;
             break;
 
         case frmCloseEvent:
+#ifdef HAVE_SILKSCREEN
+            ResizeHandleFrmCloseEvent();
+#endif
             SafeMemPtrFree( buttonList );
             break;
 
