@@ -1,5 +1,5 @@
 /*
- * $Id: fontconv.c,v 1.10 2003/09/19 12:53:56 prussar Exp $
+ * $Id: fontconv.c,v 1.12 2004/02/13 16:24:16 prussar Exp $
  *
  * The true home of this program is palmfontconv.sourceforge.net.  If you
  * find it elsewhere and improve it, please send a patch to
@@ -51,6 +51,7 @@ DWord glyphLength,glyphLength2;
 Byte  *glyph,*glyph2,*glyph3;
 Word  *locationTable,*locationTable2;
 int numChars;
+Word  ignore;
 
 struct {
   Byte offset;
@@ -62,6 +63,7 @@ struct fonttype {
   Byte type2;
   Byte afnx;
   Byte text;
+  Byte infoOnly;
   Byte includelores;
 } in,out;
 
@@ -70,7 +72,7 @@ FontTypeV2 header;
 #define SWAPBYTES(x,y) { unsigned char __store; __store=(y); (y)=(x); (x)=__store; }
 
 enum labels { LfontType=0, LmaxWidth, LfRectWidth, LfRectHeight,
-     Lascent, Ldescent };
+     Lascent, Ldescent, LleftKerning, Ladvance };
 
 struct {
    char *fieldname;
@@ -83,6 +85,8 @@ struct {
                {"fRectHeight",&header.fRectHeight,0},
                {"ascent", &header.ascent, 0},
                {"descent", &header.descent, 0},
+               {"leftKerning", &ignore, 0},
+               {"advance", &ignore, 0}
 };
 #define NUM_VALUES (sizeof values / sizeof *values)
 
@@ -169,7 +173,7 @@ void dodefault(int valuenum,Word value)
 
 void parse(char *s,struct fonttype *f)
 {
-    f->includelores=f->text=0;
+    f->includelores=f->text=f->infoOnly=0;
     switch(s[0])
     {
         case 't':
@@ -177,6 +181,11 @@ void parse(char *s,struct fonttype *f)
             f->type2=0;
             f->resolution=LORES;
             break;
+        case 'i':
+            f->text=1;
+            f->type2=0;
+            f->resolution=LORES;
+            f->infoOnly=1;
         case 'H':
             f->resolution=HIRES;
             f->type2=1;
@@ -194,6 +203,11 @@ void parse(char *s,struct fonttype *f)
             f->type2=1;
             f->includelores=1;
             break;
+        case '1':
+            f->resolution=LORES;
+            f->type2=1;
+            f->includelores=0;
+            break;
         default:
             err("Font type must be H or L or l rather than %s.\n",s);
     }
@@ -259,7 +273,7 @@ void detect(FILE *f,char *s,struct fonttype *t,char *mods)
                 err("Error in font file!");
         }
     }
-    printf("%s: ",s);
+    printf("// %s: ",s);
     if(t->afnx) printf("afnx ");
     if(t->resolution==HIRES) printf("hires ");
     if(t->resolution==LORES) printf("lores ");
@@ -359,9 +373,8 @@ char *trim(char *s)
 
 void usage(void)
 {
-        err("fontconv [-l] inputname [-i lowresfont] H|Ha|2|2a|l|t outputname\n\n"
-   	      "This converts a Palm font from the input format into the output format.\n"
-   	      "You need to specify both the input and the output formats.\n"
+        err("fontconv [-l] inputname [-i lowresfont] H|Ha|2|2a|l|t|i outputname\n"
+   	      "This converts a Palm font into the specified output format.\n"
    	      " -l: Take the low-res (default: hi-res) part of the FontTypeV2 font\n"
    	      " -i lowresfont: Use this low-res font to fill out the low-res portion of\n"
    	      "     output.  Must be used in conjunction with output options 2 or 2a.\n"
@@ -370,17 +383,17 @@ void usage(void)
           " Ha: hi-res portion of FontTypeV2 font in afnx format\n"
           " La: lo-res portion of FontTypeV2 font in afnx format\n"
           "  l: version 1 type font\n"
-          "  2: put the font into the hi-res portion of FontTypeV2 font and\n"
-          "     fill out the low-res portion with scaled or -i specified font\n"
-          " 2a: same as 2, except for afnx format\n"
+          "2/2a: put the font into the hi-res portion of FontTypeV2 font (afnx for 2a)"
+          "     and fill out the low-res portion with scaled or -i specified font\n"
+          "1/1a: put the font into lo-res portion of FontTypeV2 font (afnx for 1a)"
           "  t: text-editable/readable format\n"
+          "  i: output font header and other font information\n"
           "The inputname/outputname can be - for stdin/stdout for text format only.\n"
-          "To take a lo-res font and convert it into an OS5 hi-res font, you can \n"
-          "run:\n"
+          "To take a lo-res font and convert it into an OS5 hi-res font, you can run:\n"
           "   fontconv input.bin H output.bin\n"
           "If you want the OS5 font to have a lo-res bitmap (a very ugly and\n"
           "unreadable one) you can do:\n"
-          "   fontconv input.bin 2 output.bin\n.");
+          "   fontconv input.bin 2 output.bin.");
 }
 
 DWord findresolution(FILE *f,Word resolution,int afnx)
@@ -568,7 +581,8 @@ int main(int argc, char **argv)
         }
         else
         {
-            glyphLength*=4;
+            if ( in.resolution == HIRES )
+                glyphLength*=4;
             if(header.version!=1) err("Error in font.");
             glyphBitsOffset=findresolution(f,in.resolution,in.afnx);
             if(in.afnx)
@@ -617,29 +631,41 @@ int main(int argc, char **argv)
     else {
         toStdout = 0;
         g=fopen(argv[argpos],out.text?"w":"wb");
-    }    
+    }
     doreverse=endiancheck(!out.afnx);
-    if(!out.text && out.type2 && out.resolution!=HIRES) err("not supported output combination");
-    if(!out.text && out.type2 && out.resolution==HIRES)
+    if(!out.text && out.type2)
     {
+    	if ( out.resolution == LORES && out.includelores ) {
+    	    err("Invalid output option combiantion.");
+    	}
         header.fontType|=0x200;
         position=0;
         locationTable2=mymalloc(2*numChars+2);
         owTable2=mymalloc(2*numChars);
-        for(i=0;i<numChars;i++)
-        {
+        if ( out.resolution == HIRES ) {
+            for(i=0;i<numChars;i++)
+            {
+                locationTable2[i]=position;
+                owTable2[i].offset=owTable[i].offset;
+                if(owTable[i].width>=0)
+                    owTable2[i].width=(owTable[i].width+1)/2*2;
+                else
+                    owTable[i].width=-1;
+                position+=(locationTable[i+1]-locationTable[i]+1)/2*2;
+            }
             locationTable2[i]=position;
-            owTable2[i].offset=owTable[i].offset;
-            if(owTable[i].width>=0)
-                owTable2[i].width=(owTable[i].width+1)/2*2;
-            else
-                owTable[i].width=-1;
-            position+=(locationTable[i+1]-locationTable[i]+1)/2*2;
+            rowWords2    = (position+31)/32*2;
+            fRectHeight2 = (header.fRectHeight+1)/2*2;
+            glyph2=mymalloc(glyphLength2=rowWords2*2*fRectHeight2);
+        }
+        else {
+            memcpy( locationTable2, locationTable, 2*numChars + 2 );
+            memcpy( owTable2, owTable, 2*numChars );
+            rowWords2    = header.rowWords;
+            fRectHeight2 = header.fRectHeight;
+            glyphLength2 = rowWords2 * 2 * fRectHeight2;
+            glyph2       = mymalloc(glyphLength2);
         }
-        locationTable2[i]=position;
-        rowWords2=(position+31)/32*2;
-        fRectHeight2=(header.fRectHeight+1)/2*2;
-        glyph2=mymalloc(glyphLength2=rowWords2*2*fRectHeight2);
         if(out.includelores)
         {
             glyph3=mymalloc(glyphLength2/4);
@@ -664,7 +690,6 @@ int main(int argc, char **argv)
         }
         else
             doubleputpixel=0;
-        header.maxWidth=(header.maxWidth+1)/2;
         for(i=0;i<(int)glyphLength2;i++) glyph2[i]=0;
         if(doubleputpixel)
             for(i=0;i<(int)glyphLength2/4;i++) glyph3[i]=0;
@@ -673,31 +698,39 @@ int main(int argc, char **argv)
             x2=locationTable2[i];
             for(x=locationTable[i];x<locationTable[i+1];x++,x2++)
             {
-                for(k=0;k<header.fRectHeight;k++)
+                for(k=0;k<header.fRectHeight;k++) {
                     putpixel(x2,k,getpixel(x,k));
+                }
             }
         }
-        for(i=0;i<numChars;i++)
-        {
-         locationTable2[i]/=2;
-         owTable2[i].width/=2;
+        if ( out.resolution==HIRES ) {
+            header.maxWidth=(header.maxWidth+1)/2;
+            for(i=0;i<numChars;i++)
+            {
+             locationTable2[i]/=2;
+             owTable2[i].width/=2;
+            }
+            locationTable2[i]/=2;
+            header.rowWords=rowWords2/2;
+            if(header.fRectHeight % 2)
+              header.descent++;
+            if(header.descent % 2)
+            {
+                header.ascent++;
+                header.descent--;
+            }
+            header.maxWidth=(header.maxWidth+1)/2*2;
+            header.fRectHeight=fRectHeight2/2;
+            header.fRectWidth=(header.fRectWidth+1)/2;
+            header.ascent/=2;
+            header.descent/=2;
+            header.version=1;
+            header.densityCount=1+!!out.includelores;
         }
-        locationTable2[i]/=2;
-        header.rowWords=rowWords2/2;
-        if(header.fRectHeight % 2)
-          header.descent++;
-        if(header.descent % 2)
-        {
-            header.ascent++;
-            header.descent--;
+        else {
+            header.version=1;
+            header.densityCount=1;
         }
-        header.maxWidth=(header.maxWidth+1)/2*2;
-        header.fRectHeight=fRectHeight2/2;
-        header.fRectWidth=(header.fRectWidth+1)/2;
-        header.ascent/=2;
-        header.descent/=2;
-        header.version=1;
-        header.densityCount=1+!!out.includelores;
         glyphBitsOffset=sizeof header+(out.afnx?8:6)*header.densityCount+4*numChars+2+2+2+(out.afnx?2:0);
         header.owTLoc=((sizeof header+(out.afnx?8:6)*header.densityCount+2*numChars+2+2+(out.afnx?2:0)) - OWT_DELTA)/2;
         reverse_word_array(&header,sizeof header);
@@ -712,7 +745,7 @@ int main(int argc, char **argv)
             fwrite(&glyphBitsOffset,1,4,g);
             glyphBitsOffset=reverse_dword(glyphBitsOffset)+glyphLength2/4;
         }
-        w=reverse_word(HIRES);
+        w=reverse_word( out.resolution );
         fwrite(&w,1,2,g);
         fixalign2out(g);
         glyphBitsOffset=reverse_dword(glyphBitsOffset);
@@ -728,10 +761,28 @@ int main(int argc, char **argv)
     }
     else if(out.text)
     {
+        long avgWidth;
+        long totalWidth;
+        int  charCount;
         header.fontType &= ~0x200;
+        totalWidth = 0;
+        charCount  = 0;
+        for ( i = 0 ; i < numChars ; i++ ) {
+            if ( 0 < owTable[ i ].width ) {
+                totalWidth += owTable[ i ].width;
+                charCount++;
+            }
+        }
+        if ( charCount == 0 )
+            avgWidth = 0;
+        else
+            avgWidth = 100 * totalWidth / charCount;
+        if ( ! out.infoOnly )
+            fprintf(g,"// generated by fontconv\n");
         fprintf(g,
-              "// generated by fontconv\n"
               "fontType %d\n"
+              "firstChar %d\n"
+              "lastChar %d\n"
               "maxWidth %d\n"
               "kernMax %d\n"
               "nDescent %d\n"
@@ -739,9 +790,20 @@ int main(int argc, char **argv)
               "fRectHeight %d\n"
               "ascent %d\n"
               "descent %d\n"
-              "leading %d\n\n",header.fontType,header.maxWidth,header.kernMax,
+              "leading %d\n"
+              "averageWidth %ld.%02ld\n",
+                header.fontType,header.firstChar,
+                header.lastChar,header.maxWidth,header.kernMax,
                 header.nDescent,header.fRectWidth,header.fRectHeight,
-                header.ascent,header.descent,header.leading);
+                header.ascent,header.descent,header.leading,
+                avgWidth / 100,avgWidth % 100);
+        if ( out.infoOnly ) {
+            if ( ! toStdout )
+                fclose( g );
+            return 0;
+        }
+        fputc('\n',g);
+
         for(i=0;i<numChars;i++)
         {
             if(owTable[i].width<=0 || locationTable[i+1]==locationTable[i]) continue;
