@@ -1,14 +1,14 @@
 /*
- * $Id: fullscreenform.c,v 1.27.4.1 2003/10/23 00:19:53 prussar Exp $
+ * $Id: fullscreenform.c,v 1.48 2004/05/08 09:04:54 nordstrom Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -38,6 +38,7 @@
 #include "screen.h"
 #include "table.h"
 #include "util.h"
+#include "handera.h"
 
 #include "fullscreenform.h"
 
@@ -86,8 +87,10 @@ static Int16         tableX;
 static Int16         tableY;
 static Boolean       penDownInFrame;
 static Boolean       fullscreenformActive = false;
-static RectangleType fullBounds;     
-static Boolean       fastEnough;	/* SuperVZ or better */
+static RectangleType fullBounds;
+static Boolean       fastEnough;    /* SuperVZ or better */
+static Coord         originalBackButtonHeight = 0;
+static Coord         originalBackButtonWidth = 0;
 
 
 /***********************************************************************
@@ -97,7 +100,8 @@ static Boolean       fastEnough;	/* SuperVZ or better */
  ***********************************************************************/
 static Boolean FsFormInit( void ) FULLSCREENFORM_SECTION;
 static void MoveImage( Coord x, Coord y ) FULLSCREENFORM_SECTION;
-static void DrawImage( ImageType* image, Coord drawX, Coord drawY, RectangleType* bounds ) FULLSCREENFORM_SECTION;
+static void DrawImage( ImageType* image, Coord drawX, Coord drawY, 
+                RectangleType* bounds ) FULLSCREENFORM_SECTION;
 static void DrawScanner( void ) FULLSCREENFORM_SECTION;
 static void DrawButton( void ) FULLSCREENFORM_SECTION;
 static void FreeMemory( void ) FULLSCREENFORM_SECTION;
@@ -107,7 +111,7 @@ static Boolean HandleFsKeyDown(const EventType* event) FULLSCREENFORM_SECTION;
 static Boolean HandleFsPenUp( const Coord x, const Coord y) FULLSCREENFORM_SECTION;
 static Boolean HandleFsPenMove( const Coord x, const Coord y) FULLSCREENFORM_SECTION;
 static Boolean HandleFsPenDown( const Coord x, const Coord y) FULLSCREENFORM_SECTION;
-static Boolean FsHandleMenuEvent ( UInt16 itemID ) FULLSCREENFORM_SECTION; 
+static Boolean FsHandleMenuEvent ( UInt16 itemID ) FULLSCREENFORM_SECTION;
 
 
 /* Initialize the image form */
@@ -116,10 +120,12 @@ static Boolean FsFormInit( void )
     MemHandle       imageHandle;
     Err             err;
     Coord           maxFrameSize;
-    UInt32 processorType;
+    UInt32          processorType;
+    UInt16          backButtonIndex;
+    RectangleType   newBounds;
 
     err = FtrGet( sysFileCSystem, sysFtrNumProcessorID, &processorType );
-    if ( err == errNone && processorType >= 0x00040000 )
+    if ( err == errNone && 0x00040000 <= processorType )
         fastEnough = true;
     else
         fastEnough = false;
@@ -136,10 +142,22 @@ static Boolean FsFormInit( void )
 
     fsForm = FrmGetFormPtr( GetValidForm( frmFullscreen ) );
 
+    backButtonIndex = FrmGetObjectIndex( fsForm, frmFullscreenBack );
+
+    FrmHideObject( fsForm, backButtonIndex );
+
     FrmDrawForm( fsForm );
 
     screenWindow = WinGetDrawWindow();
-    WinGetDrawWindowBounds( &screenBounds );
+
+    screenBounds.topLeft.x = 0;
+    screenBounds.topLeft.y = 0;
+    screenBounds.extent.x  = RotMaxExtentX();
+    screenBounds.extent.y  = RotMaxExtentY();
+    newBounds              = screenBounds;
+    if ( fastEnough )
+        RotateRectangleInPlace( &newBounds );
+    WinSetBounds( screenWindow, &newBounds );
     HiResAdjustBounds( &screenBounds, palmHiRes );
 
     bufferWindow = NULL;
@@ -172,32 +190,51 @@ static Boolean FsFormInit( void )
         frameBounds.extent.y = (UInt32) imageBounds.extent.y *
             (UInt32) frameBounds.extent.x / (UInt32) imageBounds.extent.x;
     }
-    frameBounds.topLeft.x = MaxExtentX() - frameBounds.extent.x - 1;
-    frameBounds.topLeft.y = MaxExtentY() - frameBounds.extent.y - 1;
-
-    FrmGetObjectBounds( fsForm, FrmGetObjectIndex( fsForm, frmFullscreenBack ),
-                        &ctlBounds );
+    frameBounds.topLeft.x = RotMaxExtentX() - frameBounds.extent.x - 1;
+    frameBounds.topLeft.y = RotMaxExtentY() - frameBounds.extent.y - 1;
 
+    /* adjust position of back button for displays taller than 160 */
+    if ( originalBackButtonHeight == 0 ) {
+        FrmGetObjectBounds( fsForm, backButtonIndex, &ctlBounds );
+        HiResAdjustBounds( &ctlBounds, sonyHiRes );
+        originalBackButtonWidth  = ctlBounds.extent.x;
+        originalBackButtonHeight = ctlBounds.extent.y;
+    }
+    ctlBounds.topLeft.x = 1;
+    ctlBounds.topLeft.y = RotMaxExtentY() - originalBackButtonHeight - 1;
+    ctlBounds.extent.x  = originalBackButtonWidth;
+    ctlBounds.extent.y  = originalBackButtonHeight;
+    
     if ( fastEnough ) {
-        CtlGlueSetFont( GetObjectPtr( frmFullscreenBack ), symbolFont );
+        ControlType* backButton;
+
+        backButton = GetObjectPtr( frmFullscreenBack );
+        CtlGlueSetFont( backButton, symbolFont );
         switch ( Prefs()->rotate ) {
             case ROTATE_PLUS90:
-                CtlSetLabel( GetObjectPtr( frmFullscreenBack ), "\006" );
+                CtlSetLabel( backButton, "\006" );
                 break;
+
             case ROTATE_MINUS90:
-                CtlSetLabel( GetObjectPtr( frmFullscreenBack ), "\005" );
+                CtlSetLabel( backButton, "\005" );
                 break;
+
             case ROTATE_ZERO:
             default:
-                CtlSetLabel( GetObjectPtr( frmFullscreenBack ), "\003" );
+                CtlSetLabel( backButton, "\003" );
                 break;
         }
-        FrmGetObjectBounds( fsForm, FrmGetObjectIndex( fsForm,
-                            frmFullscreenBack ), &ctlBounds );
         RotateRectangleInPlace( &ctlBounds );
-        FrmSetObjectBounds( fsForm, FrmGetObjectIndex( fsForm,
-                            frmFullscreenBack ), &ctlBounds );
     }
+    if ( IsHiResTypeSony( HiResType() ) ) {
+        ctlBounds.topLeft.x /= 2;
+        ctlBounds.topLeft.y /= 2;
+        ctlBounds.extent.x  /= 2;
+        ctlBounds.extent.y  /= 2;
+    }
+    FrmSetObjectBounds( fsForm, backButtonIndex, &ctlBounds );
+
+    FrmShowObject( fsForm, backButtonIndex );
 
     ctlBounds.topLeft.x--;
     ctlBounds.topLeft.y--;
@@ -239,19 +276,22 @@ void FsAdjustVerticalOffset
     Int16   adjustment  /* adjustment in pixels */
     )
 {
-    imageBounds.topLeft.y -= adjustment;
+    Coord x;
+    Coord y;
 
-    if ( ( imageBounds.extent.y - MaxExtentY() ) <= imageBounds.topLeft.y )
-        imageBounds.topLeft.y = imageBounds.extent.y - MaxExtentY();
-    if ( imageBounds.topLeft.y < 0 )
-        imageBounds.topLeft.y = 0;
+    x = screenBounds.topLeft.x;
+    y = screenBounds.topLeft.y - adjustment;
 
-    DrawImage( image, -imageBounds.topLeft.x, -imageBounds.topLeft.y,
-               &fullBounds );
+    y = min( imageBounds.extent.y - screenBounds.extent.y, y );
+    y = max( 0, y );
+
+    imageBounds.topLeft.x = screenBounds.topLeft.x = x;
+    imageBounds.topLeft.y = screenBounds.topLeft.y = y;
+
+    DrawImage( image, -x, -y, &fullBounds );
 
     if ( IsLargeTable() )
-        CopyTableAnchors( image->reference, imageBounds.topLeft.x,
-                          imageBounds.topLeft.y );
+        CopyTableAnchors( image->reference, x, y );
 }
 
 
@@ -270,16 +310,14 @@ static void MoveImage
         screenWindow = WinSetDrawWindow( bufferWindow );
 
     DrawImage( image, -x, -y, &fullBounds );
-    if ( Prefs()->visualAid ) {
-        DrawButton();
-        DrawScanner();
-    }
 
     if ( bufferWindow != NULL ) {
         WinSetDrawWindow( screenWindow );
         WinCopyRectangle( bufferWindow, screenWindow, &bufferBounds, 0, 0,
             winPaint );
     }
+    DrawButton();
+    DrawScanner();
 }
 
 
@@ -332,30 +370,39 @@ static void DrawImage
             Coord           offsetY;
 
             RctGetIntersection( bounds, &screenBounds, &useMe);
-            if ( useMe.extent.x && useMe.extent.y ) {
+            if ( 0 < useMe.extent.x && 0 < useMe.extent.y ) {
                 if ( ! Support35() ) {
                     WinDrawBitmap( thisImage->bitmap, drawX, drawY );
                 }
                 else {
+                    /* crop the bitmap so that the rotation code only
+                       rotates the visible part */
                     offsetX = useMe.topLeft.x - screenBounds.topLeft.x;
                     offsetY = useMe.topLeft.y - screenBounds.topLeft.y;
                     drawWindow = WinCreateOffscreenWindow( useMe.extent.x,
-                                 useMe.extent.y, screenFormat, &error );
-                    screenWindow = WinSetDrawWindow( drawWindow );
-                    WinDrawBitmap( thisImage->bitmap, drawX - offsetX,
-                                   drawY - offsetY );
-                    bitmap = WinGetBitmap( drawWindow );
-                    WinSetDrawWindow( screenWindow );
-                    if ( fastEnough )
-                        RotDrawBitmap( bitmap, offsetX, offsetY );
-                    else
-                        GeneralWinDrawBitmap( bitmap, offsetX, offsetY );
-                    WinDeleteWindow( drawWindow, false );
+                                     useMe.extent.y, screenFormat, &error );
+                    if ( error == errNone ) {
+                        screenWindow = WinSetDrawWindow( drawWindow );
+                        WinDrawBitmap( thisImage->bitmap, drawX - offsetX,
+                                       drawY - offsetY );
+                        bitmap = WinGetBitmap( drawWindow );
+                        WinSetDrawWindow( screenWindow );
+                        if ( fastEnough )
+                            RotDrawBitmap( bitmap, offsetX, offsetY );
+                        else
+                            GeneralWinDrawBitmap( bitmap, offsetX, offsetY );
+                        WinDeleteWindow( drawWindow, false );
+                    }
+                    else {
+                        if ( fastEnough )
+                            RotDrawBitmap( thisImage->bitmap, drawX, drawY );
+                        else
+                            GeneralWinDrawBitmap( thisImage->bitmap, drawX, drawY );
+                    }
                 }
             }
         }
-
-        /* Specifically erase any whitespace beyound the image's bounds */
+        /* Specifically erase any whitespace beyond the image's bounds */
         if ( imageBounds.extent.x < screenBounds.extent.x ) {
             whitespace.topLeft.y = 0;
             whitespace.topLeft.x = imageBounds.extent.x;
@@ -378,7 +425,6 @@ static void DrawImage
             else
                 WinEraseRectangle( &whitespace, 0 );
         }
-
         PalmSetCoordinateSystem( prevCoordSys );
     }
 }
@@ -438,7 +484,7 @@ Boolean FsDoSelectTypeAction
 
     handled = false;
 
-    switch( selection ) {
+    switch ( selection ) {
         case SELECT_GO_FORWARD:
             /* Currently you cannot go forward to anything from here */
             handled = true;
@@ -450,6 +496,7 @@ Boolean FsDoSelectTypeAction
             FrmReturnToForm( PREVIOUS_FORM );
             /* Not handled because we do want to run their normal functions
                from within DoSelectTypeAction() */
+            handled = false;
             break;
 
         case SELECT_GO_TO_TOP:
@@ -613,6 +660,7 @@ static Boolean HandleFsKeyDown(const EventType* event)
                 DoPageMove( -RotGetScrollValue() );
             handled = true;
             break;
+
 #ifdef HAVE_FIVEWAY_SDK
         case vchrNavChange:
             if ( NavKeyPressed( event, Left ) &&
@@ -635,6 +683,7 @@ static Boolean HandleFsKeyDown(const EventType* event)
             }
             break;
 #endif
+
         case spaceChr:
             if ( Prefs()->gestMode[ GESTURES_RIGHT ] != SELECT_NONE &&
                  Prefs()->gestures )
@@ -700,22 +749,23 @@ static Boolean HandleFsPenUp
     const Coord y
     )
 {
-    if ( IsLargeTable() ) {
-        if ( anchor != NOT_FOUND ) {
-            HighlightAnchor( anchor, ANCHOR_UNSELECTED );
-            if ( ( abs(x - tableX) < 4 ) && ( abs(y - tableY ) < 4 ) ) {
-                FsDoControlAction( anchor + 1 );
-                return true;
-            }
+    if ( IsLargeTable() && anchor != NOT_FOUND ) {
+        HighlightAnchor( anchor, ANCHOR_UNSELECTED );
+        if ( ( abs(x - tableX) < 4 ) && ( abs(y - tableY ) < 4 ) ) {
+            FsDoControlAction( anchor + 1 );
+            return true;
         }
-        CopyTableAnchors( image->reference, imageBounds.topLeft.x,
-                          imageBounds.topLeft.y );
     }
+
     anchor = NOT_FOUND;
 
     DrawImage( image, -imageBounds.topLeft.x, -imageBounds.topLeft.y,
                &fullBounds );
 
+    if ( IsLargeTable() )
+        CopyTableAnchors( image->reference, imageBounds.topLeft.x,
+                          imageBounds.topLeft.y );
+
     penDownInFrame = false;
 
     return true;
@@ -834,10 +884,9 @@ static Boolean HandleFsPenDown
     if ( penDownInFrame && HandleFsPenMove( x, y ) )
         return true;
 
-    if ( Prefs()->visualAid ) {
-        DrawButton();
-        DrawScanner();
-    }
+    DrawButton();
+    DrawScanner();
+
     return true;
 }
 
@@ -858,10 +907,10 @@ Boolean FsFormHandleEvent
 
     penX = event->screenX;
     penY = event->screenY;
-    if ( fastEnough )
-        RotFromScreenXY( &penX, &penY );
     HiResAdjust( &penX, sonyHiRes );
     HiResAdjust( &penY, sonyHiRes );
+    if ( fastEnough )
+        RotFromScreenXY( &penX, &penY );
 
     switch ( event->eType ) {
         case ctlSelectEvent:
@@ -869,7 +918,7 @@ Boolean FsFormHandleEvent
             break;
 
         case keyDownEvent:
-            handled = HandleFsKeyDown(event);
+            handled = HandleFsKeyDown( event );
             break;
 
         case penDownEvent:
@@ -887,6 +936,7 @@ Boolean FsFormHandleEvent
             break;
 
         case winEnterEvent:
+            handled = ResizeHandleWinEnterEvent();
             if ( event->data.winEnter.enterWindow == (WinHandle) fsForm &&
                  event->data.winEnter.enterWindow ==
                                              (WinHandle) FrmGetFirstForm() )
@@ -896,17 +946,29 @@ Boolean FsFormHandleEvent
         case winExitEvent:
             if ( event->data.winExit.exitWindow == (WinHandle) fsForm )
                 menuActive = true;
+            handled = ResizeHandleWinExitEvent();
             break;
 
-#ifdef HAVE_HANDERA_SDK
-        case displayExtentChangedEvent:
-            SilkHandleChange();
+        case frmUpdateEvent:
+            FreeMemory();
+            UpdateDisplayExtent();
+            menuActive = false;
+            if ( ! FsFormInit() ) {
+                /* Do something */
+            }
+            handled = true;
+            break;
+
+        case winDisplayChangedEvent:
+            handled = ResizeHandleWinDisplayChangedEvent();
             break;
-#endif
 
         case frmOpenEvent:
+#ifdef HAVE_SILKSCREEN
+            ResizeHandleFrmOpenEvent();
+#endif
             menuActive = false;
-            if ( FsFormInit() == false ) {
+            if ( ! FsFormInit() ) {
                 /* Do something */
             }
             handled = true;
@@ -914,6 +976,9 @@ Boolean FsFormHandleEvent
 
         case frmCloseEvent:
             FreeMemory();
+#ifdef HAVE_SILKSCREEN
+            ResizeHandleFrmCloseEvent();
+#endif
             handled = false;
             break;
 
@@ -940,8 +1005,10 @@ Boolean FsFormHandleEvent
 
 static void FreeMemory( void )
 {
-    if ( bufferWindow != NULL )
+    if ( bufferWindow != NULL ) {
         WinDeleteWindow( bufferWindow, false );
+        bufferWindow = NULL;
+    }
 
     if ( image->type == MULTIIMAGE ) {
         MultiImageNodeType* node;
