@@ -1,5 +1,5 @@
 /*
- * $Id: image.c,v 1.69.4.1 2003/10/12 11:39:37 nordstrom Exp $
+ * $Id: image.c,v 1.81 2004/02/17 01:22:28 chrish Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -85,7 +85,7 @@ static void GetImageError( ImageType* image, Char* text ) IMAGE_SECTION;
  *      Private variables
  *
  ***********************************************************************/
-static MemHandle        fsImageHandle;
+static MemHandle        fsImageHandle = NULL;
 static ShowImagesType   imageStatus = IMAGE_LIMBO;
 
 
@@ -130,14 +130,20 @@ void GetImageMetrics
     imageHandle = GetImageHandle( reference );
     image = LockImage( imageHandle );
     if ( image->err != errNone || imageStatus == NO_IMAGES ||
-         ! ShowImages( GetHistoryCurrent() ) ) {
+         image == NULL || ! ShowImages( GetHistoryCurrent() ) ) {
         DrawImageText( image, width, height, NULL );
-        UnlockImage( image );
-        return;
+    }
+    else {
+        *width  = image->width;
+        *height = image->height;
     }
 
-    *width  = image->width;
-    *height = image->height;
+    if ( ! HasCacheNode( IMAGEHANDLE, image->reference ) &&
+         image->type != MULTIIMAGE ) {
+        UnlockImage( image );
+        FreeImageHandle( &imageHandle );
+        return;
+    }
 
     UnlockImage( image );
 }
@@ -152,9 +158,20 @@ void DrawInlineImage
     )
 {
     MemHandle imageHandle;
+    ImageType*     image;
 
     imageHandle = GetImageHandle( reference );
     DrawInlineImageByHandle( imageHandle, tContext, width );
+
+    image = LockImage( imageHandle );
+    if ( ! HasCacheNode( IMAGEHANDLE, image->reference ) &&
+         image->type != MULTIIMAGE ) {
+        MemHandleUnlock( imageHandle );
+        FreeImageHandle( &imageHandle );
+        return;
+    }
+
+    UnlockImage( image );
 }
 
 
@@ -170,14 +187,15 @@ static void DrawInlineImageByHandle
 
     image = LockImage( imageHandle );
     if ( image->err != errNone || imageStatus == NO_IMAGES ||
+          image == NULL ||
          ( ! ShowImages( GetHistoryCurrent() ) &&
-           tContext->writeMode != NoDraw ) ) {
+           tContext->writeMode != WRITEMODE_NO_DRAW ) ) {
         DrawImageText( image, width, NULL, tContext );
         UnlockImage( image );
         return;
     }
 
-    if ( tContext->writeMode == DrawChar ) {
+    if ( tContext->writeMode == WRITEMODE_DRAW_CHAR ) {
         if ( image->type == MULTIIMAGE ) {
             MultiImageNodeType* node;
 
@@ -246,7 +264,7 @@ static Boolean LoadFullScreenImageByHandle
 
     image = LockImage( imageHandle );
     result = true;
-    if ( image->err != errNone ) {
+    if ( image == NULL  || image->err != errNone ) {
         Char errorText[ 255 ];
 
         GetImageError( image, errorText );
@@ -298,7 +316,14 @@ MemHandle GetImageHandle
     if ( imageHandle != NULL )
         return imageHandle;
 
-    imageHandle = SafeMemHandleNew( sizeof( ImageType ) );
+    ErrTry {
+        imageHandle = SafeMemHandleNew( sizeof( ImageType ) );
+    }
+    ErrCatch( UNUSED_PARAM(err) ) {
+        FrmAlert( warnInsufficientMemory );
+        return NULL;
+    } ErrEndCatch
+
     image = MemHandleLock( imageHandle );
     MemSet( image, sizeof( ImageType ), 0 );
 
@@ -310,13 +335,14 @@ MemHandle GetImageHandle
     image->totalImages = 1;
 
     image->err = errNone;
-    switch( record->type ) {
+    switch ( record->type ) {
         case DATATYPE_TBMP_COMPRESSED:
             image->type = UNCOMPRESSED;
             ErrTry {
                 image->data.bitmapHandle = Uncompress( record );
             }
             ErrCatch( err ) {
+                FrmAlert( warnInsufficientMemory );
                 image->err = err;
             } ErrEndCatch
             break;
@@ -330,6 +356,8 @@ MemHandle GetImageHandle
             image->type = MULTIIMAGE;
             image->data.multiList = ReadMultiImageRecord( record,
                 &image->totalImages );
+            if ( image->data.multiList == NULL )
+                image->err = dmErrMemError;
             break;
 
         default:
@@ -362,7 +390,7 @@ MemHandle GetImageHandle
                    quick successive references of the sub-images directly... */
                 node->imageHandle = GetImageHandle( node->reference );
                 subImage = LockImage( node->imageHandle );
-                if ( subImage->err != errNone ) {
+                if ( image == NULL || subImage->err != errNone ) {
                     image->err = subImage->err;
                     UnlockImage( subImage );
                     break;
@@ -415,17 +443,19 @@ MemHandle GetImageHandle
             /* Locking at this point is only necessary to ensure we have
                a valid image->bitmap */
             LockImage( imageHandle );
-            if ( NotSupportedImageCompression( image->bitmap ) ) {
-                image->err = errBadImageCompression;
-            }
-            else {
-                BmpGlueGetDimensions( image->bitmap, &image->width,
-                    &image->height, NULL );
-                image->pixelDepth = BmpGlueGetBitDepth( image->bitmap );
-                if ( GetMaxBitDepth() < image->pixelDepth )
-                    image->err = errImageTooHighBitDepth;
-                else
-                    OptimizeImage( image );
+            if ( image != NULL && image->err == errNone ) {
+                if ( NotSupportedImageCompression( image->bitmap ) ) {
+                    image->err = errBadImageCompression;
+                }
+                else {
+                    BmpGlueGetDimensions( image->bitmap, &image->width,
+                        &image->height, NULL );
+                    image->pixelDepth = BmpGlueGetBitDepth( image->bitmap );
+                    if ( GetMaxBitDepth() < image->pixelDepth )
+                        image->err = errImageTooHighBitDepth;
+                    else
+                        OptimizeImage( image );
+                }
             }
             UnlockImage( image );
         }
@@ -474,13 +504,16 @@ void SaveImageInStorageCache
     UInt32     bmpDataSize;
     MemHandle  cacheHandle;
     UInt8*     cachePtr;
+    Boolean    separateData;
 
     image = LockImage( imageHandle );
-    if ( image->err != errNone )
+
+    if ( image == NULL || image->err != errNone )
         return;
 
     if ( Support40() ) {
         BmpGetSizes( image->bitmap, &bmpDataSize, &bmpHeaderSize );
+        separateData = true;
     }
     else if ( Support35() ) {
         bmpHeaderSize = BmpSize( image->bitmap );
@@ -489,12 +522,16 @@ void SaveImageInStorageCache
         /* If we're dealing with a non-indirect bitmap then BmpSize() will
            return not only the header but the databits with its value. Easy to
            adjust. */
-        if ( bmpHeaderSize > bmpDataSize )
+        if ( bmpDataSize < bmpHeaderSize )
             bmpHeaderSize -= bmpDataSize;
+        separateData = true;
     }
     else {
-        UnlockImage( image );
-        return;
+        /* for OS <3.5, our bitmaps are not optimized and hence are
+           portable since they come right from the distiller */
+        bmpHeaderSize  = PortableBmpSize( image->bitmap );
+        bmpDataSize    = 0;
+        separateData   = false;
     }
 
     cacheHandle = AllocateCacheRecord( IMAGEHANDLE,
@@ -503,13 +540,14 @@ void SaveImageInStorageCache
     if ( cacheHandle == NULL ) {
         UnlockImage( image );
         return;
-    }    
+    }
 
     cachePtr = MemHandleLock( cacheHandle );
 
     DmWrite( cachePtr, 0, image->bitmap, bmpHeaderSize );
-    DmWrite( cachePtr, bmpHeaderSize, BmpGetBits( image->bitmap ),
-        bmpDataSize );
+    if ( separateData )
+        DmWrite( cachePtr, bmpHeaderSize, BmpGetBits( image->bitmap ),
+            bmpDataSize );
 
     MemHandleUnlock( cacheHandle );
 
@@ -532,7 +570,7 @@ void SetImageHandleType
     ReleaseImageHandle( imageHandle, KEEPALIVE_CACHE );
     image = MemHandleLock( imageHandle );
     image->type = type;
-    switch( image->type ) {
+    switch ( image->type ) {
         case DIRECT:
         case OPTIMIZED:
         case VERSION3_BMP:
@@ -569,7 +607,7 @@ ImageType* LockImage
     actualImage = false;
 
     if ( image->err == errNone ) {
-        switch( image->type ) {
+        switch ( image->type ) {
             case DIRECT:
             case OPTIMIZED:
             case VERSION3_BMP:
@@ -616,7 +654,7 @@ MemHandle UnlockImage
         return NULL;
 
     if ( image->err == errNone ) {
-        switch( image->type ) {
+        switch ( image->type ) {
             case UNCOMPRESSED:
             case CACHE_STORAGE:
                 MemHandleUnlock( image->data.bitmapHandle );
@@ -646,7 +684,7 @@ static void ReleaseImageHandle
 
     image = MemHandleLock( imageHandle );
 
-    switch( image->type ) {
+    switch ( image->type ) {
         case MULTIIMAGE:
             MSG( _( "Releasing MULTIIMAGE handle\n" ) );
             ListRelease( image->data.multiList );
@@ -658,7 +696,7 @@ static void ReleaseImageHandle
 
         case OPTIMIZED:
             MSG( _( "Releasing OPTIMIZED handle\n" ) );
-            BmpDelete( image->data.bitmap );
+            PortableBmpDelete( image->data.bitmap );
             image->data.bitmap = NULL;
             break;
 
@@ -714,6 +752,41 @@ void FreeImageHandle
 
 
 
+void ReleaseFsImageHandle( void )
+{
+    ImageType*          imagePtr;
+    ImageType*          subImagePtr;
+    MultiImageNodeType* node;
+
+    /* Reset/RemoveCache will take care of CACHE_STORAGE */
+    if ( fsImageHandle != NULL ) {
+        imagePtr = MemHandleLock( fsImageHandle );
+        if ( HasCacheNode( IMAGEHANDLE, imagePtr->reference ) ) {
+            MemHandleUnlock( fsImageHandle );
+        }
+        else if ( imagePtr->type == MULTIIMAGE ) {
+            node = ListFirst( imagePtr->data.multiList );
+            while ( node != NULL ) {
+                subImagePtr = MemHandleLock( node->imageHandle );
+                if ( subImagePtr->type != CACHE_STORAGE ) {
+                    MemHandleUnlock( node->imageHandle );
+                    FreeImageHandle( &(node->imageHandle) );
+                }
+                node = ListNext( imagePtr->data.multiList, node );
+            }
+            MemHandleUnlock( fsImageHandle );
+            FreeImageHandle( &fsImageHandle );
+        }
+        else if ( imagePtr->type != CACHE_STORAGE ) {
+            MemHandleUnlock( fsImageHandle );
+            FreeImageHandle( &fsImageHandle );
+        }
+        fsImageHandle = NULL;
+    }
+}
+
+
+
 static LinkedList ReadMultiImageRecord
     (
     const Header* imageRecord,
@@ -730,7 +803,15 @@ static LinkedList ReadMultiImageRecord
     Int16                tmp;
 
     list = ListCreate();
-    multiImageRecord = SafeMemPtrNew( imageRecord->size );
+
+    ErrTry {
+        multiImageRecord = SafeMemPtrNew( imageRecord->size );
+    }
+    ErrCatch( UNUSED_PARAM(err) ) {
+        FrmAlert( warnInsufficientMemory );
+        return NULL;
+    } ErrEndCatch
+
     MemMove( multiImageRecord, (UInt8*) imageRecord + sizeof( Header ),
         imageRecord->size );
 
@@ -742,7 +823,15 @@ static LinkedList ReadMultiImageRecord
     MSG( _( "totalImages: %d\n", *totalImages ) );
 
     for ( i = 0; i < *totalImages; i++ ) {
-        node = SafeMemPtrNew( sizeof( MultiImageNodeType ) );
+        ErrTry {
+            node = SafeMemPtrNew( sizeof( MultiImageNodeType ) );
+        }
+        ErrCatch( UNUSED_PARAM(err) ) {
+            FrmAlert( warnInsufficientMemory );
+            SafeMemPtrFree( multiImageRecord );
+            ListRelease( list );
+            return NULL;
+        } ErrEndCatch
         MemSet( node, sizeof( MultiImageNodeType ), 0 );
         /* The record's references are Int16 and we need to */
         /* move these into a Int32 in the structure.        */
@@ -783,8 +872,6 @@ Boolean OptimizeImage_OS35
     ImageType* image
     )
 {
-    return false;
-#if 0
     BitmapType* tempBitmapPtr;
     Err         err;
     WinHandle   tempWindow;
@@ -806,8 +893,8 @@ Boolean OptimizeImage_OS35
     if ( Prefs()->screenDepth < image->pixelDepth )
         image->pixelDepth = Prefs()->screenDepth;
 
-    tempBitmapPtr = BmpCreate( image->width, image->height, image->pixelDepth,
-        BmpGetColortable( image->bitmap ), &err );
+    tempBitmapPtr = PortableBmpCreate( image->width, image->height,
+        image->pixelDepth, NULL, &err );
     if ( tempBitmapPtr == NULL || err != errNone ) {
         MSG( _( "BmpCreate() failed! Error %d\n", err ) );
         return false;
@@ -815,7 +902,7 @@ Boolean OptimizeImage_OS35
 
     tempWindow = WinCreateBitmapWindow( tempBitmapPtr, &err );
     if ( tempWindow == NULL || err != errNone ) {
-        BmpDelete( tempBitmapPtr );
+        PortableBmpDelete( tempBitmapPtr );
         MSG( _( "WinCreateBitmapWindow() failed! Error %d\n", err ) );
         return false;
     }
@@ -835,7 +922,6 @@ Boolean OptimizeImage_OS35
         image->pixelDepth, (UInt32) BmpBitsSize( image->bitmap ) ) );
 
     return true;
-#endif
 }
 
 
@@ -909,7 +995,7 @@ static void GetImageError
     Char*      text
     )
 {
-    switch( image->err ) {
+    switch ( image->err ) {
         case errBadImageType:
             StrPrintF( text, "Bad image type" );
             break;
@@ -922,7 +1008,53 @@ static void GetImageError
             StrPrintF( text, "Bad image compression" );
             break;
 
+        case dmErrMemError:
+            StrPrintF( text, "Low dynamic memory" );
+            break;
+
+        case errZLibMemError:
+            StrPrintF( text, "Zlib memory error" );
+            break;
+
         default:
             StrPrintF( text, "0x%x", image->err );
     }
 }
+
+
+
+#ifdef HAVE_IMODE
+/* Draws an I-mode icon */
+void DisplayImode
+    (
+    DmOpenRef*   plkrImodeDB,
+    TextContext* tContext,
+    UInt16       index,
+    Int16*       width,
+    Int16*       height
+    )
+{
+    MemHandle     imageHandle;
+    BitmapType*   imagePtr;
+    Coord         imageWidth;
+    Coord         imageHeight;
+
+    imageHandle = DmQueryRecord( plkrImodeDB, index );
+
+    if ( imageHandle != NULL ) {
+      imagePtr = MemHandleLock( imageHandle );
+
+      BmpGlueGetDimensions( imagePtr, &imageWidth,
+                &imageHeight, NULL );
+      *width  = imageWidth;
+      *height = imageHeight;
+
+      RotDrawBitmap( imagePtr, tContext->cursorX,
+             tContext->cursorY - imageHeight );
+
+    MemHandleUnlock( imageHandle );
+    FreeRecordHandle( &imageHandle );
+    }
+}
+#endif
+
