@@ -1,5 +1,5 @@
 /*
- * $Id: mainform.c,v 1.125 2003/10/07 20:57:15 nordstrom Exp $
+ * $Id: mainform.c,v 1.147 2004/05/08 09:04:54 nordstrom Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -42,7 +42,7 @@
 #include "prefscontrol.h"
 #include "resourceids.h"
 #include "search.h"
-#include "silkscreen.h"
+#include "DIA.h"
 #include "util.h"
 #include "screen.h"
 #include "genericfile.h"
@@ -55,6 +55,7 @@
 #include "rotate.h"
 #include "dimensions.h"
 #include "metadocument.h"
+#include "handera.h"
 
 #include "mainform.h"
 
@@ -106,6 +107,25 @@ static Boolean HandleKeyboardAction(WChar ch) MAINFORM_SECTION;
 
 
 
+/***********************************************************************
+ *
+ *      Private variables
+ *
+ ***********************************************************************/
+static Boolean forceMainFormUpdate = false;
+
+
+
+
+/* Main form needs updating when it becomes active. */
+void SetMainFormUpdate( void )
+{
+    forceMainFormUpdate = true;
+}
+
+
+
+
 /* Handle a keyboard press via the keyboard map */
 Boolean HandleKeyboardAction
     (
@@ -133,16 +153,9 @@ Boolean HandleCommonMenuItems
     handled = false;
     switch ( itemID ) {
         case mOptionsAbout:
-        {
-            FormType* form;
-
-            form = FrmInitForm( frmAbout );
-            HanderaConvertForm( form );
-            FrmDoDialog( form );
-            FrmDeleteForm( form );
+            FrmPopupForm( frmAbout );
             handled = true;
             break;
-        }
 
         case mOptionsContact:
             SetMailto( 0 );
@@ -247,6 +260,13 @@ static Boolean HandleMenuEvent
                 FrmPopupForm( frmBookmarks );
                 handled = true;
                 break;
+                
+#ifdef SUPPORT_WORD_LOOKUP
+            case mGoLookup:
+                DoSelectTypeAction( SELECT_WORD_LOOKUP );
+                handled = true;
+                break;
+#endif
 
             case mViewAutoscrollStart:
                 DoAutoscrollToggle( AUTOSCROLL_ON );
@@ -270,7 +290,8 @@ static Boolean HandleMenuEvent
 
             case mGoLibrary:
                 FrmGotoForm( GetValidForm( frmLibrary ) );
-                DeleteTableWindow();
+                ReleaseFsTableHandle();
+                ReleaseFsImageHandle();
                 handled = true;
                 break;
 
@@ -306,6 +327,7 @@ void MainFormInit( void )
     UpdateMargins();
     ReRenderAllIfNeeded();
     InitializeViewportBoundaries();
+    ResetActualAnchor();
 
     mainForm        = FrmGetFormPtr( GetMainFormId() );
 
@@ -318,7 +340,7 @@ void MainFormInit( void )
 
     ClearControlBounds();
     if ( Prefs()->toolbar == TOOLBAR_SILK ) {
-        SetSilkScreen();
+        HanderaSetSilkScreen();
     }
     else if ( Prefs()->toolbar != TOOLBAR_NONE ) {
         GetControlBounds( mainForm, bmpMenu, MENUCONTROL );
@@ -451,12 +473,7 @@ static Boolean HandleScrollbar
         SclGetScrollBar( scrollbar, &startingValue, &min, &max, &size );
         trueStartingValue = ScrollbarScale( startingValue );
     }
-    else if ( event->eType == sclRepeatEvent && Prefs()->dynamicScrollbar ) {
-        newPosition   = ScrollbarScale( event->data.sclRepeat.newValue );
-        delta         = trueStartingValue - newPosition;
-        startingValue = event->data.sclRepeat.newValue;
-    }
-    else if ( event->eType == sclExitEvent && ! Prefs()->dynamicScrollbar ) {
+    else if ( event->eType == sclExitEvent ) {
         newPosition = ScrollbarScale( event->data.sclRepeat.newValue );
         delta = trueStartingValue - newPosition;
     }
@@ -496,6 +513,48 @@ static Boolean MainFormWrong( void )
 
 
 
+
+#ifdef SUPPORT_WORD_LOOKUP
+static void GoSelectedWord( void )
+{
+     Char* word;
+
+     word = GetSelectedWord( NULL );
+     if ( word != NULL ) {
+        switch ( Prefs()->selectedWordAction ) {
+            case SELECT_WORD_SEARCH_FORM:
+                AddSearchString( word );
+                FrmPopupForm( frmSearch );
+                break;
+
+#ifdef SUPPORT_PPI
+            case SELECT_WORD_PPI:
+            {
+                LocalID dbID;
+
+                dbID = DmFindDatabase( 0, "PPI" );
+                if ( dbID != 0 ) {
+                    UInt32 outputValue;
+
+                    SysAppLaunch( 0, dbID, 0, sysAppLaunchCmdCustomBase,
+                        word, &outputValue );
+                }
+                break;
+            }
+#endif
+            case SELECT_WORD_TO_CLIPBOARD:
+                ClipboardAddItem( clipboardText, word, StrLen( word ) );
+                break;
+
+            default:
+                return;
+        }
+    }
+}
+#endif
+
+
+
 /* Event handler for the main form. */
 Boolean MainFormHandleEvent
     (
@@ -640,7 +699,8 @@ Boolean MainFormHandleEvent
                     /* The same argument can be given for '.' except that this
                        is only really here to benefit Graffiti 2 users. Normal
                        Graffiti users won't notice any difference */
-                    if ( Prefs()->gestMode[ GESTURES_TAP ] != SELECT_NONE )
+                    if ( Prefs()->gestMode[ GESTURES_TAP ] != SELECT_NONE &&
+                         Prefs()->gestures )
                         DoSelectTypeAction( Prefs()->gestMode[ GESTURES_TAP ] );
                     else
                         HandleKeyboardAction( event->data.keyDown.chr );
@@ -722,25 +782,55 @@ Boolean MainFormHandleEvent
             handled = true;
             break;
 
+
+#ifdef SUPPORT_WORD_LOOKUP
+        case pluckerSelectedWordEvent:
+            GoSelectedWord();
+            handled = true;
+            break;
+#endif
+
         case frmUpdateEvent:
             if ( MainFormWrong() ) {
                 handled = true;
                 break;
-            } 
+            }
             SetScreenMode();
             LineCacheClear();
             UpdateMargins();
             ReRenderAllIfNeeded();
             SetScrollbarLocation( FrmGetFormPtr( GetMainFormId() ) );
-            if ( event->data.frmUpdate.updateCode == frmRedrawUpdateCode ) {
+            if ( event->data.frmUpdate.updateCode == frmRedrawUpdateCode ||
+                 event->data.frmUpdate.updateCode == frmUpdateAnchors ) {
                 FormType* mainForm;
+                WriteModeType  mode;
+
+                if ( event->data.frmUpdate.updateCode == frmUpdateAnchors ) {
+                    mode = WRITEMODE_LAYOUT_ANCHORS;
+                }
+                else {
+                    mode                = WRITEMODE_DRAW_CHAR;
+                    forceMainFormUpdate = false;
+#ifdef HAVE_SILKSCREEN
+                    ResizeHandleFrmRedrawUpdateCode();
+#endif
+                }
 
                 mainForm = FrmGetFormPtr( GetMainFormId() );
-                FrmEraseForm( mainForm );
-                MainFormInit();
+                if ( mode == WRITEMODE_DRAW_CHAR ) {
+                    MainFormInit();
+                    /* If we are just coming back from frmFullscreen,
+                       don't reload GetHistoryCurrent().
+                       There is all ready a DoControlAction() pending. */
+                    if ( FrmGetFormId( FrmGetFirstForm() ) == frmFullscreen ) {
+                        handled = true;
+                        break;
+                    }
+                }
 
                 successful = ViewRecord( GetHistoryCurrent(),
-                                ! GetAddHistory(), NO_OFFSET, NO_OFFSET );
+                                ! GetAddHistory(), NO_OFFSET, NO_OFFSET,
+                                mode );
                 if ( ! successful ) {
                     FrmAlert( warnInsufficientMemory );
                     FrmGotoForm( GetValidForm( frmLibrary ) );
@@ -753,7 +843,7 @@ Boolean MainFormHandleEvent
                 reference = GetBookmarkRecordID();
                 SetVisitedLink( reference );
                 successful = ViewRecord( reference, ! GetAddHistory(),
-                                NO_OFFSET, NO_OFFSET );
+                                NO_OFFSET, NO_OFFSET, WRITEMODE_DRAW_CHAR );
                 if ( ! successful ) {
                     FrmAlert( warnInsufficientMemory );
                     FrmGotoForm( GetValidForm( frmLibrary ) );
@@ -762,23 +852,45 @@ Boolean MainFormHandleEvent
             }
             break;
 
+        case winDisplayChangedEvent:
+            handled = ResizeHandleWinDisplayChangedEvent();
+            break;
 
-#ifdef HAVE_HANDERA_SDK
-        case displayExtentChangedEvent:
-              LineCacheClear();
-              SilkHandleChange();
-              break;
+        case winExitEvent:
+            handled = ResizeHandleWinExitEvent();
+            break;
+
+        case winEnterEvent:
+#ifdef HAVE_SILKSCREEN
+            ResizeHandleWinEnterEvent();
 #endif
+            if ( forceMainFormUpdate ) {
+                forceMainFormUpdate = false;
+                if ( ! MainFormWrong() ) {
+                    EventType e;
+                    MemSet( &e, sizeof(EventType), 0 );
+                    e.eType                     = frmUpdateEvent;
+                    e.data.frmUpdate.formID     = GetMainFormId();
+                    e.data.frmUpdate.updateCode = frmRedrawUpdateCode;
+                    EvtAddUniqueEventToQueue( &e, 0, true );
+                }
+            }
+            handled = true;
+            break;
 
         case frmOpenEvent:
         {
             UInt16 reference;
+#ifdef HAVE_SILKSCREEN
+            ResizeHandleFrmOpenEvent();
+#endif
             if ( MainFormWrong() ) {
                 handled = true;
                 break;
             }
-            MainFormInit();
+            forceMainFormUpdate = false;
             SetScreenMode();
+            MainFormInit();
             LineCacheClear();
 
             /* Start with last visited page or home page at the very first
@@ -786,12 +898,14 @@ Boolean MainFormHandleEvent
             reference = GetHistoryCurrent();
             SetVisitedLink( reference );
             successful = ViewRecord( reference, ! GetAddHistory(), NO_OFFSET,
-                             NO_OFFSET );
+                             NO_OFFSET, WRITEMODE_DRAW_CHAR );
             if ( ! successful ) {
                 FrmAlert( warnInsufficientMemory );
-                DeleteTableWindow();
+                ReleaseFsTableHandle();
+                ReleaseFsImageHandle();
                 FrmGotoForm( GetValidForm( frmLibrary ) );
             }
+
             handled = true;
             break;
         }
@@ -800,13 +914,19 @@ Boolean MainFormHandleEvent
         {
             MemHandle handle;
 
-            ResetSilkScreen();
+            HanderaResetSilkScreen();
 
-            DeleteTableWindow();
+            ReleaseFsTableHandle();
+
+            ReleaseFsImageHandle();
 
             handle = GetCurrentRecord();
             FreeRecordHandle( &handle );
             ResetRecordReferences();
+            
+#ifdef HAVE_SILKSCREEN
+            ResizeHandleFrmCloseEvent();
+#endif
 
             handled = false;
             break;
