@@ -1,14 +1,14 @@
 /*
- * $Id: prefsform.c,v 1.62 2003/10/09 01:25:54 matto Exp $
+ * $Id: prefsform.c,v 1.72 2004/05/08 09:04:54 nordstrom Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -33,9 +33,11 @@
 #include "prefscontrol.h"
 #include "prefsgesture.h"
 #include "prefsjogdial.h"
+#include "prefslookup.h"
 #include "resourceids.h"
 #include "util.h"
 #include "os.h"
+#include "DIA.h"
 
 #include "prefsform.h"
 
@@ -46,7 +48,7 @@
  *
  ***********************************************************************/
 #define START_SECTION    0
-#define NUM_OF_SECTIONS  9
+#define NUM_OF_SECTIONS  10
 
 
 /***********************************************************************
@@ -91,19 +93,17 @@ static UInt16           mainObjectList[][ MAX_OBJECTS ] = {
       frmPrefsGeneralScreenDepthLabel,
       frmPrefsGeneralScreenDepthPopup,
       frmPrefsGeneralScrollbarLabel, frmPrefsGeneralScrollbarPopup, EOL },
-    { frmPrefsBrowsingLinkClick, frmPrefsBrowsingStrikethrough,
-      frmPrefsBrowsingUnderline, frmPrefsBrowsingDynamicScrollbar,
-      frmPrefsBrowsingVisualAid, frmPrefsBrowsingPageControlsLink,
-      frmPrefsBrowsingEnableSoftHyphens,
+    { frmPrefsBrowsingStrikethrough, frmPrefsBrowsingUnderline, 
+      frmPrefsBrowsingPageControlsLink, frmPrefsBrowsingEnableSoftHyphens,
       frmPrefsBrowsingForceDefaultColors, frmPrefsBrowsingAlignLabel,
       frmPrefsBrowsingAlignPopup, frmPrefsBrowsingAlignList, EOL },
     { frmPrefsLibrarySyncPolicyLabel, frmPrefsLibrarySyncPolicyPopup,
       frmPrefsLibrarySyncPolicyList, frmPrefsLibraryCategoryStyleLabel,
       frmPrefsLibraryCategoryStylePopup, frmPrefsLibrarySortByLabel,
       frmPrefsLibrarySortByPopup, frmPrefsLibrarySortOrderLabel,
-      frmPrefsLibrarySortOrderPopup, frmPrefsLibraryShowType,
-      frmPrefsLibraryShowDate, frmPrefsLibraryShowSize,
-      frmPrefsLibraryIndicateOpened, EOL },
+      frmPrefsLibrarySortOrderPopup, frmPrefsLibraryDateTime,
+      frmPrefsLibraryShowType, frmPrefsLibraryShowDate,
+      frmPrefsLibraryShowSize, frmPrefsLibraryIndicateOpened, EOL },
     { frmPrefsAutoscrollJumpLabel, frmPrefsAutoscrollJumpButton,
       frmPrefsAutoscrollJumpUpButton, frmPrefsAutoscrollJumpDownButton,
       frmPrefsAutoscrollModePopup, frmPrefsAutoscrollDirLabel,
@@ -113,7 +113,7 @@ static UInt16           mainObjectList[][ MAX_OBJECTS ] = {
       frmPrefsAutoscrollStayOn, EOL },
     { frmPrefsHardcopyActionLabel, frmPrefsHardcopyActionPopup,
       frmPrefsHardcopyRangeLabel, frmPrefsHardcopyRangePopup,
-      frmPrefsHardcopyLinkLabel, frmPrefsHardcopyLinkPopup, 
+      frmPrefsHardcopyLinkLabel, frmPrefsHardcopyLinkPopup,
       frmPrefsHardcopyTitle,
       EOL },
     { frmPrefsButtonHardKeys, frmPrefsButtonArrowKeys,
@@ -124,7 +124,9 @@ static UInt16           mainObjectList[][ MAX_OBJECTS ] = {
       bmpGestureUp, frmPrefsGestureLeft, bmpGestureLeft, frmPrefsGestureTap,
       bmpGestureTap, frmPrefsGestureRight, bmpGestureRight,
       frmPrefsGestureDown, bmpGestureDown, EOL },
-    { frmPrefsJogdialJogEnabled, frmPrefsJogdialSelectAction, EOL }
+    { frmPrefsJogdialJogEnabled, frmPrefsJogdialSelectAction, EOL },
+    { frmPrefsLookupAlwaysActive, frmPrefsLookupActionList,
+      frmPrefsLookupActionPopup, frmPrefsLookupActionLabel, EOL }
 };
 static HandlerType      handlerList[ NUM_OF_SECTIONS ] = {
     { PrefsGeneralPreferenceEvent, PrefsGeneralPalmEvent, strPrefsGeneralHelp },
@@ -139,6 +141,7 @@ static HandlerType      handlerList[ NUM_OF_SECTIONS ] = {
     { PrefsControlPreferenceEvent, PrefsControlPalmEvent, strPrefsControlHelp },
     { PrefsGesturePreferenceEvent, PrefsGesturePalmEvent, strPrefsGestureHelp },
     { PrefsJogdialPreferenceEvent, PrefsJogdialPalmEvent, strPrefsJogdialHelp },
+    { PrefsLookupPreferenceEvent, PrefsLookupPalmEvent, strPrefsLookupHelp },
 };
 
 
@@ -158,7 +161,6 @@ static void PrefsFormInit( void )
 #endif
 
     prefsForm = FrmGetFormPtr( frmPrefs );
-    HanderaConvertForm( prefsForm );
 
 #ifdef HAVE_HANDSPRING_SDK
     if ( HaveHsNav() )
@@ -227,9 +229,9 @@ void PrefsOpenSection
 
     mappedSection = handlerMap[ section ];
 
-    /* LOADing a section should only be ran the first time a preference section
+    /* LOADing a section should only be run the first time a preference section
        is opened. We should only need to initialize the controls and the
-       parameters only once ... */
+       parameters once ... */
     if ( ! ( openedPrefs & 1 << section ) ) {
         openedPrefs |= 1 << section;
         handlerList[ mappedSection ].PreferenceEvent( LOAD );
@@ -535,14 +537,32 @@ Boolean PrefsFormHandleEvent
             }
             break;
         }
+        
+        case winEnterEvent:
+            handled = ResizeHandleWinEnterEvent();
+            break;
+
+        case winDisplayChangedEvent:
+            handled = ResizeHandleWinDisplayChangedEvent();
+            break;
+
+        case winExitEvent:
+            handled = ResizeHandleWinExitEvent();
+            break;
 
         case frmOpenEvent:
+#ifdef HAVE_SILKSCREEN
+            ResizeHandleFrmOpenEvent();
+#endif
             PrefsFormInit();
             handled = true;
             break;
 
         case frmCloseEvent:
             PrefsReleaseSections();
+#ifdef HAVE_SILKSCREEN
+            ResizeHandleFrmCloseEvent();
+#endif
             handled = false;
             break;
 
@@ -550,12 +570,12 @@ Boolean PrefsFormHandleEvent
             handled = false;
     }
 
-    if ( handled == false )
+    if ( ! handled )
         handled = JogdialPrefHandler( event );
 
     /* If the event still hasn't been handled, dispatch it
        to the individual sections' event handlers */
-    if ( handled == false ) {
+    if ( ! handled ) {
         mappedSection = handlerMap[ currentSection ];
         handled = handlerList[ mappedSection ].PalmEvent( event );
     }
