@@ -1,5 +1,5 @@
 /*
- * $Id: ramfile.c,v 1.18 2003/10/07 20:57:15 nordstrom Exp $
+ * $Id: ramfile.c,v 1.19 2003/12/29 20:33:42 nordstrom Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -601,7 +601,7 @@ void RenameRAMDocument
 
     /* If a document with the given name doesn't already exists, rename it */
     dbID = DmFindDatabase( cardNo, newName );
-    THROW_IF( dbID != NULL, dmErrAlreadyExists );
+    THROW_IF( dbID != 0, dmErrAlreadyExists );
 
     dbID = DmFindDatabase( cardNo, docInfo->name );
     err  = DmSetDatabaseInfo( cardNo, dbID, newName, NULL, NULL,
