@@ -1,5 +1,5 @@
 /*
- * $Id: resourceids.h,v 1.121 2003/10/09 01:51:43 matto Exp $
+ * $Id: resourceids.h,v 1.140 2004/05/12 01:52:07 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -50,8 +50,9 @@
 
    Middle digit:
 
-   Plucker-supplied    0
-   User-substitutable  2
+   Plucker-supplied                0
+   User-substitutable - nonitalic  2
+   User-substitutable - italic     6
 
    First digit:
 
@@ -68,49 +69,54 @@
 #define narrowBoldFont          130
 #define narrowBoldFontID        0x205
 
-#define userStdFont             131
+/* 144 dpi Fonts : type v1 */
+#define stdFixedFont_sony       131
+#define stdFixedFontID_sony     0x406
+
+/* 144 dpi Fonts : type v2 */
+#define stdFixedFont_palm       132
+#define stdFixedFontID_palm     0x606
+#define tinyFont_palm           133
+#define tinyFontID_palm         0x60A
+#define tinyBoldFont_palm       134
+#define tinyBoldFontID_palm     0x60B
+#define smallFont_palm          135
+#define smallFontID_palm        0x60C
+#define smallBoldFont_palm      136
+#define smallBoldFontID_palm    0x60D
+
+
+#define START_USER_FONTS        137
+
+/* 72 dpi Fonts */
+#define userStdFont             START_USER_FONTS
 #define userStdFontID           0x220
-#define userBoldFont            132
+#define userBoldFont            138
 #define userBoldFontID          0x221
-#define userLargeFont           133
+#define userLargeFont           139
 #define userLargeFontID         0x222
-#define userLargeBoldFont       134
+#define userLargeBoldFont       140
 #define userLargeBoldFontID     0x223
-#define userNarrowFont          135
+#define userNarrowFont          141
 #define userNarrowFontID        0x224
-#define userFixedFont           136
+#define userFixedFont           142
 #define userFixedFontID         0x226
 
 /* 144 dpi Fonts : type v1 */
-#define stdFixedFont_sony       137
-#define stdFixedFontID_sony     0x406
-
-#define userStdFont_sony        138
+#define userStdFont_sony        143
 #define userStdFontID_sony      0x420
-#define userBoldFont_sony       139
+#define userBoldFont_sony       144
 #define userBoldFontID_sony     0x421
-#define userLargeFont_sony      140
+#define userLargeFont_sony      145
 #define userLargeFontID_sony    0x422
-#define userLargeBoldFont_sony  141
+#define userLargeBoldFont_sony  146
 #define userLargeBoldFontID_sony 0x423
-#define userNarrowFont_sony     142
+#define userNarrowFont_sony     147
 #define userNarrowFontID_sony   0x424
-#define userFixedFont_sony      143
+#define userFixedFont_sony      148
 #define userFixedFontID_sony    0x426
 
 /* 144 dpi Fonts : type v2 */
-#define stdFixedFont_palm       144
-#define START_LOADED_FONTS_144_V2     stdFixedFont_palm
-#define stdFixedFontID_palm     0x606
-#define tinyFont_palm           145
-#define tinyFontID_palm         0x60A
-#define tinyBoldFont_palm       146
-#define tinyBoldFontID_palm     0x60B
-#define smallFont_palm          147
-#define smallFontID_palm        0x60C
-#define smallBoldFont_palm      148
-#define smallBoldFontID_palm    0x60D
-
 #define userStdFont_palm        149
 #define userStdFontID_palm      0x620
 #define userBoldFont_palm       150
@@ -123,7 +129,19 @@
 #define userNarrowFontID_palm   0x624
 #define userFixedFont_palm      154
 #define userFixedFontID_palm    0x626
-#define END_LOADED_FONTS_144_V2     userFixedFontID_palm
+
+#define START_ITALIC_USER_FONTS   155
+#define ITALIC_FONT_DELTA         ( START_ITALIC_USER_FONTS - START_USER_FONTS )
+#define ITALIC_FONTID_MASK        0x0040
+#define END_ITALIC_USER_FONTS     ( START_ITALIC_USER_FONTS + \
+                                    ITALIC_FONT_DELTA - 1 )
+#define NUM_ITALIC_USER_FONTS     ( END_ITALIC_USER_FONTS + 1 - \
+                                    START_ITALIC_USER_FONTS )
+
+#define FONT_IS_ITALIC( x )       ( START_ITALIC_USER_FONTS <= ( x ) && \
+                                    ( x ) <= END_ITALIC_USER_FONTS )
+#define FONT_IS_USER( x )         ( START_USER_FONTS <= ( x ) && \
+                                    ( x ) <= END_ITALIC_USER_FONTS )
 
 /* Control IDs */
 #define HOMECONTROL             0
@@ -141,6 +159,7 @@
 #define AUTOSCROLLDECRCONTROL   12
 #define COPYTOMEMOCONTROL       13
 #define COMMANDKEYCONTROL       14
+#define SELECTEDWORDCONTROL     15
 
 /* Control bitmaps */
 #define bmpHome                 1000 + HOMECONTROL
@@ -158,6 +177,7 @@
 #define bmpAutoscrollDecr       1000 + AUTOSCROLLDECRCONTROL
 #define bmpCopyToMemo           1000 + COPYTOMEMOCONTROL
 /* RESERVED                     1000 + COMMANDKEYCONTROL */
+/* RESERVED                     1000 + SELECTEDWORDCONTROL */
 
 /* Other bitmaps */
 #define bmpWait                 2000
@@ -201,6 +221,7 @@
 #define bmpArrowRight           2038
 #define bmpArrowUp              2039
 #define bmpArrowDown            2040
+#define bmpLookup               2041
 
 /* Forms */
 #define frmMainTop              3000
@@ -217,7 +238,9 @@
 #define frmMainNoneHandera      3011
 #define frmMainBottomHanderaLow 3012
 #define frmMainBottomSonyLow    3013
-#define frmMainSonyFull         3014
+#define frmMainBottomSonyWide   3014
+#define frmMainSonyFull         3015
+#define frmMainDrawingArea      3016
 
 #define frmAbout                3100
 #define frmAboutOK              3101
@@ -242,12 +265,9 @@
 #define frmPrefsGeneralToolbarModePopup   3220
 #define frmPrefsGeneralToolbarModeList    3221
 
-#define frmPrefsBrowsingLinkClick         3225
 #define frmPrefsBrowsingStrikethrough     3226
 #define frmPrefsBrowsingUnderline         3227
-#define frmPrefsBrowsingDynamicScrollbar  3228
 #define frmPrefsBrowsingForceDefaultColors 3229
-#define frmPrefsBrowsingVisualAid         3230
 #define frmPrefsBrowsingPageControlsLink  3231
 #define frmPrefsBrowsingAlignLabel        3232
 #define frmPrefsBrowsingAlignPopup        3233
@@ -270,6 +290,7 @@
 #define frmPrefsLibraryShowDate           3253
 #define frmPrefsLibraryShowSize           3254
 #define frmPrefsLibraryIndicateOpened     3255
+#define frmPrefsLibraryDateTime           3256
 
 #define frmPrefsAutoscrollJumpLabel       3260
 #define frmPrefsAutoscrollJumpButton      3261
@@ -357,6 +378,11 @@
 #define frmPrefsJogdialPushDown           3347
 #define frmPrefsJogdialBack               3348
 
+#define frmPrefsLookupAlwaysActive 3350
+#define frmPrefsLookupActionLabel  3351
+#define frmPrefsLookupActionList   3352
+#define frmPrefsLookupActionPopup  3353
+
 #define frmLibrary              3400
 #define frmLibraryTable         3401
 #define frmLibraryScrollBar     3402
@@ -376,9 +402,10 @@
 #define frmRenameDocCancel      3416
 #define frmLibraryHandera       3417
 #define frmLibraryHanderaLow    3418
-#define frmLibraryCategoryPopup 3419
-#define frmLibraryCategoryList  3420
-#define frmLibraryNoDocuments   3421
+#define frmLibrarySonyWide      3419
+#define frmLibraryCategoryPopup 3420
+#define frmLibraryCategoryList  3421
+#define frmLibraryNoDocuments   3422
 
 #define frmDetails              3500
 #define frmDetailsOK            3501
@@ -456,10 +483,7 @@
 #define frmNewCatOK             3843
 #define frmNewCatCancel         3844
 #define frmNewCatName           3845
-#define frmSyncDocList          3846
-#define frmOpenDocList          3847
-#define frmSortDoc              3848
-#define frmExportLink           3849
+/* #define frmExportLink           3849 */
 
 #define frmEmail                3900
 #define frmEmailTo              3901
@@ -547,7 +571,6 @@
 #define frmFullscreenBack       4901
 
 
-
 /* Menus */
 #define menuMainForm            5000
 #define menuLibraryForm         5001
@@ -566,6 +589,7 @@
 #define mGoBottom               5109
 #define mGoLibrary              5110
 #define mGoDeleteDocument       5111
+#define mGoLookup               5112
 
 #define mViewTopToolbar         5201
 #define mViewBottomToolbar      5202
@@ -631,7 +655,8 @@
 #define strPrefsControl         6036
 #define strPrefsGesture         6037
 #define strPrefsJogdial         6038
-#define strPrefsGeneralHelp     6039
+#define strPrefsLookup          6039
+#define strPrefsGeneralHelp     6040
 #define strPrefsBrowsingHelp    3040
 #define strPrefsLibraryHelp     6041
 #define strPrefsAutoscrollHelp  6042
@@ -640,17 +665,21 @@
 #define strPrefsControlHelp     6045
 #define strPrefsGestureHelp     6046
 #define strPrefsJogdialHelp     6047
-#define strLibraryOpen          6048
-#define strLibraryCategorize    6049
-#define strLibraryDelete        6050
-#define strLibraryRename        6051
-#define strLibraryBeam          6052
-/* AVAILABLE                    6053 */
+#define strPrefsLookupHelp      6048
+#define strLibraryOpen          6049
+#define strLibraryCategorize    6050
+#define strLibraryDelete        6051
+#define strLibraryRename        6052
+#define strLibraryBeam          6053
 #define strLibraryUnread        6054
 #define strTblKeys              6055
 #define strFontHelp             6056
 #define strKeyboardHelp         6057
 #define strSearchHelp           6058
+#define strNoTransliteration    6059
+#define strSortDoc              6060
+#define strOpenDocList          6061
+#define strSyncDocList          6062
 
 /* Alerts */
 #define warnBrokenDocument      7001
@@ -659,7 +688,7 @@
 #define errCannotDeleteDoc      7004
 #define warnInsufficientMemory  7005
 #define confirmEndOfDoc         7006
-#define errStringNotFound       7007
+#define confirmEndOfPage        7007
 #define errBadMailto            7008
 #define errAlreadyExists        7009
 #define warnLowEmailMem         7010
@@ -692,13 +721,25 @@
 #define confirmDeleteReadDoc    7037
 #define errCannotDeleteReadDoc  7038
 
-/* Misc */
 /* #define frmShowResults          8001 */
 #define frmJumpToDoc            8002
 #define frmUpdateList           8003
 #define frmUpdateTable          8004
 #define frmViewRecord           8005
+#define frmUpdateAnchors        8006
+
+#define resizeIndex             8500
 
 /* Armlets */
 #define armDoSearch             9000
 #define armRotateBitmap         9001
+#define armColorizeBitmap       9002
+
+/* Custom events */
+#define pluckerSelectedWordEvent  ( firstUserEvent + 0 )
+#define pluckerClosePopupEvent    ( firstUserEvent + 1 )
+#define pluckerDeleteFormEvent    ( firstUserEvent + 2 )
+
+/* Miscellaneous */
+#define MAX_BOOKMARK_LEN  100
+
