@@ -1,5 +1,5 @@
 /*
- * $Id: resultform.h,v 1.15 2003/01/08 18:32:12 adamm Exp $
+ * $Id: resultform.h,v 1.16 2004/01/01 15:27:30 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -33,7 +33,7 @@ extern Boolean ResultFormHandleEvent( EventType* event );
 extern Boolean DrawResultHeader( Char* header ) RESULTFORM_SECTION;
 
 /* Write search result, return true if end of record has been reached */
-extern Boolean DrawResultString( UInt16 record, UInt16 pos, UInt16 length,
+extern Boolean DrawResultString( UInt16 record, UInt16 pos, UInt16 endPos,
                 Char* result ) RESULTFORM_SECTION;
 
 #endif
