@@ -1,5 +1,5 @@
 /*
- * $Id: search.c,v 1.53 2003/09/14 13:14:48 prussar Exp $
+ * $Id: search.c,v 1.72 2004/04/20 21:03:33 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -37,6 +37,7 @@
 #include "const.h"
 #include "search8.h"
 #include "control.h"
+#include "xlit.h"
 
 #include "search.h"
 
@@ -52,6 +53,8 @@
  ***********************************************************************/
 #define SEARCH_AGAIN    false
 #define SELECT_OK       0
+#define SELECT_NEXT     0
+#define SELECT_RESTART  1
 /* ideal number of characters before pattern in result listing */
 #define LEFT_CONTEXT    15
 /* ideal number of characters in and after pattern in result listing */
@@ -82,12 +85,14 @@ static void FindContext( Header* record, Char* text, UInt16 pos,
                 SEARCH_SECTION;
 static void ParseSearchString(Char *pattern) SEARCH_SECTION;
 static void InitSearchRecord(Header *record) SEARCH_SECTION;
-static Boolean DoMultipleTermSearch(Char *text,UInt16 size,
-    UInt8 matchMode, UInt16* pos, Int16 depth) SEARCH_SECTION;
+static Boolean DoMultipleTermSearch(Char *text, UInt16 size,
+    UInt8 matchMode, UInt16 xlitMode, UInt16* pos, UInt16* endFound,
+    Int16 depth) SEARCH_SECTION;
 static Boolean DoSearch( Char* text, UInt16 size, Char* pattern,
-                UInt8 matchMode, UInt16* pos,
+                UInt8 matchMode, UInt16 xlitMode, UInt16* pos, UInt16* endFound,
                 Int16 depth ) SEARCH_SECTION;
 static void PushNode( UInt16 reference, Int16 depth ) SEARCH_SECTION;
+static void AddReference( UInt16 reference, Int16 depth ) SEARCH_SECTION;
 static Boolean SearchRecord(MemHandle handle, Char* pattern, Boolean onlyOne)
     SEARCH_SECTION;
 static Boolean SearchInSubPages( Char* pattern, Boolean startFromBeginning,
@@ -102,6 +107,7 @@ static Boolean SearchInSubPages( Char* pattern, Boolean startFromBeginning,
  ***********************************************************************/
 static UInt16   recordNum;
 static UInt16   position          = 0;
+static UInt16   endPosition       = 0;
 static Char     resultFormat[ 7 ] = { 0 };
 
 static LinkedList searchQueue  = NULL;
@@ -114,6 +120,7 @@ static UInt16     numSearchTerms;
 static Char       searchTermBuffer[ MAX_PATTERN_LEN + MAX_SEARCH_TERMS ];
 static Boolean    alreadySearched[ MAX_SEARCH_TERMS ];
 static UInt16     lastFound[ MAX_SEARCH_TERMS ];
+static UInt16     endLastFound[ MAX_SEARCH_TERMS ];
 static Paragraph* curSearchPara;
 static UInt16     curSearchParaOffset;
 static UInt16     lastSearchedRecordId = 0;
@@ -138,9 +145,9 @@ void SetLastSearchedRecordId
 {
     lastSearchedRecordId = uid;
     lastSearchedSequenceFirstRecordId = GetSequenceBoundaryRecordId( uid,
-                                            DIRECTION_UP );
+                                            DIRECTION_UP, NULL );
     lastSearchedSequenceLastRecordId = GetSequenceBoundaryRecordId( uid,
-                                            DIRECTION_DOWN );
+                                            DIRECTION_DOWN, NULL );
 }
 
 
@@ -340,7 +347,6 @@ static void FindContext
 }
 
 
-
 /* Find pattern in text string */
 static Boolean DoSearch
     (
@@ -349,8 +355,10 @@ static Boolean DoSearch
     Char*   pattern,        /* pattern to search for */
     UInt8   matchMode,      /* indicates if the search should be
                                case-sensitive or not */
+    UInt16  xlitMode,       /* how should we transliterate? */
     UInt16* pos,            /* start position, will contain the position
                                of the pattern if found */
+    UInt16* endFound,       /* end of position where pattern is found */
     Int16   depth           /* depth of this page */
     )
 {
@@ -372,33 +380,27 @@ static Boolean DoSearch
     MemHandle      armChunkH = NULL;
 #endif
     static UInt8   lastMatchMode = SEARCH_UNINITIALIZED;
+    static UInt16  lastXlitMode  = 0;
     static Char    xlat[ 256 ];
-    static Boolean multibyteCharToDot;
+    static Boolean multibyteCharFix;
+    static WChar   multibyteCharFixTo;
+#ifdef SUPPORT_TRANSLITERATION
+    static Boolean symmetricXlit;
+#endif
 
-    if ( lastMatchMode != matchMode ) {
-        multibyteCharToDot = false;
-        switch( matchMode & SEARCH_XLIT_MASK ) {
-            case SEARCH_XLIT_LATIN1:
-                for ( i = 0; i < 256; i++ )
-                    xlat[ i ] = Latin1xlit[ i ];
-                break;
-            case SEARCH_XLIT_NON_ASCII_TO_DOT:
-                for ( i = 1; i < 32; i++ )
-                    xlat[ i ] = '.';
-                for ( i = 127; i < 256; i++ )
-                    xlat[ i ] = '.';
-                for ( i = 32 ; i < 127; i++ )
-                    xlat[ i ] = i;
-                xlat[ 0 ] = 0;
-                multibyteCharToDot = true;
-                break;
-            default:
-                for ( i = 0 ; i < 256; i++ )
-                    xlat[ i ]= i;
+    multibyteCharFix = false;
+    if ( lastMatchMode != matchMode || xlitMode != lastXlitMode ) {
+#ifdef SUPPORT_TRANSLITERATION
+        symmetricXlit = SetTransliteration( xlitMode, xlat, &multibyteCharFix, 
+                            &multibyteCharFixTo );
+#else
+        for ( i = 0 ; i < 256 ; i++ ) {
+            xlat[ i ] = i;
         }
+#endif
         if ( ! ( matchMode & SEARCH_CASESENSITIVE ) ) {
             for ( i = 0; i < 256; i++ )
-                xlat[ i ] = TxtGlueLowerChar( xlat[ i ] );
+                xlat[ i ] = TxtGlueLowerChar( ( UInt8 )xlat[ i ] );
         }
         lastMatchMode = matchMode;
     }
@@ -411,10 +413,20 @@ static Boolean DoSearch
     i = wplen = 0;
     while ( i < patternLen ) {
         charsize = TxtGlueGetNextChar( pattern, i, &ch );
-        if ( charsize == 1 )
-            ch = xlat[ (UInt8) ch ];
-        else if ( multibyteCharToDot )
-            ch = '.';
+        if ( charsize == 1 ) {
+#ifdef SUPPORT_TRANSLITERATION
+            if ( symmetricXlit ) {
+#endif
+                ch = xlat[ (UInt8) ch ];
+#ifdef SUPPORT_TRANSLITERATION
+            }
+            else if ( ! ( matchMode & SEARCH_CASESENSITIVE ) ) {
+                ch = TxtGlueLowerChar( (UInt8) ch );
+            }
+#endif
+        }
+        else if ( multibyteCharFix )
+            ch = multibyteCharFixTo;
 
         wpattern[ wplen ] = ch;
 
@@ -460,7 +472,7 @@ static Boolean DoSearch
             j     = armData.j;
             top   = armData.top;
 
-            if ( j >= wplen || i >= size )
+            if ( wplen <= j || size <= i )
                 break;
             charsize = topcharsize = 1;
         }
@@ -468,8 +480,8 @@ static Boolean DoSearch
 #endif
         if ( guaranteed8Bits ) {
             ch = DoSearch8BitText( text, size, wpattern, wplen, xlat,
-                     &i, &j, &top, depth > 0 );
-            if ( j >= wplen || i >= size )
+                     &i, &j, &top, 0 < depth );
+            if ( wplen <= j || size <= i )
                 break;
             charsize = topcharsize = 1;
         }
@@ -499,7 +511,7 @@ static Boolean DoSearch
                             break;
                     }
                     if ( q == listSize )
-                        PushNode( reference, depth );
+                        AddReference( reference, depth );
 
                 }
             }
@@ -510,8 +522,8 @@ static Boolean DoSearch
         }
         if ( charsize == 1 )
             ch = xlat[ (UInt8) ch ];
-        else if ( multibyteCharToDot )
-            ch = '.';
+        else if ( multibyteCharFix )
+            ch = multibyteCharFixTo;
         if ( ch != wpattern[ j ] ) {
             top += topcharsize;
             i    = top;
@@ -528,7 +540,8 @@ static Boolean DoSearch
     }
 #endif
     if ( j == wplen ) {
-        *pos = i - patternLen;
+        *pos      = top;
+        *endFound = i;
         return true;
     }
     else {
@@ -586,7 +599,9 @@ static Boolean DoMultipleTermSearch
     Char*   text,
     UInt16  size,
     UInt8   matchMode,
+    UInt16  xlitMode,
     UInt16* pos,
+    UInt16* endFound,
     Int16   depth
     )
 {
@@ -619,8 +634,8 @@ static Boolean DoMultipleTermSearch
                 else
                     lastFound[ i ] = curSearchParaOffset;
 
-                if ( DoSearch( text, size, searchTerms[ i ], matchMode,
-                               lastFound + i, i ? 0 : depth ) == false )
+                if ( ! DoSearch( text, size, searchTerms[ i ], matchMode,
+                        xlitMode, lastFound + i, endLastFound + i, i ? 0 : depth ) )
                     return false;
 
                 alreadySearched[ i ] = true;
@@ -629,7 +644,8 @@ static Boolean DoMultipleTermSearch
                 inThisPara = false;
         }
         if ( inThisPara ) {
-            *pos = lastFound[ 0 ];
+            *pos      = lastFound[ 0 ];
+            *endFound = endLastFound[ 0 ];
             return true;
         }
         for ( i = 0; i < numSearchTerms; i++ ) {
@@ -643,9 +659,9 @@ static Boolean DoMultipleTermSearch
 
 static Boolean SearchRecord
          (
-         MemHandle handle, 
-         Char* pattern, 
-         Boolean onlyOne
+         MemHandle  handle,   /* handle of record to search */
+         Char*      pattern,  /* pattern to search for */
+         Boolean    onlyOne   /* are we looking only for a single match? */
          )
 {
     Boolean     multiple;
@@ -674,6 +690,7 @@ static Boolean SearchRecord
             uncompressHandle    = Uncompress( record );
             text                = MemHandleLock( uncompressHandle );
             size                = MemPtrSize( text );
+            ResetLastUncompressedPHTML();
         }
         ErrCatch( UNUSED_PARAM( err ) ) {
             MemHandleUnlock( handle );
@@ -692,11 +709,13 @@ static Boolean SearchRecord
         if ( multiple )
             match = DoMultipleTermSearch( text, size,
                     Prefs()->searchFlags & SEARCH_MATCHMODE_MASK,
-                    &position, lastDepth + 1 );
+                    Prefs()->searchXlit,
+                    &position, &endPosition, lastDepth + 1 );
         else
             match = DoSearch( text, size, pattern,
                     Prefs()->searchFlags & SEARCH_MATCHMODE_MASK,
-                    &position, lastDepth + 1 );
+                    Prefs()->searchXlit,
+                    &position, &endPosition, lastDepth + 1 );
 
         if ( onlyOne ) {
             break;
@@ -705,9 +724,8 @@ static Boolean SearchRecord
             Char result[ 3*MAX_CONTEXT + 1 ];
 
             FindContext( record, text, position, result, 3*MAX_CONTEXT + 1 );
-            done = DrawResultString( uniqueID, position,
-                multiple ? StrLen( searchTerms[ 0 ]) : StrLen( pattern ),
-                result );
+            done = DrawResultString( uniqueID, position, endPosition - position,
+                    result );
             position++;
             if ( done )
                 break;
@@ -750,25 +768,19 @@ Boolean SearchDocument
     if ( searchMode == SEARCH_IN_SUB_PAGES )
         return SearchInSubPages( pattern, startFromBeginning, findMore );
 
-    onlyOnePage = (searchMode == SEARCH_IN_ONE_PAGE );
+    onlyOnePage = ( searchMode == SEARCH_IN_ONE_PAGE );
 
     lastDepth = -1;
 
     if ( startFromBeginning ) {
         position    = 0;
         if ( onlyOnePage ) {
-            UInt16    recordId;
-            MemHandle handle;
-            recordId = GetSequenceBoundaryRecordId( GetCurrentRecordId(),
-                           DIRECTION_UP );
-            handle = FindRecord( recordId, &recordNum );
-            if ( handle == NULL )
-                return false;
-            else
-                FreeRecordHandle( &handle );
+            GetSequenceBoundaryRecordId( GetCurrentRecordId(), DIRECTION_UP,
+                &recordNum );
         }
-        else
+        else {
             recordNum = 0;
+        }
     }
 
     if ( NULL != findMore )
@@ -826,20 +838,24 @@ Boolean SearchDocument
             record = MemHandleLock( handle );
             flags  = record->flags;
             MemHandleUnlock( handle );
+            FreeRecordHandle( &handle );
 
             if ( ! ( flags & HEADER_FLAG_CONTINUATION ) ) {
-                FreeRecordHandle( &handle );
-                if ( FrmAlert( confirmEndOfDoc ) == SELECT_OK ) {
+                Int16  selection;
+                selection = FrmAlert( confirmEndOfPage );
+                if ( selection == SELECT_RESTART ) {
                     position  = 0;
                     recordNum = initialRecordNum;
                     continue;
                 }
-                else
+                else if ( selection != SELECT_NEXT ) {
                     break;
+                }
             }
         }
-
-        FreeRecordHandle(&handle);
+        else {
+            FreeRecordHandle(&handle);
+        }
 
         if ( done )
             break;
@@ -847,10 +863,12 @@ Boolean SearchDocument
         recordNum++;
         position = 0;
     }
-    if ( onlyOnePage && done )
-        SetFindPatternData( position,
-            ( Prefs()->searchFlags & SEARCH_MULTIPLE ) ?
-                  StrLen( searchTerms[ 0 ] ) : StrLen( pattern ) );
+    if ( onlyOnePage ) {
+        if ( done )
+            SetFindPatternData( position, endPosition - position );
+        else
+            FrmUpdateForm( Prefs()->lastForm, frmRedrawUpdateCode );
+    }
 
     SetLastSearchedRecordId( lastSearchedRecordId );
 
@@ -878,6 +896,37 @@ static void PushNode
 
 
 
+/* Add a reference, being sure to include all the fragments. */
+static void AddReference
+    (
+    UInt16 reference,
+    Int16  depth
+    )
+{
+    UInt16     recordNum;
+    MemHandle  handle;
+
+    reference = GetSequenceBoundaryRecordId( reference, DIRECTION_UP,
+                    &recordNum );
+    /* Is this a non-existent reference, i.e., external link? */
+    if ( reference == NO_RECORD )
+        return;
+    PushNode( reference, depth );
+    handle = GetSequentialRecordHandle( &recordNum, DIRECTION_DOWN );
+    while ( NULL != handle ) {
+        Header* record;
+
+        record = MemHandleLock( handle );
+        PushNode( record->uid, depth );
+        MemHandleUnlock( handle );
+        FreeRecordHandle( &handle );
+
+        handle = GetSequentialRecordHandle( &recordNum, DIRECTION_DOWN );
+    }
+}
+
+
+
 void ReleaseSearchQueue( void )
 {
     QueueNode *node;
@@ -925,7 +974,7 @@ static Boolean SearchInSubPages
         ReleaseSearchQueue();
         searchQueue = ListCreate();
         r = MemHandleLock( m = GetCurrentRecord() );
-        PushNode( r->uid , 0 );
+        AddReference( r->uid , 0 );
         MemHandleUnlock( m );
         /*!!! isnt it a picture?*/
     }
@@ -990,7 +1039,7 @@ void SearchAgain( void )
     if ( StrLen( pattern ) == 0 )
         return;
 
-    if ( SearchDocument( pattern, false, NULL, SEARCH_IN_ONE_PAGE ) );
+    if ( SearchDocument( pattern, false, NULL, SEARCH_IN_ONE_PAGE ) )
         GoToSearchResult();
 }
 
