@@ -1,5 +1,5 @@
 /*
- * $Id: skins.h,v 1.2 2003/06/15 23:49:01 prussar Exp $
+ * $Id: skins.h,v 1.5 2004/02/01 11:26:33 nordstrom Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -23,10 +23,22 @@
 #ifndef PLUCKER_SKINS_H
 #define PLUCKER_SKINS_H
 
+#include "viewer.h"
+
+#ifdef SUPPORT_SKINS
+
 /* load skins */
-extern void OpenSkins( void );
+extern void OpenSkins( void ) SKINS_SECTION;
 
 /* close skins */
-extern void CloseSkins( void );
+extern void CloseSkins( void ) SKINS_SECTION;
+
+#else
+
+#define OpenSkins()
+#define CloseSkins()
+
+#endif /* SUPPORT_SKINS */
+
+#endif /* PLUCKER_SKINS_H */
 
-#endif
