@@ -1,5 +1,5 @@
 /*
- * $Id: util.c,v 1.74 2003/10/08 17:50:12 matto Exp $
+ * $Id: util.c,v 1.87 2004/04/29 01:52:06 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -19,7 +19,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
  */
-
 #include <stdarg.h>
 #include <FntGlue.h>
 
@@ -30,13 +29,26 @@
 #include "os.h"
 #include "document.h"
 #include "search.h"
-#include "silkscreen.h"
 #include "resourceids.h"
 #include "rotate.h"
 #include "font.h"
+#include "const.h"
+#include "resize.h"
 
 #include "util.h"
 
+#ifdef HAVE_PALMCUNIT
+#include "mock_Header.h"
+#endif
+
+
+/***********************************************************************
+ *
+ *      Internal Constants
+ *
+ ***********************************************************************/
+static const Char internalMemoDBName[] = "Plkr-MemoDB";
+
 
 /***********************************************************************
  *
@@ -45,6 +57,7 @@
  ***********************************************************************/
 static Char** GetActionStrings(void) UTIL_SECTION;
 static Char** GetKeyStrings(void) UTIL_SECTION;
+static void WriteToMemo( void ) UTIL_SECTION;
 
 
 /***********************************************************************
@@ -384,9 +397,107 @@ void SafeMemHandleFree
 
 
 
+/* PalmOne did some "tricks" with the PIM databases on T|3 and T|E,
+   which prevents us from working directly with the MemoDB. Instead
+   we create an 'internal' MemoDB and write to that database; before
+   the app exits we write the stored data to the real MemoDB */
+Err CreateInternalMemoDB( void )
+{
+    /* If the device crashed before writing the text to the real MemoDB
+       we will have a left-over internal MemoDB; write the text to the
+       real MemoDB and remove the internal MemoDB */
+    DeleteInternalMemoDB();
+
+    return DmCreateDatabase( 0, internalMemoDBName, ViewerAppID,
+            PlkrMemoDBType, false );
+}
+
+
+
+/* Delete the internal MemoDB (after writing the data to the real MemoDB). */
+void DeleteInternalMemoDB( void )
+{
+    LocalID dbId;
+
+    dbId = DmFindDatabase( 0, internalMemoDBName );
+    if ( dbId != 0 ) {
+        WriteToMemo();
+        DmDeleteDatabase( 0, dbId );
+    }
+}
+
+
+
+/* Write data from internal MemoDB to real MemoDB. */
+static void WriteToMemo( void )
+{
+    DmOpenRef   MemopadDB;
+    DmOpenRef   InternalMemopadDB;
+    UInt16      i;
+
+    /* Open real and internal MemoDBs. */
+    MemopadDB = DmOpenDatabaseByTypeCreator( 'DATA', 'memo', dmModeReadWrite );
+    if ( MemopadDB == NULL ) {
+        return;
+    }
+    InternalMemopadDB = DmOpenDatabaseByTypeCreator( PlkrMemoDBType,
+                        ViewerAppID, dmModeReadWrite );
+    if ( InternalMemopadDB == NULL ) {
+        DmCloseDatabase( MemopadDB );
+        return;
+    }
+
+    /* Traverse all records in the internal MemoDB and write the
+       data to the real MemoDB. */
+    for ( i = 0; i < DmNumRecords( InternalMemopadDB ); i++ ) {
+        MemHandle srcHandle;
+
+        srcHandle = DmQueryRecord( InternalMemopadDB, i );
+        if ( srcHandle != NULL ) {
+            MemHandle   destHandle;
+            UInt16      index;
+            UInt16      attr;
+            Int16       category;
+
+            index       = dmMaxRecordIndex;
+            destHandle  = DmNewRecord( MemopadDB, &index,
+                            MemHandleSize( srcHandle ) );
+            if ( destHandle != NULL ) {
+                MemPtr  srcPtr;
+                MemPtr  destPtr;
+
+                srcPtr  = MemHandleLock( srcHandle );
+                destPtr = MemHandleLock( destHandle );
+                DmWrite( destPtr, 0, srcPtr, MemHandleSize( srcHandle ) );
+                MemHandleUnlock( destHandle );
+                MemHandleUnlock( srcHandle );
+            
+                /* If there is a Plucker category we assign that
+                   category to the record. Otherwise, we use the
+                   Unfiled category. */
+                DmRecordInfo( MemopadDB, index, &attr, NULL, NULL );
+                attr &= ~dmRecAttrCategoryMask;
+
+                category = CategoryFind( MemopadDB, "Plucker" );
+                if ( category == dmAllCategories )
+                    attr |= dmUnfiledCategory;
+                else
+                    attr |= category;
+
+                DmSetRecordInfo( MemopadDB, index, &attr, NULL );
+                DmReleaseRecord( MemopadDB, index, true );
+            }
+        }
+    }
+    DmCloseDatabase( InternalMemopadDB );
+    DmCloseDatabase( MemopadDB );
+}
+
+
+
 #define MAX_MSG_LEN     3000
 
-/* write message to memo */
+/* Write message to memo. */
 void WriteMemoEntry
     (
     UInt16* index,  /* record index in MemoDB */
@@ -400,8 +511,8 @@ void WriteMemoEntry
     va_list     args;
     DmOpenRef   MemopadDB;
     MemHandle   recH;
-    UInt16      mode;
     UInt16      length;
+    Boolean     releaseRecord = false;
 
     va_start( args, fmt );
     StrVPrintF( msg, fmt, args );
@@ -411,41 +522,35 @@ void WriteMemoEntry
     if ( length == 0 )
         return;
 
-    mode = dmModeReadWrite;
-    MemopadDB = DmOpenDatabaseByTypeCreator( 'DATA', 'memo', mode );
+    MemopadDB = DmOpenDatabaseByTypeCreator( PlkrMemoDBType, ViewerAppID,
+                    dmModeReadWrite );
     if ( MemopadDB == NULL )
         return;
 
     if ( *index == dmMaxRecordIndex )
+    {
         recH = DmNewRecord( MemopadDB, index, length + 1 );
+        releaseRecord = true;
+    }
     else
+    {
         recH = DmResizeRecord( MemopadDB, *index, *offset + length + 1 );
+        releaseRecord = false;
+    }
 
     if ( recH != NULL ) {
         Char    null;
         MemPtr  recP;
-        UInt16  attr;
-        Int16   category;
 
         null = '\0';
         recP = MemHandleLock( recH );
-
         DmWrite( recP, *offset, msg, length );
         DmWrite( recP, *offset + length, &null, 1 );
-
         MemHandleUnlock( recH );
-
-        DmRecordInfo( MemopadDB, *index, &attr, NULL, NULL );
-        attr &= ~dmRecAttrCategoryMask;
-
-        category = CategoryFind( MemopadDB, "Plucker" );
-        if ( category == dmAllCategories )
-            attr |= dmUnfiledCategory;
-        else
-            attr |= category;
-
-        DmSetRecordInfo( MemopadDB, *index, &attr, NULL );
-        DmReleaseRecord( MemopadDB, *index, true );
+        if ( releaseRecord )
+        {
+            DmReleaseRecord( MemopadDB, *index, true );
+        }
 
         *offset += length;
     }
@@ -476,14 +581,11 @@ UInt16 GetValidForm
 {
     UInt16           properFormId;
     HiRes            hiResType;
-    SilkScreenStatus silkStatus;
 
     properFormId  = formId;
     hiResType     = HiResType();
 
-    silkStatus = GetSilkScreenStatus();
-
-    switch( formId ) {
+    switch ( formId ) {
         case frmMainTop:
         case frmMainTopHandera:
             if ( IsHiResTypeHandera( hiResType ) )
@@ -494,21 +596,8 @@ UInt16 GetValidForm
 
         case frmMainBottom:
         case frmMainBottomHandera:
-        case frmMainBottomHanderaLow:
-        case frmMainBottomSonyLow:
-            if ( IsHiResTypeHandera( hiResType ) &&
-                 silkStatus == SILKSCREEN_DOWN )
-                properFormId = frmMainBottomHanderaLow;
-            else if ( IsHiResTypeHandera( hiResType ) &&
-                      silkStatus == SILKSCREEN_UP )
+            if ( IsHiResTypeHandera( hiResType ) )
                 properFormId = frmMainBottomHandera;
-            else if ( ( SilkScreenType() == sonyV1SilkScreen ||
-                        SilkScreenType() == sonyV2SilkScreen ||
-                        SilkScreenType() == sonyV3SilkScreen ) &&
-                      silkStatus == SILKSCREEN_DOWN )
-                properFormId = frmMainBottomSonyLow;
-            else if ( silkStatus == SILKSCREEN_NONE )
-                properFormId = frmMainSonyFull;
             else
                 properFormId = frmMainBottom;
             break;
@@ -523,30 +612,12 @@ UInt16 GetValidForm
 
         case frmLibrary:
         case frmLibraryHandera:
-        case frmLibraryHanderaLow:
-            if ( IsHiResTypeHandera( hiResType ) &&
-                 silkStatus == SILKSCREEN_DOWN )
-                properFormId = frmLibraryHanderaLow;
-            else if ( IsHiResTypeHandera( hiResType ) &&
-                      silkStatus == SILKSCREEN_UP )
+            if ( IsHiResTypeHandera( hiResType ) )
                 properFormId = frmLibraryHandera;
             else
                 properFormId = frmLibrary;
             break;
 
-        case frmResult:
-        case frmResultSonySilkMin:
-            if ( ( SilkScreenType() == sonyV1SilkScreen ||
-                   SilkScreenType() == sonyV2SilkScreen ||
-                   SilkScreenType() == sonyV3SilkScreen ) &&
-               ( silkStatus == SILKSCREEN_DOWN ||
-                 silkStatus == SILKSCREEN_NONE )
-              )
-                properFormId = frmResultSonySilkMin;
-            else
-                properFormId = frmResult;
-            break;
-
         case frmFont:
         case frmFontOS2:
 #ifdef HAVE_HANDERA_SDK
@@ -636,7 +707,6 @@ Boolean IsFormMain
          formId == frmMainTopHandera ||
          formId == frmMainBottomHandera ||
          formId == frmMainBottomHanderaLow ||
-         formId == frmMainBottomSonyLow ||
          formId == frmMainNoneHandera )
         return true;
     else
@@ -651,9 +721,10 @@ Boolean IsFormLibrary
     UInt16 formId   /* formId to check against */
     )
 {
-    if ( formId == frmLibrary ||
-         formId == frmLibraryHandera ||
-         formId == frmLibraryHanderaLow )
+    if ( formId == frmLibrary           ||
+         formId == frmLibraryHandera    ||
+         formId == frmLibraryHanderaLow ||
+         formId == frmLibrarySonyWide )
         return true;
     else
         return false;
@@ -668,7 +739,7 @@ UInt16 GetMainFormId( void )
     const ToolbarType toolbar = Prefs()->toolbar;
     UInt16            toolbarId;
 
-    switch( toolbar ) {
+    switch ( toolbar ) {
         case TOOLBAR_TOP:
             toolbarId = GetValidForm( frmMainTop );
             break;
@@ -895,6 +966,7 @@ UInt16 StandardCoordinateSystem( void )
 
 
 
+#if 0
 /*Show small message window without any control*/
 FormType* PopupMessage
     (
@@ -904,7 +976,6 @@ FormType* PopupMessage
     FormType* frm;
 
     frm = FrmInitForm( frmExportLink );
-    HanderaConvertForm( frm );
     FrmDrawForm( frm );
 
     return frm;
@@ -912,6 +983,7 @@ FormType* PopupMessage
 
 
 
+
 /*Hide popup window created by PopupMessage */
 void HidePopup
     (
@@ -921,7 +993,7 @@ void HidePopup
     FrmEraseForm( frm );
     FrmDeleteForm( frm );
 }
-
+#endif
 
 
 /* Insert text in to a textfield and scroll to the top */
@@ -1005,3 +1077,17 @@ void UpdateFieldScrollbar( FieldType* field, ScrollBarType* scrollbar )
         maxValue = scrollPos;
     SclSetScrollBar( scrollbar, scrollPos, 0, maxValue, 6 );
 }
+
+
+/* Check low power situation */
+Boolean HasEnoughPower( void )
+{
+    UInt16 warnThreshold;
+    UInt16 currentVoltage;
+
+    currentVoltage = SysBatteryInfoV20( false, &warnThreshold,
+                        NULL, NULL, NULL, NULL );
+
+    return ( warnThreshold <= currentVoltage );
+}
+
