@@ -1,14 +1,14 @@
 /*
- * $Id: vfsfile.c,v 1.34.4.1 2003/10/22 20:33:20 prussar Exp $
+ * $Id: vfsfile.c,v 1.43 2004/04/24 03:12:27 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -30,9 +30,14 @@
 #include "metadocument.h"
 #include "document.h"
 #include "axxpacimp.h"
+#include "font.h"
 
 #include "vfsfile.h"
 
+#ifdef HAVE_PALMCUNIT
+#include "mock_Header.h"
+#endif
+
 
 /***********************************************************************
  *
@@ -50,6 +55,11 @@
 /* MIME type for Plucker documents */
 #define ViewerMIMEType  "application/prs.plucker"
 
+
+#define IsDocScan( e )     (( ( e ) & ENUMERATECARD_DOCS ) != 0 )
+#define IsFontScan( e )    (( ( e ) & ENUMERATECARD_FONTS ) != 0 )
+
+
 static const Char* PluckerDirList[] = {
     "INVALID",
     "/Palm/Programs/Plucker/",
@@ -57,8 +67,12 @@ static const Char* PluckerDirList[] = {
     "/Palm/Launcher/"
 };
 
-static const Char illegalFilenameChars[] = "\"*?\\/<>:|";
+static const Char* FontDirList[] = {
+    "/Palm/Programs/Plucker/Fonts/",
+    "/Palm/Launcher/"
+};
 
+static const Char illegalFilenameChars[] = "\"*?\\/<>:|";
 static const Char fakeVolumeNameFormat[] = "{Plkr--volume number %04X}";
 
 /***********************************************************************
@@ -69,9 +83,10 @@ static const Char fakeVolumeNameFormat[] = "{Plkr--volume number %04X}";
 static void GetPluckerVolumeLabel( UInt16 volRefNum, Char* volumeLabel, 
     UInt16 length );
 static void AddVFSFile( UInt16 volRefNum, FileInfoType* info,
-                const Char* dir, UInt16 location ) VFSFILE_SECTION;
-static void EnumerateDir( UInt16 volRefNum, UInt16 location,
-                const Char* dir ) VFSFILE_SECTION;
+                const Char* dir, UInt16 location,
+                EnumerateCardType enumerateWhat ) VFSFILE_SECTION;
+static void EnumerateDir( UInt16 volRefNum, UInt16 location, const Char* dir,
+                EnumerateCardType enumerateWhat ) VFSFILE_SECTION;
 static void BeamStream( ExgSocketType* s, FileRef fileRef ) VFSFILE_SECTION;
 static Err OpenVFSDatabase( UInt16 Ref, Char* path, FileRef* fileRef,
                 UInt16* version, Char* name, UInt32* date,
@@ -122,11 +137,11 @@ static void GetPluckerVolumeLabel
     UInt16 length      /* must be at least large enough to hold fake name */
     )
 {
-	Err  err;
-	err = VFSVolumeGetLabel( volRefNum, volumeLabel, length );
-	if ( err != errNone )
-	    StrPrintF( volumeLabel, fakeVolumeNameFormat, volRefNum );
-}	    
+    Err  err;
+    err = VFSVolumeGetLabel( volRefNum, volumeLabel, length );
+    if ( err != errNone )
+        StrPrintF( volumeLabel, fakeVolumeNameFormat, volRefNum );
+}        
 
 
 
@@ -161,13 +176,14 @@ static Err OpenVFSDatabase
 
 
 
-/* add file to doc list */
+/* Add file to doc list */
 static void AddVFSFile
     (
-    UInt16          volRefNum,
-    FileInfoType*   info,
-    const Char*     dir,
-    UInt16          location
+    UInt16              volRefNum,
+    FileInfoType*       info,
+    const Char*         dir,
+    UInt16              location,
+    EnumerateCardType   enumerateWhat
     )
 {
     Err             err;
@@ -177,20 +193,20 @@ static void AddVFSFile
     UInt32          type;
     UInt32          creatorID;
 
-    /* set path to document */
+    /* Set path to document */
     path = SafeMemPtrNew( StrLen( dir ) + StrLen( info->nameP ) + 2 );
     StrCopy( path, dir );
     StrCat( path, info->nameP );
     err = VFSFileOpen( volRefNum, path, vfsModeRead, &ref );
-    SafeMemPtrFree( path );
     if ( err == errNone ) {
+        Char volumeLabel[ LABEL_LEN ];
+
         err = VFSFileDBInfo( ref, docInfo.name,
                 &docInfo.attributes, NULL, &docInfo.created,
                 NULL, NULL, NULL, NULL, NULL, &type, &creatorID,
                 NULL );
-        if ( err == errNone && type == ViewerDocumentType &&
-             creatorID == ViewerAppID ) {
-            Char            volumeLabel[ LABEL_LEN ];
+        if ( err == errNone && IsDocScan( enumerateWhat ) &&
+             type == ViewerDocumentType && creatorID == ViewerAppID ) {
             DocumentData*   handlePtr;
             MemHandle       handle;
             UInt16          index;
@@ -225,22 +241,22 @@ static void AddVFSFile
                 StrNCopy( oldVolumeLabel, handlePtr->data + fileLen,
                     volumeLabelLen );
                 MemHandleUnlock( handle );
-                /* if the document is newer than the stored one, or
+                /* If the document is newer than the stored one, or
                    moved to external card then updated info */
                 if ( ! oldActive || oldCreated < docInfo.created ||
                      oldLocation != location ||
                      ! STREQ( volumeLabel, oldVolumeLabel ) ) {
-                    /* assign remaining document data */
+                    /* Assign remaining document data */
                     docInfo.location    = location;
                     docInfo.active      = true;
                     docInfo.categories  = categories;
-                    /* a "new" document should be indicated as unread */
+                    /* A "new" document should be indicated as unread */
                     if ( ! oldActive || oldCreated < docInfo.created )
                         docInfo.timestamp = 0;
                     else
                         docInfo.timestamp = timestamp;
 
-                    /* if document has been moved to external card the
+                    /* If document has been moved to external card the
                        meta database must be renamed */
                     if ( oldLocation != location )
                         VFSRenameMetaDocument( docInfo.name, oldLocation,
@@ -265,19 +281,30 @@ static void AddVFSFile
                 AddDocument( &docInfo, volumeLabel );
             }
         }
+#ifdef SUPPORT_VFS_FONTS
+        else if ( err == errNone && IsFontScan( enumerateWhat ) &&
+                  type == UserFontResourceType && creatorID == ViewerAppID ) {
+            GetPluckerVolumeLabel( volRefNum, volumeLabel, LABEL_LEN );
+            AddVFSFont( docInfo.name, volumeLabel, path );
+            VFSFileClose( ref );
+        }
+#endif
         else {
             VFSFileClose( ref );
         }
     }
+    SafeMemPtrFree( path );
 }
 
 
-/* enumerate documents in volume */
+
+/* Enumerate documents in directory */
 static void EnumerateDir
     (
-    UInt16      volRefNum,
-    UInt16      location,
-    const Char* dir
+    UInt16              volRefNum,
+    UInt16              location,
+    const Char*         dir,
+    EnumerateCardType   enumerateWhat
     )
     /* THROWS */
 {
@@ -305,8 +332,9 @@ static void EnumerateDir
             if ( err == errNone ) {
                 if ( info.attributes != vfsFileAttrDirectory ) {
                     ErrTry {
-                        AddVFSFile( volRefNum, &info, dir, location );
-                    } 
+                        AddVFSFile( volRefNum, &info, dir, location,
+                            enumerateWhat );
+                    }
                     ErrCatch( err ) {
                         SafeMemPtrFree( filename );
                         VFSFileClose( dirRef );
@@ -446,8 +474,11 @@ UInt16 FindVolRefNum
 
 
 
-/* enumerate mounted volumes */
-void EnumerateCards( void )
+/* Enumerate mounted volumes */
+void EnumerateCards
+     (
+     EnumerateCardType enumerateWhat
+     )
     /* THROWS */
 {
     UInt16  volRefNum;
@@ -458,16 +489,17 @@ void EnumerateCards( void )
 
     if ( ! SupportVFS() )
         return;
-        
+
     haveLastVolumeLabel = false;
 
     /* set number of elements to search */
-    numOfElements = GetNumOfDocuments();
+    if ( IsDocScan( enumerateWhat ) )
+        numOfElements = GetNumOfDocuments();
 
     volRefNum   = 0;
     err         = errNone;
 
-    /* iterate all volumes */
+    /* Iterate all volumes */
     volume      = 0;
     volIterator = vfsIteratorStart;
     while ( volIterator != vfsIteratorStop ) {
@@ -479,11 +511,23 @@ void EnumerateCards( void )
             ErrThrow( err );
         }
 
-        /* find all Plucker documents on the volume */
-        for ( dirIndex = 1;
-              dirIndex < sizeof PluckerDirList / sizeof PluckerDirList[ 0 ];
-              dirIndex++ ) {
-            EnumerateDir( volRefNum, dirIndex, PluckerDirList[ dirIndex ] );
+        /* Find all Plucker documents on the volume */
+        if ( IsDocScan( enumerateWhat ) ) {
+            for ( dirIndex = 1;
+                  dirIndex < sizeof PluckerDirList / sizeof PluckerDirList[ 0 ];
+                  dirIndex++ ) {
+                EnumerateDir( volRefNum, dirIndex, PluckerDirList[ dirIndex ],
+                    ENUMERATECARD_DOCS );
+            }
+        }
+        /* Find all fonts on the volume */
+        if ( IsFontScan( enumerateWhat ) ) {
+            for ( dirIndex = 0;
+                  dirIndex < sizeof FontDirList / sizeof FontDirList[ 0 ];
+                  dirIndex++ ) {
+                EnumerateDir( volRefNum, dirIndex, FontDirList[ dirIndex ],
+                    ENUMERATECARD_FONTS );
+            }
         }
     }
 }
@@ -728,7 +772,7 @@ static MemHandle SearchVFSDocument
 
     while ( lower <= upper ) {
         UInt16      currentUID;
-        UInt16       oldUpper;
+        UInt16      oldUpper;
 
         /*
            We now make some arithmetical adjustments to to the lower and upper
@@ -739,14 +783,14 @@ static MemHandle SearchVFSDocument
            of VFSFileDBGetRecord()).  The adjustments are based on two
            inequalities, where uid[n] is the uid of the nth record,
            and we are searching for current, knowing uid[x], and knowing that
-           lower <= current and current <= upper.  Both inequalities are based 
+           lower <= current and current <= upper.  Both inequalities are based
            on the fact that for any i and j, if i <= j, then:
              j - i <= uid[j] - uid[i].
            From this follow two helpful inequalities:
              (1) x <= lower + ( uid[x] - uid[lower] )
              (2) upper - ( uid[upper] - uid[x] ) <= x
-           What this means is that we can replace the upper bound, upper, of 
-           the binary search by lower + (uid[current] - uid[lower]) when this 
+           What this means is that we can replace the upper bound, upper, of
+           the binary search by lower + (uid[current] - uid[lower]) when this
            is less than upper, and the lower bound by
            upper - ( uid[upper] - uid[current] ).
            Moreover, we are free to replace uid[lower] by an underestimate and
@@ -830,7 +874,7 @@ MemHandle FindVFSRecord
 static void MakeLegalFilename( Char* s )
 {
     while ( *s ) {
-        if ( NULL != StrChr( illegalFilenameChars, *s ) ||
+        if ( NULL != StrChr( illegalFilenameChars, ( UInt8 )*s ) ||
              (UInt8)*s < ' ' )
             *s = '_';
         s++;
