@@ -1,5 +1,5 @@
 /*
- * $Id: viewer.c,v 1.114 2003/10/11 16:05:09 nordstrom Exp $
+ * $Id: viewer.c,v 1.135 2004/05/12 01:49:10 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -40,7 +40,7 @@
 #include "screen.h"
 #include "search.h"
 #include "session.h"
-#include "silkscreen.h"
+#include "DIA.h"
 #include "timeout.h"
 #include "uncompress.h"
 #include "util.h"
@@ -52,6 +52,8 @@
 #include "keyboard.h"
 #include "keyboardform.h"
 #include "grayfont.h"
+#include "xlit.h"
+#include "statusform.h"
 
 #include "viewer.h"
 
@@ -89,15 +91,15 @@ typedef struct {
  *      Local Functions
  *
  ***********************************************************************/
-static void EnableNotification ( Boolean registering );
-static void HandleNotification ( SysNotifyParamType* cmdPBP );
-static Boolean HandleVChrs(EventType *event);
-static Boolean HandleShiftGestures(void);
-static void HandleFormLoad(EventType *event);
-static UInt16 CalculateAutoscrollTimeout(void);
-static Err StartApplication(MemPtr cmdPBP);
-static void EventLoop(void);
-static void StopApplication(void);
+static void EnableNotification ( Boolean registering ) VIEWER_SECTION;
+static void HandleNotification ( SysNotifyParamType* cmdPBP ) VIEWER_SECTION;
+static Boolean HandleVChrs(EventType *event) VIEWER_SECTION;
+static Boolean HandleShiftGestures(void) VIEWER_SECTION;
+static void HandleFormLoad(EventType *event) VIEWER_SECTION;
+static UInt16 CalculateAutoscrollTimeout(void) VIEWER_SECTION;
+static Err StartApplication(MemPtr cmdPBP) VIEWER_SECTION;
+static void EventLoop(void) VIEWER_SECTION;
+static void StopApplication(void) VIEWER_SECTION;
 
 
 /***********************************************************************
@@ -113,7 +115,6 @@ static EventHandler ListOfEventHandlers[] = {
     { frmMainTopHandera, MainFormHandleEvent },
     { frmMainBottomHandera, MainFormHandleEvent },
     { frmMainBottomHanderaLow, MainFormHandleEvent },
-    { frmMainBottomSonyLow, MainFormHandleEvent },
     { frmMainNoneHandera, MainFormHandleEvent },
     { frmFont, FontFormHandleEvent },
     { frmFontOS2, FontFormHandleEvent },
@@ -121,6 +122,7 @@ static EventHandler ListOfEventHandlers[] = {
     { frmKeyboard, KeyboardFormHandleEvent },
     { frmPrefs, PrefsFormHandleEvent },
     { frmLibrary, LibraryFormHandleEvent },
+    { frmAbout, StatusFormHandleEvent },
     { frmLibraryHandera, LibraryFormHandleEvent },
     { frmLibraryHanderaLow, LibraryFormHandleEvent },
     { frmDetails, DetailsFormHandleEvent },
@@ -128,7 +130,6 @@ static EventHandler ListOfEventHandlers[] = {
     { frmEmail, EmailFormHandleEvent },
     { frmSearch, SearchFormHandleEvent },
     { frmResult, ResultFormHandleEvent },
-    { frmResultSonySilkMin, ResultFormHandleEvent },
     { frmAddBookmark, AddBookmarkFormHandleEvent },
     { frmBookmarks, BookmarksFormHandleEvent },
     { frmRenameDoc, RenameDocFormHandleEvent },
@@ -141,7 +142,7 @@ static EventHandler ListOfEventHandlers[] = {
 
 
 
-/* This routine opens the document, loads the saved-state information 
+/* This routine opens the document, loads the saved-state information
    and initializes global variables */
 static Err StartApplication
     (
@@ -151,20 +152,27 @@ static Err StartApplication
     volatile UInt16 formId;
     Err             err;
 
+
     OS_Init();
 
     err = RomVersionCompatible( appMinRomVersion );
     if ( err != errNone )
         return err;
 
+    err = CreateInternalMemoDB();
+    if ( err != errNone )
+        return err;
+
     GrayFntStart();
+    OpenTransliterations();
     InitializeLanguageSupport();
     InitializeCache();
 
     ReadPrefs();
 
     OpenKeyboardMap();
-    LoadUserFont( GetUserFontNumber( Prefs()->mainUserFontName ) );
+    LoadUserFont( GetUserFontNumber( Prefs()->mainUserFontName, true,
+                      fontCacheDoc ) );
 
     InitializeViewportBoundaries();
 
@@ -172,8 +180,8 @@ static Err StartApplication
     SetScreenMode();
     SetFontFunctions();
 
-    /* Restore the silkscreen to the last-used position */
-    SetSilkScreenStatus( Prefs()->previousSilkStatus );
+    InitializeResizeSupport( resizeIndex );
+    LoadResizePrefs( ( UInt32 )ViewerAppID, ( UInt16 )ViewerPrefDIA );
 
     if ( cmdPBP != NULL ) {
         Char    name[ dmDBNameLength ];
@@ -189,22 +197,13 @@ static Err StartApplication
             NULL, NULL, NULL, &type, &creator );
 
         if ( creator == ViewerAppID && type == ViewerDocumentType ) {
-            FormType* frm;
-
-            if ( Prefs()->syncPolicy == SYNC_MANUAL )
-                frm = FrmInitForm( frmOpenDocList );
-            else
-                frm = FrmInitForm( frmSyncDocList );
-
-            FrmSetActiveForm( frm );
-            FrmDrawForm( frm );
+            ShowSyncMessage();
 
             StrCopy( Prefs()->docName, name );
             Prefs()->lastForm = GetMainFormId();
             InitializeDocInfoList();
 
-            FrmEraseForm( frm );
-            FrmDeleteForm( frm );
+            HideMessage();
         }
     }
 
@@ -266,12 +265,6 @@ static void EnableNotification
                 SysNotifyRegister( cardNo, dbID, sysNotifyVolumeUnmountedEvent,
                                    NULL, sysNotifyNormalPriority, 0 );
             }
-            if ( SilkScreenType() == sonyV1SilkScreen ||
-                 SilkScreenType() == sonyV2SilkScreen ||
-                 SilkScreenType() == sonyV3SilkScreen ) {
-                SysNotifyRegister( cardNo, dbID, sysNotifyDisplayChangeEvent,
-                                   NULL, sysNotifyNormalPriority, 0 );
-            }
         } else {
 #ifdef HAVE_SONY_SDK
             if ( IsSony() ) {
@@ -287,12 +280,6 @@ static void EnableNotification
                                      sysNotifyVolumeUnmountedEvent,
                                      sysNotifyNormalPriority );
             }
-            if ( SilkScreenType() == sonyV1SilkScreen ||
-                 SilkScreenType() == sonyV2SilkScreen ||
-                 SilkScreenType() == sonyV3SilkScreen ) {
-                SysNotifyUnregister( cardNo, dbID, sysNotifyDisplayChangeEvent,
-                                     sysNotifyNormalPriority );
-            }
         }
     }
 }
@@ -306,6 +293,8 @@ static void HandleNotification
     SysNotifyParamType* cmdPBP   /* pointer to command parameter block */
     )
 {
+    if ( HandleResizeNotification( cmdPBP->notifyType ) )
+        return;
     switch ( cmdPBP->notifyType ) {
 #ifdef HAVE_SONY_SDK
         case sonySysNotifyHoldStatusChangeEvent:
@@ -325,10 +314,6 @@ static void HandleNotification
             cmdPBP->handled |= vfsHandledStartPrc;
             break;
 
-        case sysNotifyDisplayChangeEvent:
-            SilkVerifyChange();
-            break;
-
         default:
             break;
     }
@@ -362,6 +347,19 @@ static Boolean HandleVChrs
     oldMask = KeySetMask( commandKeyMask );
     if ( IsMainFormWinActive() || IsFullscreenformActive() ) {
         switch ( event->data.keyDown.chr ) {
+            /* Disable Japanese henkan-keys and keyboard. */
+            case vchrTsm1:
+            case vchrTsm2:
+            case vchrTsm3:
+            case vchrTsm4:
+            case vchrKeyboard:
+            case vchrKeyboardAlpha:
+            case vchrKeyboardNumeric:
+                if ( Prefs()->gestMode[ GESTURES_TAP ] != SELECT_NONE )
+                    DoSelectTypeAction( Prefs()->gestMode[ GESTURES_TAP ] );
+                handled = true;
+                break;
+
             case vchrHard1:
                 if ( Prefs()->hardKeys &&
                      Prefs()->hwMode[ DATEBOOK_BUTTON ] != SELECT_NONE ) {
@@ -399,13 +397,7 @@ static Boolean HandleVChrs
                 /* OS 3.3 seems to be sending events to itself for
                    internal purposes -- make sure it really is a
                    low battery situation */
-                UInt16 warnThreshold;
-                UInt16 currentVoltage;
-
-                currentVoltage = SysBatteryInfoV20( false, &warnThreshold,
-                                    NULL, NULL, NULL, NULL );
-
-                if ( currentVoltage < warnThreshold ) {
+                if ( ! HasEnoughPower() ) {
                     DoAutoscrollToggle( AUTOSCROLL_OFF );
                 }
                 break;
@@ -419,10 +411,56 @@ static Boolean HandleVChrs
                 DoAutoscrollToggle( AUTOSCROLL_OFF );
                 break;
 
+#ifdef RETURN_TO_PREV_APP
+            case vchrLaunch:
+                DoAutoscrollToggle( AUTOSCROLL_OFF );
+                SendAppStopEvent();
+                handled = true;
+                break;
+#endif
+
             default:
                 break;
         }
     }
+    else if ( FrmGetActiveFormID() == frmExternalLinks && Prefs()->hardKeys ) {
+        /* Convert HW button go back action into go back event for
+           external links form */
+        switch ( event->data.keyDown.chr ) {
+            case vchrHard1:
+                if ( Prefs()->hwMode[ DATEBOOK_BUTTON ] == SELECT_GO_BACK ) {
+                    handled = true;
+                }
+                break;
+
+            case vchrHard2:
+                if ( Prefs()->hwMode[ ADDRESS_BUTTON ] == SELECT_GO_BACK ) {
+                    handled = true;
+                }
+                break;
+
+            case vchrHard3:
+                if ( Prefs()->hwMode[ TODO_BUTTON ] == SELECT_GO_BACK ) {
+                    handled = true;
+                }
+                break;
+
+            case vchrHard4:
+                if ( Prefs()->hwMode[ MEMO_BUTTON ] == SELECT_GO_BACK ) {
+                    handled = true;
+                }
+                break;
+
+            default:
+                handled = false;
+                break;
+        }
+        if ( handled ) {
+            event->eType                    = ctlSelectEvent;
+            event->data.ctlEnter.controlID  = frmExternalLinksBack;
+            handled                         = false;
+        }
+    }
     oldMask = KeySetMask( oldMask );
     return handled;
 }
@@ -504,6 +542,7 @@ static void HandleFormLoad
 
         for ( list = ListOfEventHandlers; list->formID != 0; list++ ) {
             if ( list->formID == formID ) {
+                SetResizePolicy( formID );
                 FrmSetEventHandler( form, list->handler );
                 return;
             }
@@ -639,12 +678,6 @@ static void StopApplication( void )
 
     /* Save in the prefs what our silkscreen is set to, so its status is
        restored whenever plucker's loaded again */
-    Prefs()->previousSilkStatus = GetSilkScreenStatus();
-    /* But do not allow SILKSCREEN_NONE to be saved: it's too confusing
-       not to have the status bar showing. */
-    if ( Prefs()->previousSilkStatus == SILKSCREEN_NONE )
-        Prefs()->previousSilkStatus = SILKSCREEN_DOWN;
-        
 
     FrmSaveAllForms();
     FrmCloseAllForms();
@@ -657,17 +690,24 @@ static void StopApplication( void )
     ReleaseSearchQueue();
     DeleteUncompressBuffer();
     WritePrefs();
+    SaveResizePrefs( ( UInt32 )ViewerAppID, ( UInt16 )ViewerPrefDIA,
+        ( Int16 )ViewerVersion );
+    TerminateResizeSupport();
     CloseKeyboardMap();
     AnchorListRelease();
     ReleaseActionList();
     ReleaseKeyList();
+    CloseTransliterations();
     GrayFntStop();
 
+    DeleteInternalMemoDB();
+
     OS_Release();
 }
 
 
 
+#ifndef HAVE_PALMCUNIT
 /* This is the main entry point for the viewer application */
 UInt32 PilotMain
     (
@@ -699,7 +739,9 @@ UInt32 PilotMain
         UInt32 romVersion;
 
         FtrGet( sysFtrCreator, sysFtrNumROMVersion, &romVersion );
-        if ( 0x03000000 <= romVersion )
+        if ( 0x04000000 <= romVersion )
+            ExgRegisterDatatype( ViewerAppID, exgRegTypeID, ViewerMIMEType, NULL, 0);
+        else if ( 0x03000000 <= romVersion )
             ExgRegisterData( ViewerAppID, exgRegTypeID, ViewerMIMEType);
     }
     else if ( cmd == sysAppLaunchCmdExgAskUser ) {
@@ -721,4 +763,5 @@ UInt32 PilotMain
 
     return err;
 }
+#endif
 
